@@ -1,14 +1,14 @@
-from __future__ import print_function
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #
 
+from __future__ import print_function
 import os, sys, tempfile, shutil, tarfile
 import log, argdb
 try:
@@ -16,7 +16,7 @@ try:
 except ImportError:
   from urllib.request import urlretrieve
 try:
-  import urlparse as urlparse_local
+  import urlparse as urlparse_local # novermin
 except ImportError:
   from urllib import parse as urlparse_local
 if sys.version_info < (3,):
@@ -46,7 +46,8 @@ class Package:
     self.fortran         = False
     self.hasheaders      = False
     self.hasdloadflags   = False
-    self.builtafterslepc = False
+    self.requested       = False
+    self.havepackage     = False
 
   def RunCommand(self,instr):
     try:
@@ -70,9 +71,6 @@ class Package:
   def ProcessArgs(self,argdb,petscpackages=''):
     if hasattr(self,'petscdepend') and self.petscdepend in petscpackages:
       self.requested = True
-    else:
-      self.requested = False
-    self.havepackage = False
     if self.installable and not hasattr(self,'petscdepend'):
       string,found = argdb.PopPath('with-'+self.packagename+'-dir',exist=True)
       if found:
@@ -112,11 +110,7 @@ class Package:
       self.slflag = petsc.slflag
     if self.requested:
       name = self.packagename.upper()
-      if self.downloadpackage and self.builtafterslepc:
-        self.log.NewSection('Downloading '+name+'...')
-        self.Precondition(slepc,petsc)
-        self.DownloadOnly(slepcconf,slepcvars,slepcrules,slepc,petsc,archdir,slepc.prefixdir)
-      elif self.downloadpackage:
+      if self.downloadpackage:
         if hasattr(self,'version'):
           self.log.NewSection('Installing '+name+' version '+self.version+'...')
         else:
@@ -157,7 +151,7 @@ class Package:
 
   def MissingTarball(self,downloaddir):
     '''Check if tarball is missing in downloaddir'''
-    if self.downloadable and hasattr(self,'download') and self.download:
+    if self.downloadable and hasattr(self,'download') and self.download and hasattr(self,'archive'):
       localFile = os.path.join(downloaddir,self.archive)
       if not os.path.exists(localFile):
         url = self.packageurl
@@ -285,13 +279,17 @@ Downloaded package %s from: %s is not a tarball.
 
   def ShowInfo(self):
     if self.havepackage:
-      if hasattr(self,'petscdepend'):
-        self.log.Println(self.packagename.upper()+' from %s linked by PETSc' % self.petscdepend.upper())
-      elif self.builtafterslepc:
-        self.log.Println(self.packagename.upper()+' to be built after SLEPc')
+      if hasattr(self,'version') and self.downloadpackage:
+        packagename = self.packagename.upper()+' version '+self.version
       else:
-        self.log.Println(self.packagename.upper()+' library flags:')
+        packagename = self.packagename.upper()
+      if hasattr(self,'petscdepend'):
+        self.log.Println(packagename+' from %s linked by PETSc' % self.petscdepend.upper())
+      elif hasattr(self,'packageflags'):
+        self.log.Println(packagename+' library flags:')
         self.log.Println(' '+' '.join(self.packageflags))
+      else:
+        self.log.Println(packagename+' installed')
 
   def Link(self,functions,callbacks,flags,givencode='',cflags='',clanguage='c',logdump=True):
 
@@ -429,3 +427,9 @@ Downloaded package %s from: %s is not a tarball.
     self.havepackage = True
     self.packageflags = flags
 
+  def WriteMakefile(self,fname,builddir,cont):
+    self.log.write('Using makefile:\n')
+    self.log.write(cont)
+    with open(os.path.join(builddir,fname),'w') as mfile:
+      mfile.write(cont)
+
