@@ -1,7 +1,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -9,8 +9,8 @@
 !
 !  Include file for Fortran use of the NEP object in SLEPc
 !
-#if !defined(__SLEPCNEPDEF_H)
-#define __SLEPCNEPDEF_H
+#if !defined(SLEPCNEPDEF_H)
+#define SLEPCNEPDEF_H
 
 #include "slepc/finclude/slepcbv.h"
 #include "slepc/finclude/slepcds.h"
@@ -30,6 +30,7 @@
 #define NEPStop            PetscEnum
 #define NEPRefine          PetscEnum
 #define NEPRefineScheme    PetscEnum
+#define NEPCISSExtraction  PetscEnum
 
 #define NEPRII       'rii'
 #define NEPSLP       'slp'
