@@ -1,20 +1,30 @@
+/*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+
 #if !defined(SLEPCVERSION_H)
 #define SLEPCVERSION_H
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      15
-#define SLEPC_VERSION_SUBMINOR   0
+#define SLEPC_VERSION_MINOR      16
+#define SLEPC_VERSION_SUBMINOR   1
 #define SLEPC_VERSION_PATCH      0
-#define SLEPC_RELEASE_DATE       "March 31, 2021"
-#define SLEPC_VERSION_DATE       "Mar 31, 2021"
+#define SLEPC_RELEASE_DATE       "September 30, 2021"
+#define SLEPC_VERSION_DATE       "Nov 17, 2021"
 
 #if !defined (SLEPC_VERSION_GIT)
-#define SLEPC_VERSION_GIT        "v3.15"
+#define SLEPC_VERSION_GIT        "v3.16.1"
 #endif
 
 #if !defined(SLEPC_VERSION_DATE_GIT)
-#define SLEPC_VERSION_DATE_GIT   "2021-03-31 13:06:08 +0200"
+#define SLEPC_VERSION_DATE_GIT   "2021-11-17 15:21:47 +0100"
 #endif
 
 #define SLEPC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
