@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -44,11 +44,11 @@ PetscErrorCode EPSSetDimensions_LOBPCG(EPS eps,PetscInt nev,PetscInt *ncv,PetscI
 
   PetscFunctionBegin;
   k = PetscMax(3*ctx->bs,((eps->nev-1)/ctx->bs+3)*ctx->bs);
-  if (*ncv) { /* ncv set */
-    if (*ncv<k) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv is not sufficiently large");
+  if (*ncv!=PETSC_DEFAULT) { /* ncv set */
+    if (*ncv<k) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"The value of ncv is not sufficiently large");
   } else *ncv = k;
-  if (!*mpd) *mpd = 3*ctx->bs;
-  else if (*mpd!=3*ctx->bs) SETERRQ(PetscObjectComm((PetscObject)eps),1,"This solver does not allow a value of mpd different from 3*blocksize");
+  if (*mpd==PETSC_DEFAULT) *mpd = 3*ctx->bs;
+  else if (*mpd!=3*ctx->bs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"This solver does not allow a value of mpd different from 3*blocksize");
   PetscFunctionReturn(0);
 }
 
@@ -56,20 +56,17 @@ PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
-  PetscBool      istrivial;
 
   PetscFunctionBegin;
-  if (!eps->ishermitian || (eps->isgeneralized && !eps->ispositive)) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"LOBPCG only works for Hermitian problems");
+  EPSCheckHermitianDefinite(eps);
   if (!ctx->bs) ctx->bs = PetscMin(16,eps->nev);
   if (eps->n-eps->nds<5*ctx->bs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"The problem size is too small relative to the block size");
   ierr = EPSSetDimensions_LOBPCG(eps,eps->nev,&eps->ncv,&eps->mpd);CHKERRQ(ierr);
-  if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
+  if (eps->max_it==PETSC_DEFAULT) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
   if (!eps->which) eps->which = EPS_SMALLEST_REAL;
-  if (eps->which!=EPS_SMALLEST_REAL && eps->which!=EPS_LARGEST_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
-  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
-  if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
-  ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
-  if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
+  if (eps->which!=EPS_SMALLEST_REAL && eps->which!=EPS_LARGEST_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver supports only smallest real or largest real eigenvalues");
+  EPSCheckUnsupported(eps,EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_EXTRACTION);
+  EPSCheckIgnored(eps,EPS_FEATURE_BALANCE);
 
   if (!ctx->restart) ctx->restart = 0.9;
 
@@ -89,11 +86,11 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
-  PetscInt       i,j,k,ld,nv,ini,nmat,nc,nconv,locked,its;
+  PetscInt       i,j,k,ld,nv,ini,nmat,nc,nconv,locked,its,prev=0;
   PetscReal      norm;
   PetscScalar    *eigr,dot;
   PetscBool      breakdown,countc,flip=PETSC_FALSE,checkprecond=PETSC_FALSE;
-  Mat            A,B,M;
+  Mat            A,B,M,V=NULL,W=NULL;
   Vec            v,z,w=eps->work[0];
   BV             X,Y=NULL,Z,R,P,AX,BX;
   SlepcSC        sc;
@@ -151,7 +148,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
 
   /* 4. Compute initial Ritz vectors */
   ierr = BVMatMult(X,A,AX);CHKERRQ(ierr);
-  ierr = DSSetDimensions(eps->ds,nv,0,0,0);CHKERRQ(ierr);
+  ierr = DSSetDimensions(eps->ds,nv,0,0);CHKERRQ(ierr);
   ierr = DSGetMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
   ierr = BVMatProject(AX,NULL,X,M);CHKERRQ(ierr);
   if (flip) { ierr = MatScale(M,-1.0);CHKERRQ(ierr); }
@@ -276,7 +273,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
       /* compute initial Ritz vectors */
       nv = ctx->bs;
       ierr = BVMatMult(X,A,AX);CHKERRQ(ierr);
-      ierr = DSSetDimensions(eps->ds,nv,0,0,0);CHKERRQ(ierr);
+      ierr = DSSetDimensions(eps->ds,nv,0,0);CHKERRQ(ierr);
       ierr = DSGetMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
       ierr = BVMatProject(AX,NULL,X,M);CHKERRQ(ierr);
       if (flip) { ierr = MatScale(M,-1.0);CHKERRQ(ierr); }
@@ -307,30 +304,44 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     }
 
     /* 9. Apply preconditioner to the residuals */
-    for (j=ini;j<ctx->bs;j++) {
-      ierr = BVGetColumn(R,j,&v);CHKERRQ(ierr);
-      ierr = STApply(eps->st,v,w);CHKERRQ(ierr);
-      if (checkprecond) {
+    ierr = BVGetMat(R,&V);CHKERRQ(ierr);
+    if (prev != ctx->bs-ini) {
+      prev = ctx->bs-ini;
+      ierr = MatDestroy(&W);CHKERRQ(ierr);
+      ierr = MatDuplicate(V,MAT_SHARE_NONZERO_PATTERN,&W);CHKERRQ(ierr);
+    }
+    ierr = STApplyMat(eps->st,V,W);CHKERRQ(ierr);
+    if (checkprecond) {
+      for (j=ini;j<ctx->bs;j++) {
+        ierr = MatDenseGetColumnVecRead(V,j-ini,&v);CHKERRQ(ierr);
+        ierr = MatDenseGetColumnVecRead(W,j-ini,&w);CHKERRQ(ierr);
         ierr = VecDot(v,w,&dot);CHKERRQ(ierr);
+        ierr = MatDenseRestoreColumnVecRead(W,j-ini,&w);CHKERRQ(ierr);
+        ierr = MatDenseRestoreColumnVecRead(V,j-ini,&v);CHKERRQ(ierr);
         if (PetscRealPart(dot)<0.0) {
           ierr = PetscInfo(eps,"The preconditioner is not positive-definite\n");CHKERRQ(ierr);
           eps->reason = EPS_DIVERGED_BREAKDOWN;
           goto diverged;
         }
       }
-      if (nc+locked>0) {
+    }
+    if (nc+locked>0) {
+      for (j=ini;j<ctx->bs;j++) {
+        ierr = MatDenseGetColumnVecWrite(W,j-ini,&w);CHKERRQ(ierr);
         ierr = BVOrthogonalizeVec(Y,w,NULL,&norm,&breakdown);CHKERRQ(ierr);
         if (norm>0.0 && !breakdown) {
           ierr = VecScale(w,1.0/norm);CHKERRQ(ierr);
-        } else {
+        }
+        ierr = MatDenseRestoreColumnVecWrite(W,j-ini,&w);CHKERRQ(ierr);
+        if (norm<=0.0 || breakdown) {
           ierr = PetscInfo(eps,"Orthogonalization of preconditioned residual failed\n");CHKERRQ(ierr);
           eps->reason = EPS_DIVERGED_BREAKDOWN;
           goto diverged;
         }
       }
-      ierr = VecCopy(w,v);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(R,j,&v);CHKERRQ(ierr);
     }
+    ierr = MatCopy(W,V,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+    ierr = BVRestoreMat(R,&V);CHKERRQ(ierr);
 
     /* 11. B-orthonormalize preconditioned residuals */
     ierr = BVOrthogonalize(R,NULL);CHKERRQ(ierr);
@@ -355,7 +366,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     else nv = 2*ctx->bs-ini;
 
     ierr = BVSetActiveColumns(Z,0,nv);CHKERRQ(ierr);
-    ierr = DSSetDimensions(eps->ds,nv,0,0,0);CHKERRQ(ierr);
+    ierr = DSSetDimensions(eps->ds,nv,0,0);CHKERRQ(ierr);
     ierr = DSGetMat(eps->ds,DS_MAT_A,&M);CHKERRQ(ierr);
     ierr = BVMatProject(Z,A,Z,M);CHKERRQ(ierr);
     if (flip) { ierr = MatScale(M,-1.0);CHKERRQ(ierr); }
@@ -397,6 +408,10 @@ diverged:
 
   if (flip) sc->comparison = SlepcCompareLargestReal;
   ierr = PetscFree(eigr);CHKERRQ(ierr);
+  ierr = MatDestroy(&W);CHKERRQ(ierr);
+  if (V) { /* only needed when goto diverged is reached */
+    ierr = BVRestoreMat(R,&V);CHKERRQ(ierr);
+  }
   ierr = BVDestroy(&Z);CHKERRQ(ierr);
   ierr = BVDestroy(&X);CHKERRQ(ierr);
   ierr = BVDestroy(&R);CHKERRQ(ierr);
@@ -517,7 +532,7 @@ static PetscErrorCode EPSLOBPCGSetRestart_LOBPCG(EPS eps,PetscReal restart)
 .  -eps_lobpcg_restart - Sets the restart parameter
 
    Notes:
-   Allowed values are in the range [0.1,1.0]. The default is 0.6.
+   Allowed values are in the range [0.1,1.0]. The default is 0.9.
 
    Level: advanced
 
@@ -715,6 +730,7 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
 
   eps->ops->solve          = EPSSolve_LOBPCG;
   eps->ops->setup          = EPSSetUp_LOBPCG;
+  eps->ops->setupsort      = EPSSetUpSort_Default;
   eps->ops->setfromoptions = EPSSetFromOptions_LOBPCG;
   eps->ops->destroy        = EPSDestroy_LOBPCG;
   eps->ops->view           = EPSView_LOBPCG;
