@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -27,12 +27,12 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
 
   PetscFunctionBegin;
   eps->ncv = eps->n;
-  if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
+  if (eps->mpd!=PETSC_DEFAULT) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
+  if (eps->max_it==PETSC_DEFAULT) eps->max_it = 1;
   if (!eps->which) { ierr = EPSSetWhichEigenpairs_Default(eps);CHKERRQ(ierr); }
-  if (eps->which==EPS_ALL && eps->inta!=eps->intb) SETERRQ(PetscObjectComm((PetscObject)eps),1,"This solver does not support interval computation");
-  if (eps->balance!=EPS_BALANCE_NONE) { ierr = PetscInfo(eps,"Warning: balancing ignored\n");CHKERRQ(ierr); }
-  if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"User-defined stopping test not supported");
-  if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
+  if (eps->which==EPS_ALL && eps->inta!=eps->intb) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support interval computation");
+  EPSCheckUnsupported(eps,EPS_FEATURE_BALANCE | EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_STOPPING);
+  EPSCheckIgnored(eps,EPS_FEATURE_EXTRACTION | EPS_FEATURE_CONVERGENCE);
   ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
 
   /* attempt to get dense representations of A and B separately */
@@ -42,7 +42,7 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
     ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
     ierr = MatHasOperation(A,MATOP_CREATE_SUBMATRICES,&flg);CHKERRQ(ierr);
     if (flg) {
-      PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
+      PetscPushErrorHandler(PetscReturnErrorHandler,NULL);
       ierra  = MatCreateRedundantMatrix(A,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Ar);
       if (!ierra) { ierra |= MatConvert(Ar,MATSEQDENSE,MAT_INITIAL_MATRIX,&Adense); }
       ierra |= MatDestroy(&Ar);
@@ -52,7 +52,7 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
       ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
       ierr = MatHasOperation(B,MATOP_CREATE_SUBMATRICES,&flg);CHKERRQ(ierr);
       if (flg) {
-        PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
+        PetscPushErrorHandler(PetscReturnErrorHandler,NULL);
         ierrb  = MatCreateRedundantMatrix(B,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Br);
         if (!ierrb) { ierrb |= MatConvert(Br,MATSEQDENSE,MAT_INITIAL_MATRIX,&Bdense); }
         ierrb |= MatDestroy(&Br);
@@ -86,12 +86,16 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
   }
   ierr = DSAllocate(eps->ds,eps->ncv);CHKERRQ(ierr);
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
-  ierr = DSSetDimensions(eps->ds,eps->ncv,0,0,0);CHKERRQ(ierr);
+  ierr = DSSetDimensions(eps->ds,eps->ncv,0,0);CHKERRQ(ierr);
 
   if (denseok) {
     ierr = STGetShift(eps->st,&shift);CHKERRQ(ierr);
     if (shift != 0.0) {
-      ierr = MatShift(Adense,shift);CHKERRQ(ierr);
+      if (nmat>1) {
+        ierr = MatAXPY(Adense,-shift,Bdense,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+      } else {
+        ierr = MatShift(Adense,-shift);CHKERRQ(ierr);
+      }
     }
     /* use dummy pc and ksp to avoid problems when B is not positive definite */
     ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
@@ -183,11 +187,11 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
 {
   PetscFunctionBegin;
   eps->useds = PETSC_TRUE;
-  eps->hasts = PETSC_TRUE;
   eps->categ = EPS_CATEGORY_OTHER;
 
   eps->ops->solve          = EPSSolve_LAPACK;
   eps->ops->setup          = EPSSetUp_LAPACK;
+  eps->ops->setupsort      = EPSSetUpSort_Default;
   eps->ops->backtransform  = EPSBackTransform_Default;
   PetscFunctionReturn(0);
 }
