@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,6 +12,7 @@
 */
 
 #include <slepc/private/epsimpl.h>   /*I "slepceps.h" I*/
+#include <slepc/private/bvimpl.h>    /*I "slepcbv.h" I*/
 #include <petscdraw.h>
 
 PetscErrorCode EPSComputeVectors(EPS eps)
@@ -48,7 +49,7 @@ static PetscErrorCode EPSComputeValues(EPS eps)
         } else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, spectral transform should have a backtransform operation");
       } else {
         /* compute eigenvalues from Rayleigh quotient */
-        ierr = DSGetDimensions(eps->ds,&n,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+        ierr = DSGetDimensions(eps->ds,&n,NULL,NULL,NULL);CHKERRQ(ierr);
         if (!n) break;
         ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
         ierr = BVSetActiveColumns(eps->V,0,n);CHKERRQ(ierr);
@@ -99,7 +100,7 @@ static PetscErrorCode EPSComputeValues(EPS eps)
 /*@
    EPSSolve - Solves the eigensystem.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameter:
 .  eps - eigensolver context obtained from EPSCreate()
@@ -131,7 +132,7 @@ PetscErrorCode EPSSolve(EPS eps)
   if (eps->state>=EPS_STATE_SOLVED) PetscFunctionReturn(0);
   ierr = PetscLogEventBegin(EPS_Solve,eps,0,0,0);CHKERRQ(ierr);
 
-  /* call setup */
+  /* Call setup */
   ierr = EPSSetUp(eps);CHKERRQ(ierr);
   eps->nconv = 0;
   eps->its   = 0;
@@ -144,24 +145,27 @@ PetscErrorCode EPSSolve(EPS eps)
   ierr = EPSViewFromOptions(eps,NULL,"-eps_view_pre");CHKERRQ(ierr);
   ierr = RGViewFromOptions(eps->rg,NULL,"-rg_view");CHKERRQ(ierr);
 
-  /* call solver */
+  /* Call solver */
   ierr = (*eps->ops->solve)(eps);CHKERRQ(ierr);
+  if (!eps->reason) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
   eps->state = EPS_STATE_SOLVED;
 
+  /* Only the first nconv columns contain useful information (except in CISS) */
+  ierr = BVSetActiveColumns(eps->V,0,eps->nconv);CHKERRQ(ierr);
+  if (eps->twosided) { ierr = BVSetActiveColumns(eps->W,0,eps->nconv);CHKERRQ(ierr); }
+
+  /* If inplace, purify eigenvectors before reverting operator */
   ierr = STGetMatMode(eps->st,&matmode);CHKERRQ(ierr);
   if (matmode == ST_MATMODE_INPLACE && eps->ispositive) {
-    /* Purify eigenvectors before reverting operator */
     ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
   }
   ierr = STPostSolve(eps->st);CHKERRQ(ierr);
 
-  if (!eps->reason) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
-
   /* Map eigenvalues back to the original problem if appropriate */
   ierr = EPSComputeValues(eps);CHKERRQ(ierr);
 
 #if !defined(PETSC_USE_COMPLEX)
-  /* reorder conjugate eigenvalues (positive imaginary first) */
+  /* Reorder conjugate eigenvalues (positive imaginary first) */
   for (i=0;i<eps->nconv-1;i++) {
     if (eps->eigi[i] != 0) {
       if (eps->eigi[i] < 0) {
@@ -176,13 +180,13 @@ PetscErrorCode EPSSolve(EPS eps)
   }
 #endif
 
-  /* sort eigenvalues according to eps->which parameter */
+  /* Sort eigenvalues according to eps->which parameter */
   ierr = SlepcSortEigenvalues(eps->sc,eps->nconv,eps->eigr,eps->eigi,eps->perm);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(EPS_Solve,eps,0,0,0);CHKERRQ(ierr);
 
-  /* various viewers */
+  /* Various viewers */
   ierr = EPSViewFromOptions(eps,NULL,"-eps_view");CHKERRQ(ierr);
-  ierr = EPSReasonViewFromOptions(eps);CHKERRQ(ierr);
+  ierr = EPSConvergedReasonViewFromOptions(eps);CHKERRQ(ierr);
   ierr = EPSErrorViewFromOptions(eps);CHKERRQ(ierr);
   ierr = EPSValuesViewFromOptions(eps);CHKERRQ(ierr);
   ierr = EPSVectorsViewFromOptions(eps);CHKERRQ(ierr);
@@ -327,10 +331,12 @@ PetscErrorCode EPSGetConvergedReason(EPS eps,EPSConvergedReason *reason)
 
 .seealso: EPSGetEigenpair(), EPSGetConverged(), EPSSolve()
 @*/
-PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec *v)
+PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec v[])
 {
   PetscErrorCode ierr;
   PetscInt       i;
+  BV             V=eps->V;
+  Vec            w;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -338,12 +344,22 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec *v)
   PetscValidHeaderSpecific(*v,VEC_CLASSID,2);
   EPSCheckSolved(eps,1);
   if (!eps->ishermitian && eps->state==EPS_STATE_EIGENVECTORS) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSGetInvariantSubspace must be called before EPSGetEigenpair,EPSGetEigenvector or EPSComputeError");
-  for (i=0;i<eps->nconv;i++) {
-    ierr = BVCopyVec(eps->V,i,v[i]);CHKERRQ(ierr);
-    if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
-      ierr = VecPointwiseDivide(v[i],v[i],eps->D);CHKERRQ(ierr);
-      ierr = VecNormalize(v[i],NULL);CHKERRQ(ierr);
+  if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
+    ierr = BVDuplicateResize(eps->V,eps->nconv,&V);CHKERRQ(ierr);
+    ierr = BVSetActiveColumns(eps->V,0,eps->nconv);CHKERRQ(ierr);
+    ierr = BVCopy(eps->V,V);CHKERRQ(ierr);
+    for (i=0;i<eps->nconv;i++) {
+      ierr = BVGetColumn(V,i,&w);CHKERRQ(ierr);
+      ierr = VecPointwiseDivide(w,w,eps->D);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(V,i,&w);CHKERRQ(ierr);
     }
+    ierr = BVOrthogonalize(V,NULL);CHKERRQ(ierr);
+  }
+  for (i=0;i<eps->nconv;i++) {
+    ierr = BVCopyVec(V,i,v[i]);CHKERRQ(ierr);
+  }
+  if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
+    ierr = BVDestroy(&V);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -352,7 +368,7 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec *v)
    EPSGetEigenpair - Gets the i-th solution of the eigenproblem as computed by
    EPSSolve(). The solution consists in both the eigenvalue and the eigenvector.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - eigensolver context
@@ -384,7 +400,7 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec *v)
 
    Level: beginner
 
-.seealso: EPSGetEigenvalue(), EPSGetEigenvector(), EPSSolve(),
+.seealso: EPSGetEigenvalue(), EPSGetEigenvector(), EPSGetLeftEigenvector(), EPSSolve(),
           EPSGetConverged(), EPSSetWhichEigenpairs(), EPSGetInvariantSubspace()
 @*/
 PetscErrorCode EPSGetEigenpair(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar *eigi,Vec Vr,Vec Vi)
@@ -395,7 +411,8 @@ PetscErrorCode EPSGetEigenpair(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,i,2);
   EPSCheckSolved(eps,1);
-  if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  if (i<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  if (i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
   ierr = EPSGetEigenvalue(eps,i,eigr,eigi);CHKERRQ(ierr);
   if (Vr || Vi) { ierr = EPSGetEigenvector(eps,i,Vr,Vi);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
@@ -434,7 +451,8 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   EPSCheckSolved(eps,1);
-  if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  if (i<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  if (i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
   k = eps->perm[i];
 #if defined(PETSC_USE_COMPLEX)
   if (eigr) *eigr = eps->eigr[k];
@@ -446,10 +464,10 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-/*@C
+/*@
    EPSGetEigenvector - Gets the i-th right eigenvector as computed by EPSSolve().
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - eigensolver context
@@ -465,7 +483,8 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
 
    If the corresponding eigenvalue is real, then Vi is set to zero. If PETSc is
    configured with complex scalars the eigenvector is stored
-   directly in Vr (Vi is set to zero). In both cases, the user can pass NULL in Vi.
+   directly in Vr (Vi is set to zero). In any case, the user can pass NULL in Vr
+   or Vi if one of them is not required.
 
    The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established
@@ -477,7 +496,7 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
 
    Level: beginner
 
-.seealso: EPSSolve(), EPSGetConverged(), EPSSetWhichEigenpairs(), EPSGetEigenpair()
+.seealso: EPSSolve(), EPSGetConverged(), EPSSetWhichEigenpairs(), EPSGetEigenpair(), EPSGetLeftEigenvector()
 @*/
 PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
 {
@@ -487,33 +506,66 @@ PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,i,2);
-  PetscValidHeaderSpecific(Vr,VEC_CLASSID,3);
-  PetscCheckSameComm(eps,1,Vr,3);
+  if (Vr) { PetscValidHeaderSpecific(Vr,VEC_CLASSID,3); PetscCheckSameComm(eps,1,Vr,3); }
   if (Vi) { PetscValidHeaderSpecific(Vi,VEC_CLASSID,4); PetscCheckSameComm(eps,1,Vi,4); }
   EPSCheckSolved(eps,1);
+  if (i<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  if (i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
+  ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
+  k = eps->perm[i];
+  ierr = BV_GetEigenvector(eps->V,k,eps->eigi[k],Vr,Vi);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   EPSGetLeftEigenvector - Gets the i-th left eigenvector as computed by EPSSolve().
+
+   Logically Collective on eps
+
+   Input Parameters:
++  eps - eigensolver context
+-  i   - index of the solution
+
+   Output Parameters:
++  Wr   - real part of left eigenvector
+-  Wi   - imaginary part of left eigenvector
+
+   Notes:
+   The caller must provide valid Vec objects, i.e., they must be created
+   by the calling program with e.g. MatCreateVecs().
+
+   If the corresponding eigenvalue is real, then Wi is set to zero. If PETSc is
+   configured with complex scalars the eigenvector is stored directly in Wr
+   (Wi is set to zero). In any case, the user can pass NULL in Wr or Wi if
+   one of them is not required.
+
+   The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
+   Eigensolutions are indexed according to the ordering criterion established
+   with EPSSetWhichEigenpairs().
+
+   Left eigenvectors are available only if the twosided flag was set, see
+   EPSSetTwoSided().
+
+   Level: intermediate
+
+.seealso: EPSGetEigenvector(), EPSGetConverged(), EPSSetWhichEigenpairs(), EPSSetTwoSided()
+@*/
+PetscErrorCode EPSGetLeftEigenvector(EPS eps,PetscInt i,Vec Wr,Vec Wi)
+{
+  PetscErrorCode ierr;
+  PetscInt       k;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,i,2);
+  if (Wr) { PetscValidHeaderSpecific(Wr,VEC_CLASSID,3); PetscCheckSameComm(eps,1,Wr,3); }
+  if (Wi) { PetscValidHeaderSpecific(Wi,VEC_CLASSID,4); PetscCheckSameComm(eps,1,Wi,4); }
+  EPSCheckSolved(eps,1);
+  if (!eps->twosided) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"Must request left vectors with EPSSetTwoSided");
   if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
   ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
   k = eps->perm[i];
-#if defined(PETSC_USE_COMPLEX)
-  ierr = BVCopyVec(eps->V,k,Vr);CHKERRQ(ierr);
-  if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
-#else
-  if (eps->eigi[k] > 0) { /* first value of conjugate pair */
-    ierr = BVCopyVec(eps->V,k,Vr);CHKERRQ(ierr);
-    if (Vi) {
-      ierr = BVCopyVec(eps->V,k+1,Vi);CHKERRQ(ierr);
-    }
-  } else if (eps->eigi[k] < 0) { /* second value of conjugate pair */
-    ierr = BVCopyVec(eps->V,k-1,Vr);CHKERRQ(ierr);
-    if (Vi) {
-      ierr = BVCopyVec(eps->V,k,Vi);CHKERRQ(ierr);
-      ierr = VecScale(Vi,-1.0);CHKERRQ(ierr);
-    }
-  } else { /* real eigenvalue */
-    ierr = BVCopyVec(eps->V,k,Vr);CHKERRQ(ierr);
-    if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
-  }
-#endif
+  ierr = BV_GetEigenvector(eps->W,k,eps->eigi[k],Wr,Wi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -523,7 +575,7 @@ PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
 
    Not Collective
 
-   Input Parameter:
+   Input Parameters:
 +  eps - eigensolver context
 -  i   - index of eigenpair
 
@@ -543,7 +595,7 @@ PetscErrorCode EPSGetErrorEstimate(EPS eps,PetscInt i,PetscReal *errest)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(errest,3);
+  PetscValidRealPointer(errest,3);
   EPSCheckSolved(eps,1);
   if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
   *errest = eps->errest[eps->perm[i]];
@@ -555,20 +607,23 @@ PetscErrorCode EPSGetErrorEstimate(EPS eps,PetscInt i,PetscReal *errest)
    associated with an eigenpair.
 
    Input Parameters:
+     trans - whether A' must be used instead of A
      kr,ki - eigenvalue
      xr,xi - eigenvector
      z     - three work vectors (the second one not referenced in complex scalars)
 */
-PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscScalar kr,PetscScalar ki,Vec xr,Vec xi,Vec *z,PetscReal *norm)
+PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscBool trans,PetscScalar kr,PetscScalar ki,Vec xr,Vec xi,Vec *z,PetscReal *norm)
 {
   PetscErrorCode ierr;
   PetscInt       nmat;
   Mat            A,B;
   Vec            u,w;
+  PetscScalar    alpha;
 #if !defined(PETSC_USE_COMPLEX)
   Vec            v;
   PetscReal      ni,nr;
 #endif
+  PetscErrorCode (*matmult)(Mat,Vec,Vec) = trans? MatMultHermitianTranspose: MatMult;
 
   PetscFunctionBegin;
   u = z[0]; w = z[2];
@@ -580,29 +635,30 @@ PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscScalar kr,PetscScalar
   v = z[1];
   if (ki == 0 || PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
 #endif
-    ierr = MatMult(A,xr,u);CHKERRQ(ierr);                             /* u=A*x */
+    ierr = (*matmult)(A,xr,u);CHKERRQ(ierr);                          /* u=A*x */
     if (PetscAbsScalar(kr) > PETSC_MACHINE_EPSILON) {
-      if (nmat>1) { ierr = MatMult(B,xr,w);CHKERRQ(ierr); }
+      if (nmat>1) { ierr = (*matmult)(B,xr,w);CHKERRQ(ierr); }
       else { ierr = VecCopy(xr,w);CHKERRQ(ierr); }                    /* w=B*x */
-      ierr = VecAXPY(u,-kr,w);CHKERRQ(ierr);                          /* u=A*x-k*B*x */
+      alpha = trans? -PetscConj(kr): -kr;
+      ierr = VecAXPY(u,alpha,w);CHKERRQ(ierr);                        /* u=A*x-k*B*x */
     }
     ierr = VecNorm(u,NORM_2,norm);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
   } else {
-    ierr = MatMult(A,xr,u);CHKERRQ(ierr);                             /* u=A*xr */
+    ierr = (*matmult)(A,xr,u);CHKERRQ(ierr);                          /* u=A*xr */
     if (SlepcAbsEigenvalue(kr,ki) > PETSC_MACHINE_EPSILON) {
-      if (nmat>1) { ierr = MatMult(B,xr,v);CHKERRQ(ierr); }
+      if (nmat>1) { ierr = (*matmult)(B,xr,v);CHKERRQ(ierr); }
       else { ierr = VecCopy(xr,v);CHKERRQ(ierr); }                    /* v=B*xr */
       ierr = VecAXPY(u,-kr,v);CHKERRQ(ierr);                          /* u=A*xr-kr*B*xr */
-      if (nmat>1) { ierr = MatMult(B,xi,w);CHKERRQ(ierr); }
+      if (nmat>1) { ierr = (*matmult)(B,xi,w);CHKERRQ(ierr); }
       else { ierr = VecCopy(xi,w);CHKERRQ(ierr); }                    /* w=B*xi */
-      ierr = VecAXPY(u,ki,w);CHKERRQ(ierr);                           /* u=A*xr-kr*B*xr+ki*B*xi */
+      ierr = VecAXPY(u,trans?-ki:ki,w);CHKERRQ(ierr);                 /* u=A*xr-kr*B*xr+ki*B*xi */
     }
     ierr = VecNorm(u,NORM_2,&nr);CHKERRQ(ierr);
-    ierr = MatMult(A,xi,u);CHKERRQ(ierr);                             /* u=A*xi */
+    ierr = (*matmult)(A,xi,u);CHKERRQ(ierr);                          /* u=A*xi */
     if (SlepcAbsEigenvalue(kr,ki) > PETSC_MACHINE_EPSILON) {
       ierr = VecAXPY(u,-kr,w);CHKERRQ(ierr);                          /* u=A*xi-kr*B*xi */
-      ierr = VecAXPY(u,-ki,v);CHKERRQ(ierr);                          /* u=A*xi-kr*B*xi-ki*B*xr */
+      ierr = VecAXPY(u,trans?ki:-ki,v);CHKERRQ(ierr);                 /* u=A*xi-kr*B*xi-ki*B*xr */
     }
     ierr = VecNorm(u,NORM_2,&ni);CHKERRQ(ierr);
     *norm = SlepcAbsEigenvalue(nr,ni);
@@ -615,9 +671,9 @@ PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscScalar kr,PetscScalar
    EPSComputeError - Computes the error (based on the residual norm) associated
    with the i-th computed eigenpair.
 
-   Collective on EPS
+   Collective on eps
 
-   Input Parameter:
+   Input Parameters:
 +  eps  - the eigensolver context
 .  i    - the solution index
 -  type - the type of error to compute
@@ -638,7 +694,7 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
   PetscErrorCode ierr;
   Mat            A,B;
   Vec            xr,xi,w[3];
-  PetscReal      t,vecnorm=1.0;
+  PetscReal      t,vecnorm=1.0,errorl;
   PetscScalar    kr,ki;
   PetscBool      flg;
 
@@ -646,7 +702,7 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,i,2);
   PetscValidLogicalCollectiveEnum(eps,type,3);
-  PetscValidPointer(error,4);
+  PetscValidRealPointer(error,4);
   EPSCheckSolved(eps,1);
 
   /* allocate work vectors */
@@ -663,15 +719,22 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
   w[0] = eps->work[1];
   w[2] = eps->work[2];
 
-  /* compute residual norms */
+  /* compute residual norm */
   ierr = EPSGetEigenpair(eps,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
-  ierr = EPSComputeResidualNorm_Private(eps,kr,ki,xr,xi,w,error);CHKERRQ(ierr);
+  ierr = EPSComputeResidualNorm_Private(eps,PETSC_FALSE,kr,ki,xr,xi,w,error);CHKERRQ(ierr);
 
   /* compute 2-norm of eigenvector */
   if (eps->problem_type==EPS_GHEP) {
     ierr = VecNorm(xr,NORM_2,&vecnorm);CHKERRQ(ierr);
   }
 
+  /* if two-sided, compute left residual norm and take the maximum */
+  if (eps->twosided) {
+    ierr = EPSGetLeftEigenvector(eps,i,xr,xi);CHKERRQ(ierr);
+    ierr = EPSComputeResidualNorm_Private(eps,PETSC_TRUE,kr,ki,xr,xi,w,&errorl);CHKERRQ(ierr);
+    *error = PetscMax(*error,errorl);
+  }
+
   /* compute error */
   switch (type) {
     case EPS_ERROR_ABSOLUTE:
@@ -708,7 +771,7 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
    EPSGetStartVector - Generate a suitable vector to be used as the starting vector
    for the recurrence that builds the right subspace.
 
-   Collective on EPS and Vec
+   Collective on eps
 
    Input Parameters:
 +  eps - the eigensolver context
@@ -758,10 +821,40 @@ PetscErrorCode EPSGetStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
   ierr = BVOrthogonalizeColumn(eps->V,i,NULL,&norm,&lindep);CHKERRQ(ierr);
   if (breakdown) *breakdown = lindep;
   else if (lindep || norm == 0.0) {
-    if (i==0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Initial vector is zero or belongs to the deflation space");
-    else SETERRQ(PetscObjectComm((PetscObject)eps),1,"Unable to generate more start vectors");
+    if (i==0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Initial vector is zero or belongs to the deflation space");
+    else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more start vectors");
   }
   ierr = BVScaleColumn(eps->V,i,1.0/norm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+/*
+   EPSGetLeftStartVector - Generate a suitable vector to be used as the left starting
+   vector for the recurrence that builds the left subspace. See EPSGetStartVector().
+*/
+PetscErrorCode EPSGetLeftStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
+{
+  PetscErrorCode ierr;
+  PetscReal      norm;
+  PetscBool      lindep;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,i,2);
+
+  /* For the first step, use the first initial vector, otherwise a random one */
+  if (i>0 || eps->ninil==0) {
+    ierr = BVSetRandomColumn(eps->W,i);CHKERRQ(ierr);
+  }
+
+  /* Orthonormalize the vector with respect to previous vectors */
+  ierr = BVOrthogonalizeColumn(eps->W,i,NULL,&norm,&lindep);CHKERRQ(ierr);
+  if (breakdown) *breakdown = lindep;
+  else if (lindep || norm == 0.0) {
+    if (i==0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Left initial vector is zero");
+    else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more left start vectors");
+  }
+  ierr = BVScaleColumn(eps->W,i,1.0/norm);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
