@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -140,13 +140,13 @@ int main(int argc,char **argv)
 
    test:
       suffix: 4
-      args: -eps_nev 4 -eps_tol 1e-5 -eps_smallest_real -eps_type {{gd lobpcg rqcg}} -terse
+      args: -eps_nev 4 -eps_tol 1e-4 -eps_smallest_real -eps_type {{gd lobpcg rqcg}} -terse
       output_file: output/ex13_1.out
       filter: grep -v Solution
 
    test:
       suffix: 5_primme
       args: -n 10 -m 12 -eps_nev 4 -eps_target 0.9 -eps_max_it 15000 -eps_type primme -st_pc_type jacobi -terse
-      requires: primme define(SLEPC_HAVE_PRIMME3)
+      requires: primme defined(SLEPC_HAVE_PRIMME3) !single
 
 TEST*/
