@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -22,7 +22,6 @@
 
 static char help[] = "Nonlinear inverse iteration for A(x)*x=lambda*B(x)*x.\n\n";
 
-
 #include <slepceps.h>
 #include <petscdmplex.h>
 #include <petscds.h>
@@ -123,7 +122,7 @@ int main(int argc,char **argv)
   /* Set a preconditioning matrix to ST */
   if (use_shell_matrix) {
     ierr = EPSGetST(eps,&st);CHKERRQ(ierr);
-    ierr = STPrecondSetMatForPC(st,P);CHKERRQ(ierr);
+    ierr = STSetPreconditionerMat(st,P);CHKERRQ(ierr);
   }
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -152,7 +151,7 @@ int main(int argc,char **argv)
     /* Number of Newton iterations supposes to be zero */
     ierr = SNESGetIterationNumber(snes,&nits);CHKERRQ(ierr);
     if (nits) {
-      ierr = PetscPrintf(comm," Number of Newtoniterations %D should be zero \n",nits);CHKERRQ(ierr);
+      ierr = PetscPrintf(comm," Number of Newton iterations %D should be zero \n",nits);CHKERRQ(ierr);
     }
   }
 
@@ -282,7 +281,7 @@ PetscErrorCode CreateSquareMesh(MPI_Comm comm,DM *dm)
   ierr = DMPlexCreateBoxMesh(comm,dim,PETSC_FALSE,cells,NULL,NULL,NULL,PETSC_TRUE,dm);CHKERRQ(ierr);
   ierr = DMSetFromOptions(*dm);CHKERRQ(ierr);
   ierr = DMSetUp(*dm);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRMPI(ierr);
   if (size > 1) {
     ierr = DMPlexDistribute(*dm,0,NULL,&pdm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
@@ -354,13 +353,16 @@ PetscErrorCode FormJacobianA(SNES snes,Vec X,Mat A,Mat B,void *ctx)
   PetscErrorCode ierr;
   DM             dm;
   PetscDS        prob;
+  PetscWeakForm  wf;
   AppCtx         *userctx = (AppCtx *)ctx;
 
   PetscFunctionBegin;
   ierr = MatSetOption(B,MAT_KEEP_NONZERO_PATTERN,PETSC_TRUE);CHKERRQ(ierr);
   ierr = SNESGetDM(snes,&dm);CHKERRQ(ierr);
   ierr = DMGetDS(dm,&prob);CHKERRQ(ierr);
-  ierr = PetscDSSetJacobian(prob,0,0,NULL,NULL,NULL,g3_uu);CHKERRQ(ierr);
+  ierr = PetscDSGetWeakForm(prob, &wf);CHKERRQ(ierr);
+  ierr = PetscWeakFormClearIndex(wf, NULL, 0, 0, 0, PETSC_WF_G3, 0);CHKERRQ(ierr);
+  ierr = PetscWeakFormSetIndexJacobian(wf, NULL, 0, 0, 0, 0, 0, NULL, 0, NULL, 0, NULL, 0, g3_uu);CHKERRQ(ierr);
   ierr = FormJacobian(snes,X,A,B,ctx);CHKERRQ(ierr);
   ierr = MatZeroRowsIS(B,userctx->bdis,1.0,NULL,NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -371,13 +373,16 @@ PetscErrorCode FormJacobianB(SNES snes,Vec X,Mat A,Mat B,void *ctx)
   PetscErrorCode ierr;
   DM             dm;
   PetscDS        prob;
+  PetscWeakForm  wf;
   AppCtx         *userctx = (AppCtx *)ctx;
 
   PetscFunctionBegin;
   ierr = MatSetOption(B,MAT_KEEP_NONZERO_PATTERN,PETSC_TRUE);CHKERRQ(ierr);
   ierr = SNESGetDM(snes,&dm);CHKERRQ(ierr);
   ierr = DMGetDS(dm,&prob);CHKERRQ(ierr);
-  ierr = PetscDSSetJacobian(prob,0,0,g0_uu,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = PetscDSGetWeakForm(prob, &wf);CHKERRQ(ierr);
+  ierr = PetscWeakFormClearIndex(wf, NULL, 0, 0, 0, PETSC_WF_G3, 0);CHKERRQ(ierr);
+  ierr = PetscWeakFormSetIndexJacobian(wf, NULL, 0, 0, 0, 0, 0, g0_uu, 0, NULL, 0, NULL, 0, NULL);CHKERRQ(ierr);
   ierr = FormJacobian(snes,X,A,B,ctx);CHKERRQ(ierr);
   ierr = MatZeroRowsIS(B,userctx->bdis,0.0,NULL,NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -399,7 +404,6 @@ PetscErrorCode FormFunctionAB(SNES snes,Vec x,Vec Ax,Vec Bx,void *ctx)
   PetscFunctionReturn(0);
 }
 
-
 static PetscErrorCode FormFunction(SNES snes,Vec X,Vec F,void *ctx)
 {
   DM             dm;
@@ -430,6 +434,7 @@ PetscErrorCode FormFunctionA(SNES snes,Vec X,Vec F,void *ctx)
   PetscErrorCode ierr;
   DM             dm;
   PetscDS        prob;
+  PetscWeakForm  wf;
   PetscInt       nindices,iStart,iEnd,i;
   AppCtx         *userctx = (AppCtx *)ctx;
   PetscScalar    *array,value;
@@ -440,7 +445,9 @@ PetscErrorCode FormFunctionA(SNES snes,Vec X,Vec F,void *ctx)
   ierr = SNESGetDM(snes,&dm);CHKERRQ(ierr);
   ierr = DMGetDS(dm,&prob);CHKERRQ(ierr);
   /* hook functions */
-  ierr = PetscDSSetResidual(prob,0,NULL,f1_u);CHKERRQ(ierr);
+  ierr = PetscDSGetWeakForm(prob, &wf);CHKERRQ(ierr);
+  ierr = PetscWeakFormClearIndex(wf, NULL, 0, 0, 0, PETSC_WF_F0, 0);CHKERRQ(ierr);
+  ierr = PetscWeakFormSetIndexResidual(wf, NULL, 0, 0, 0, 0, NULL, 0, f1_u);CHKERRQ(ierr);
   ierr = FormFunction(snes,X,F,ctx);CHKERRQ(ierr);
   /* Boundary condition */
   ierr = VecLockGet(X,&vecstate);CHKERRQ(ierr);
@@ -481,6 +488,7 @@ PetscErrorCode FormFunctionB(SNES snes,Vec X,Vec F,void *ctx)
   PetscErrorCode ierr;
   DM             dm;
   PetscDS        prob;
+  PetscWeakForm  wf;
   PetscInt       nindices,iStart,iEnd,i;
   AppCtx         *userctx = (AppCtx *)ctx;
   PetscScalar    value;
@@ -490,7 +498,9 @@ PetscErrorCode FormFunctionB(SNES snes,Vec X,Vec F,void *ctx)
   ierr = SNESGetDM(snes,&dm);CHKERRQ(ierr);
   ierr = DMGetDS(dm,&prob);CHKERRQ(ierr);
   /* hook functions */
-  ierr = PetscDSSetResidual(prob,0,f0_u,NULL);CHKERRQ(ierr);
+  ierr = PetscDSGetWeakForm(prob, &wf);CHKERRQ(ierr);
+  ierr = PetscWeakFormClearIndex(wf, NULL, 0, 0, 0, PETSC_WF_F1, 0);CHKERRQ(ierr);
+  ierr = PetscWeakFormSetIndexResidual(wf, NULL, 0, 0, 0, 0, f0_u, 0, NULL);CHKERRQ(ierr);
   ierr = FormFunction(snes,X,F,ctx);CHKERRQ(ierr);
   /* Boundary condition */
   ierr = VecGetOwnershipRange(F,&iStart,&iEnd);CHKERRQ(ierr);
@@ -541,4 +551,9 @@ PetscErrorCode MatMult_B(Mat B,Vec x,Vec y)
          args: -use_shell_matrix -eps_power_update -init_eps_power_snes_mf_operator 1 -eps_power_snes_mf_operator 1 -form_function_ab {{0 1}} -test_init_sol 1
          filter: sed -e "s/ with monolithic update//"
 
+      test:
+         suffix: 6
+         args: -use_shell_matrix -eps_power_update -init_eps_power_snes_mf_operator 1 -eps_power_snes_mf_operator 1 -form_function_ab {{0 1}} -eps_monitor_all
+         output_file: output/ex34_6.out
+         filter: sed -e "s/\([+-].*i\)//g" -e "1,3s/[0-9]//g" -e "/[45] EPS/d"
 TEST*/
