@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -9,6 +9,12 @@
 */
 /*
    NEP routines related to the solution process
+
+   References:
+
+       [1] C. Campos and J.E. Roman, "NEP: a module for the parallel solution
+           of nonlinear eigenvalue problems in SLEPc", ACM Trans. Math. Soft.
+           (to appear), arXiv:1910.11712, 2021.
 */
 
 #include <slepc/private/nepimpl.h>       /*I "slepcnep.h" I*/
@@ -20,9 +26,12 @@ static const char citation[] =
   "@Article{slepc-nep,\n"
   "   author = \"C. Campos and J. E. Roman\",\n"
   "   title = \"{NEP}: a module for the parallel solution of nonlinear eigenvalue problems in {SLEPc}\",\n"
-  "   journal = \"arXiv:1910.11712\",\n"
-  "   url = \"https://arxiv.org/abs/1910.11712\",\n"
-  "   year = \"2019\"\n"
+  "   journal = \"{ACM} Trans. Math. Software\",\n"
+  "   volume = \"47\",\n"
+  "   number = \"3\",\n"
+  "   pages = \"23:1--23:29\",\n"
+  "   year = \"2021\",\n"
+  "   doi = \"10.1145/3447544\"\n"
   "}\n";
 
 PetscErrorCode NEPComputeVectors(NEP nep)
@@ -469,7 +478,8 @@ PetscErrorCode NEPGetEigenpair(NEP nep,PetscInt i,PetscScalar *eigr,PetscScalar
   if (Vr) { PetscValidHeaderSpecific(Vr,VEC_CLASSID,5); PetscCheckSameComm(nep,1,Vr,5); }
   if (Vi) { PetscValidHeaderSpecific(Vi,VEC_CLASSID,6); PetscCheckSameComm(nep,1,Vi,6); }
   NEPCheckSolved(nep,1);
-  if (i<0 || i>=nep->nconv) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  if (i<0) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  if (i>=nep->nconv) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see NEPGetConverged()");
 
   ierr = NEPComputeVectors(nep);CHKERRQ(ierr);
   k = nep->perm[i];
@@ -546,7 +556,7 @@ PetscErrorCode NEPGetLeftEigenvector(NEP nep,PetscInt i,Vec Wr,Vec Wi)
 
    Not Collective
 
-   Input Parameter:
+   Input Parameters:
 +  nep - nonlinear eigensolver context
 -  i   - index of eigenpair
 
@@ -555,11 +565,11 @@ PetscErrorCode NEPGetLeftEigenvector(NEP nep,PetscInt i,Vec Wr,Vec Wi)
 
    Notes:
    This is the error estimate used internally by the eigensolver. The actual
-   error bound can be computed with NEPComputeRelativeError().
+   error bound can be computed with NEPComputeError().
 
    Level: advanced
 
-.seealso: NEPComputeRelativeError()
+.seealso: NEPComputeError()
 @*/
 PetscErrorCode NEPGetErrorEstimate(NEP nep,PetscInt i,PetscReal *errest)
 {
@@ -605,7 +615,7 @@ PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscBool adj,PetscScalar
 
    Collective on nep
 
-   Input Parameter:
+   Input Parameters:
 +  nep  - the nonlinear eigensolver context
 .  i    - the solution index
 -  type - the type of error to compute
@@ -657,7 +667,7 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
   /* compute residual norms */
   ierr = NEPGetEigenpair(nep,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
-  if (ki) SETERRQ(PetscObjectComm((PetscObject)nep),1,"Not implemented for complex eigenvalues with real scalars");
+  if (ki) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Not implemented for complex eigenvalues with real scalars");
 #endif
   ierr = NEPComputeResidualNorm_Private(nep,PETSC_FALSE,kr,xr,nep->work,error);CHKERRQ(ierr);
   ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);
@@ -752,7 +762,7 @@ PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat A,Mat B)
       ierr = FNEvaluateFunction(nep->f[i],lambda,&alpha);CHKERRQ(ierr);
       ierr = MatAXPY(A,alpha,nep->A[i],nep->mstr);CHKERRQ(ierr);
     }
-    if (A != B) SETERRQ(PetscObjectComm((PetscObject)nep),1,"Not implemented");
+    if (A != B) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Not implemented");
     break;
   }
   PetscFunctionReturn(0);
