@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,6 +12,7 @@
 */
 
 #include <slepc/private/nepimpl.h>      /*I "slepcnep.h" I*/
+#include <slepc/private/bvimpl.h>       /*I "slepcbv.h" I*/
 #include <petscdraw.h>
 
 /*@C
@@ -85,7 +86,7 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
     }
     ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
-    ierr = SlepcSNPrintfScalar(str,50,nep->target,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = SlepcSNPrintfScalar(str,sizeof(str),nep->target,PETSC_FALSE);CHKERRQ(ierr);
     if (!nep->which) {
       ierr = PetscViewerASCIIPrintf(viewer,"not yet set\n");CHKERRQ(ierr);
     } else switch (nep->which) {
@@ -219,7 +220,7 @@ PetscErrorCode NEPViewFromOptions(NEP nep,PetscObject obj,const char name[])
 }
 
 /*@C
-   NEPReasonView - Displays the reason a NEP solve converged or diverged.
+   NEPConvergedReasonView - Displays the reason a NEP solve converged or diverged.
 
    Collective on nep
 
@@ -230,22 +231,31 @@ PetscErrorCode NEPViewFromOptions(NEP nep,PetscObject obj,const char name[])
    Options Database Keys:
 .  -nep_converged_reason - print reason for convergence, and number of iterations
 
+   Note:
+   To change the format of the output call PetscViewerPushFormat(viewer,format) before
+   this call. Use PETSC_VIEWER_DEFAULT for the default, use PETSC_VIEWER_FAILED to only
+   display a reason if it fails. The latter can be set in the command line with
+   -nep_converged_reason ::failed
+
    Level: intermediate
 
-.seealso: NEPSetConvergenceTest(), NEPSetTolerances(), NEPGetIterationNumber()
+.seealso: NEPSetConvergenceTest(), NEPSetTolerances(), NEPGetIterationNumber(), NEPConvergedReasonViewFromOptions(
 @*/
-PetscErrorCode NEPReasonView(NEP nep,PetscViewer viewer)
+PetscErrorCode NEPConvergedReasonView(NEP nep,PetscViewer viewer)
 {
-  PetscErrorCode ierr;
-  PetscBool      isAscii;
+  PetscErrorCode    ierr;
+  PetscBool         isAscii;
+  PetscViewerFormat format;
 
   PetscFunctionBegin;
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)nep));
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isAscii);CHKERRQ(ierr);
   if (isAscii) {
+    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)nep)->tablevel);CHKERRQ(ierr);
-    if (nep->reason > 0) {
+    if (nep->reason > 0 && format != PETSC_VIEWER_FAILED) {
       ierr = PetscViewerASCIIPrintf(viewer,"%s Nonlinear eigensolve converged (%D eigenpair%s) due to %s; iterations %D\n",((PetscObject)nep)->prefix?((PetscObject)nep)->prefix:"",nep->nconv,(nep->nconv>1)?"s":"",NEPConvergedReasons[nep->reason],nep->its);CHKERRQ(ierr);
-    } else {
+    } else if (nep->reason <= 0) {
       ierr = PetscViewerASCIIPrintf(viewer,"%s Nonlinear eigensolve did not converge due to %s; iterations %D\n",((PetscObject)nep)->prefix?((PetscObject)nep)->prefix:"",NEPConvergedReasons[nep->reason],nep->its);CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)nep)->tablevel);CHKERRQ(ierr);
@@ -254,7 +264,7 @@ PetscErrorCode NEPReasonView(NEP nep,PetscViewer viewer)
 }
 
 /*@
-   NEPReasonViewFromOptions - Processes command line options to determine if/how
+   NEPConvergedReasonViewFromOptions - Processes command line options to determine if/how
    the NEP converged reason is to be viewed.
 
    Collective on nep
@@ -263,8 +273,10 @@ PetscErrorCode NEPReasonView(NEP nep,PetscViewer viewer)
 .  nep - the nonlinear eigensolver context
 
    Level: developer
+
+.seealso: NEPConvergedReasonView()
 @*/
-PetscErrorCode NEPReasonViewFromOptions(NEP nep)
+PetscErrorCode NEPConvergedReasonViewFromOptions(NEP nep)
 {
   PetscErrorCode    ierr;
   PetscViewer       viewer;
@@ -278,7 +290,7 @@ PetscErrorCode NEPReasonViewFromOptions(NEP nep)
   ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,"-nep_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
-    ierr = NEPReasonView(nep,viewer);CHKERRQ(ierr);
+    ierr = NEPConvergedReasonView(nep,viewer);CHKERRQ(ierr);
     ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
@@ -332,20 +344,19 @@ static PetscErrorCode NEPErrorView_DETAIL(NEP nep,NEPErrorType etype,PetscViewer
   PetscReal      error,re,im;
   PetscScalar    kr,ki;
   PetscInt       i;
-#define EXLEN 30
-  char           ex[EXLEN],sep[]=" ---------------------- --------------------\n";
+  char           ex[30],sep[]=" ---------------------- --------------------\n";
 
   PetscFunctionBegin;
   if (!nep->nconv) PetscFunctionReturn(0);
   switch (etype) {
     case NEP_ERROR_ABSOLUTE:
-      ierr = PetscSNPrintf(ex,EXLEN,"    ||T(k)x||");CHKERRQ(ierr);
+      ierr = PetscSNPrintf(ex,sizeof(ex),"    ||T(k)x||");CHKERRQ(ierr);
       break;
     case NEP_ERROR_RELATIVE:
-      ierr = PetscSNPrintf(ex,EXLEN," ||T(k)x||/||kx||");CHKERRQ(ierr);
+      ierr = PetscSNPrintf(ex,sizeof(ex)," ||T(k)x||/||kx||");CHKERRQ(ierr);
       break;
     case NEP_ERROR_BACKWARD:
-      ierr = PetscSNPrintf(ex,EXLEN,"    eta(x,k)");CHKERRQ(ierr);
+      ierr = PetscSNPrintf(ex,sizeof(ex),"    eta(x,k)");CHKERRQ(ierr);
       break;
   }
   ierr = PetscViewerASCIIPrintf(viewer,"%s            k             %s\n%s",sep,ex,sep);CHKERRQ(ierr);
@@ -365,7 +376,7 @@ static PetscErrorCode NEPErrorView_DETAIL(NEP nep,NEPErrorType etype,PetscViewer
       ierr = PetscViewerASCIIPrintf(viewer,"    % 12f           %12g\n",(double)re,(double)error);CHKERRQ(ierr);
     }
   }
-  ierr = PetscViewerASCIIPrintf(viewer,sep);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"%s",sep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -424,8 +435,8 @@ PetscErrorCode NEPErrorView(NEP nep,NEPErrorType etype,PetscViewer viewer)
   if (!viewer) {
     ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)nep),&viewer);CHKERRQ(ierr);
   }
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
-  PetscCheckSameComm(nep,1,viewer,2);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,3);
+  PetscCheckSameComm(nep,1,viewer,3);
   NEPCheckSolved(nep,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) PetscFunctionReturn(0);
@@ -525,6 +536,78 @@ static PetscErrorCode NEPValuesView_DRAW(NEP nep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode NEPValuesView_BINARY(NEP nep,PetscViewer viewer)
+{
+#if defined(PETSC_HAVE_COMPLEX)
+  PetscInt       i,k;
+  PetscComplex   *ev;
+  PetscErrorCode ierr;
+#endif
+
+  PetscFunctionBegin;
+#if defined(PETSC_HAVE_COMPLEX)
+  ierr = PetscMalloc1(nep->nconv,&ev);CHKERRQ(ierr);
+  for (i=0;i<nep->nconv;i++) {
+    k = nep->perm[i];
+#if defined(PETSC_USE_COMPLEX)
+    ev[i] = nep->eigr[k];
+#else
+    ev[i] = PetscCMPLX(nep->eigr[k],nep->eigi[k]);
+#endif
+  }
+  ierr = PetscViewerBinaryWrite(viewer,ev,nep->nconv,PETSC_COMPLEX);CHKERRQ(ierr);
+  ierr = PetscFree(ev);CHKERRQ(ierr);
+#endif
+  PetscFunctionReturn(0);
+}
+
+#if defined(PETSC_HAVE_HDF5)
+static PetscErrorCode NEPValuesView_HDF5(NEP nep,PetscViewer viewer)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,k,n,N;
+  PetscMPIInt    rank;
+  Vec            v;
+  char           vname[30];
+  const char     *ename;
+
+  PetscFunctionBegin;
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)nep),&rank);CHKERRMPI(ierr);
+  N = nep->nconv;
+  n = rank? 0: N;
+  /* create a vector containing the eigenvalues */
+  ierr = VecCreateMPI(PetscObjectComm((PetscObject)nep),n,N,&v);CHKERRQ(ierr);
+  ierr = PetscObjectGetName((PetscObject)nep,&ename);CHKERRQ(ierr);
+  ierr = PetscSNPrintf(vname,sizeof(vname),"eigr_%s",ename);CHKERRQ(ierr);
+  ierr = PetscObjectSetName((PetscObject)v,vname);CHKERRQ(ierr);
+  if (!rank) {
+    for (i=0;i<nep->nconv;i++) {
+      k = nep->perm[i];
+      ierr = VecSetValue(v,i,nep->eigr[k],INSERT_VALUES);CHKERRQ(ierr);
+    }
+  }
+  ierr = VecAssemblyBegin(v);CHKERRQ(ierr);
+  ierr = VecAssemblyEnd(v);CHKERRQ(ierr);
+  ierr = VecView(v,viewer);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+  /* in real scalars write the imaginary part as a separate vector */
+  ierr = PetscSNPrintf(vname,sizeof(vname),"eigi_%s",ename);CHKERRQ(ierr);
+  ierr = PetscObjectSetName((PetscObject)v,vname);CHKERRQ(ierr);
+  if (!rank) {
+    for (i=0;i<nep->nconv;i++) {
+      k = nep->perm[i];
+      ierr = VecSetValue(v,i,nep->eigi[k],INSERT_VALUES);CHKERRQ(ierr);
+    }
+  }
+  ierr = VecAssemblyBegin(v);CHKERRQ(ierr);
+  ierr = VecAssemblyEnd(v);CHKERRQ(ierr);
+  ierr = VecView(v,viewer);CHKERRQ(ierr);
+#endif
+  ierr = VecDestroy(&v);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+#endif
+
 static PetscErrorCode NEPValuesView_ASCII(NEP nep,PetscViewer viewer)
 {
   PetscInt       i,k;
@@ -589,9 +672,12 @@ static PetscErrorCode NEPValuesView_MATLAB(NEP nep,PetscViewer viewer)
 @*/
 PetscErrorCode NEPValuesView(NEP nep,PetscViewer viewer)
 {
-  PetscBool         isascii,isdraw;
+  PetscBool         isascii,isdraw,isbinary;
   PetscViewerFormat format;
   PetscErrorCode    ierr;
+#if defined(PETSC_HAVE_HDF5)
+  PetscBool         ishdf5;
+#endif
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -602,9 +688,19 @@ PetscErrorCode NEPValuesView(NEP nep,PetscViewer viewer)
   PetscCheckSameComm(nep,1,viewer,2);
   NEPCheckSolved(nep,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_HDF5)
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERHDF5,&ishdf5);CHKERRQ(ierr);
+#endif
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isdraw) {
     ierr = NEPValuesView_DRAW(nep,viewer);CHKERRQ(ierr);
+  } else if (isbinary) {
+    ierr = NEPValuesView_BINARY(nep,viewer);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_HDF5)
+  } else if (ishdf5) {
+    ierr = NEPValuesView_HDF5(nep,viewer);CHKERRQ(ierr);
+#endif
   } else if (isascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     switch (format) {
@@ -685,10 +781,7 @@ PetscErrorCode NEPVectorsView(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscInt       i,k;
-  Vec            x;
-#define NMLEN 30
-  char           vname[NMLEN];
-  const char     *ename;
+  Vec            xr,xi=NULL;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -699,23 +792,24 @@ PetscErrorCode NEPVectorsView(NEP nep,PetscViewer viewer)
   PetscCheckSameComm(nep,1,viewer,2);
   NEPCheckSolved(nep,1);
   if (nep->nconv) {
-    ierr = PetscObjectGetName((PetscObject)nep,&ename);CHKERRQ(ierr);
     ierr = NEPComputeVectors(nep);CHKERRQ(ierr);
+    ierr = BVCreateVec(nep->V,&xr);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+    ierr = BVCreateVec(nep->V,&xi);CHKERRQ(ierr);
+#endif
     for (i=0;i<nep->nconv;i++) {
       k = nep->perm[i];
-      ierr = PetscSNPrintf(vname,NMLEN,"X%d_%s",(int)i,ename);CHKERRQ(ierr);
-      ierr = BVGetColumn(nep->V,k,&x);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject)x,vname);CHKERRQ(ierr);
-      ierr = VecView(x,viewer);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(nep->V,k,&x);CHKERRQ(ierr);
+      ierr = BV_GetEigenvector(nep->V,k,nep->eigi[k],xr,xi);CHKERRQ(ierr);
+      ierr = SlepcViewEigenvector(viewer,xr,xi,"X",i,(PetscObject)nep);CHKERRQ(ierr);
       if (nep->twosided) {
-        ierr = PetscSNPrintf(vname,NMLEN,"Y%d_%s",(int)i,ename);CHKERRQ(ierr);
-        ierr = BVGetColumn(nep->W,k,&x);CHKERRQ(ierr);
-        ierr = PetscObjectSetName((PetscObject)x,vname);CHKERRQ(ierr);
-        ierr = VecView(x,viewer);CHKERRQ(ierr);
-        ierr = BVRestoreColumn(nep->W,k,&x);CHKERRQ(ierr);
+        ierr = BV_GetEigenvector(nep->W,k,nep->eigi[k],xr,xi);CHKERRQ(ierr);
+        ierr = SlepcViewEigenvector(viewer,xr,xi,"Y",i,(PetscObject)nep);CHKERRQ(ierr);
       }
     }
+    ierr = VecDestroy(&xr);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+    ierr = VecDestroy(&xi);CHKERRQ(ierr);
+#endif
   }
   PetscFunctionReturn(0);
 }
