@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -20,6 +20,8 @@ const char *PEPExtractTypes[] = {"","NONE","NORM","RESIDUAL","STRUCTURED","PEPEx
 const char *PEPErrorTypes[] = {"ABSOLUTE","RELATIVE","BACKWARD","PEPErrorType","PEP_ERROR_",0};
 const char *const PEPConvergedReasons_Shifted[] = {"","DIVERGED_SYMMETRY_LOST","DIVERGED_BREAKDOWN","DIVERGED_ITS","CONVERGED_ITERATING","CONVERGED_TOL","CONVERGED_USER","PEPConvergedReason","PEP_",0};
 const char *const*PEPConvergedReasons = PEPConvergedReasons_Shifted + 4;
+const char *PEPJDProjectionTypes[] = {"HARMONIC","ORTHOGONAL","PEPJDProjection","PEP_JD_PROJECTION_",0};
+const char *PEPCISSExtractions[] = {"RITZ","HANKEL","CAA","PEPCISSExtraction","PEP_CISS_EXTRACTION_",0};
 
 /*@C
    PEPFinalizePackage - This function destroys everything in the Slepc interface
@@ -35,8 +37,12 @@ PetscErrorCode PEPFinalizePackage(void)
 
   PetscFunctionBegin;
   ierr = PetscFunctionListDestroy(&PEPList);CHKERRQ(ierr);
-  PEPPackageInitialized = PETSC_FALSE;
-  PEPRegisterAllCalled  = PETSC_FALSE;
+  ierr = PetscFunctionListDestroy(&PEPMonitorList);CHKERRQ(ierr);
+  ierr = PetscFunctionListDestroy(&PEPMonitorCreateList);CHKERRQ(ierr);
+  ierr = PetscFunctionListDestroy(&PEPMonitorDestroyList);CHKERRQ(ierr);
+  PEPPackageInitialized       = PETSC_FALSE;
+  PEPRegisterAllCalled        = PETSC_FALSE;
+  PEPMonitorRegisterAllCalled = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
@@ -52,8 +58,8 @@ PetscErrorCode PEPFinalizePackage(void)
 PetscErrorCode PEPInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
+  PetscClassId   classids[1];
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -63,26 +69,23 @@ PetscErrorCode PEPInitializePackage(void)
   ierr = PetscClassIdRegister("PEP Solver",&PEP_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = PEPRegisterAll();CHKERRQ(ierr);
+  /* Register Monitors */
+  ierr = PEPMonitorRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("PEPSetUp",PEP_CLASSID,&PEP_SetUp);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("PEPSolve",PEP_CLASSID,&PEP_Solve);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("PEPRefine",PEP_CLASSID,&PEP_Refine);CHKERRQ(ierr);
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrstr(logList,"pep",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(PEP_CLASSID);CHKERRQ(ierr);
-    }
-  }
+  ierr = PetscLogEventRegister("PEPCISS_SVD",PEP_CLASSID,&PEP_CISS_SVD);CHKERRQ(ierr);
+  /* Process Info */
+  classids[0] = PEP_CLASSID;
+  ierr = PetscInfoProcessClass("pep",1,&classids[0]);CHKERRQ(ierr);
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"pep",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(PEP_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("pep",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscLogEventDeactivateClass(PEP_CLASSID);CHKERRQ(ierr); }
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(PEPFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -95,7 +98,7 @@ PetscErrorCode PEPInitializePackage(void)
   This one registers all the PEP methods that are in the basic SLEPc libslepcpep
   library.
  */
-PETSC_EXTERN PetscErrorCode PetscDLLibraryRegister_slepcpep()
+SLEPC_EXTERN PetscErrorCode PetscDLLibraryRegister_slepcpep()
 {
   PetscErrorCode ierr;
 
