@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,10 +13,19 @@
 
 #include <slepc/private/pepimpl.h>      /*I "slepcpep.h" I*/
 
+/* Logging support */
+PetscClassId      PEP_CLASSID = 0;
+PetscLogEvent     PEP_SetUp = 0,PEP_Solve = 0,PEP_Refine = 0,PEP_CISS_SVD = 0;
+
+/* List of registered PEP routines */
 PetscFunctionList PEPList = 0;
 PetscBool         PEPRegisterAllCalled = PETSC_FALSE;
-PetscClassId      PEP_CLASSID = 0;
-PetscLogEvent     PEP_SetUp = 0,PEP_Solve = 0,PEP_Refine = 0;
+
+/* List of registered PEP monitors */
+PetscFunctionList PEPMonitorList              = NULL;
+PetscFunctionList PEPMonitorCreateList        = NULL;
+PetscFunctionList PEPMonitorDestroyList       = NULL;
+PetscBool         PEPMonitorRegisterAllCalled = PETSC_FALSE;
 
 /*@
    PEPCreate - Creates the default PEP context.
@@ -47,10 +56,10 @@ PetscErrorCode PEPCreate(MPI_Comm comm,PEP *outpep)
   ierr = PEPInitializePackage();CHKERRQ(ierr);
   ierr = SlepcHeaderCreate(pep,PEP_CLASSID,"PEP","Polynomial Eigenvalue Problem","PEP",comm,PEPDestroy,PEPView);CHKERRQ(ierr);
 
-  pep->max_it          = 0;
+  pep->max_it          = PETSC_DEFAULT;
   pep->nev             = 1;
-  pep->ncv             = 0;
-  pep->mpd             = 0;
+  pep->ncv             = PETSC_DEFAULT;
+  pep->mpd             = PETSC_DEFAULT;
   pep->nini            = 0;
   pep->target          = 0.0;
   pep->tol             = PETSC_DEFAULT;
@@ -231,6 +240,50 @@ PetscErrorCode PEPRegister(const char *name,PetscErrorCode (*function)(PEP))
   PetscFunctionReturn(0);
 }
 
+/*@C
+   PEPMonitorRegister - Adds PEP monitor routine.
+
+   Not Collective
+
+   Input Parameters:
++  name    - name of a new monitor routine
+.  vtype   - a PetscViewerType for the output
+.  format  - a PetscViewerFormat for the output
+.  monitor - monitor routine
+.  create  - creation routine, or NULL
+-  destroy - destruction routine, or NULL
+
+   Notes:
+   PEPMonitorRegister() may be called multiple times to add several user-defined monitors.
+
+   Sample usage:
+.vb
+   PEPMonitorRegister("my_monitor",PETSCVIEWERASCII,PETSC_VIEWER_ASCII_INFO_DETAIL,MyMonitor,NULL,NULL);
+.ve
+
+   Then, your monitor can be chosen with the procedural interface via
+$      PEPMonitorSetFromOptions(pep,"-pep_monitor_my_monitor","my_monitor",NULL)
+   or at runtime via the option
+$      -pep_monitor_my_monitor
+
+   Level: advanced
+
+.seealso: PEPMonitorRegisterAll()
+@*/
+PetscErrorCode PEPMonitorRegister(const char name[],PetscViewerType vtype,PetscViewerFormat format,PetscErrorCode (*monitor)(PEP,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscErrorCode (*create)(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**),PetscErrorCode (*destroy)(PetscViewerAndFormat**))
+{
+  char           key[PETSC_MAX_PATH_LEN];
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PEPInitializePackage();CHKERRQ(ierr);
+  ierr = SlepcMonitorMakeKey_Internal(name,vtype,format,key);CHKERRQ(ierr);
+  ierr = PetscFunctionListAdd(&PEPMonitorList,key,monitor);CHKERRQ(ierr);
+  if (create)  { ierr = PetscFunctionListAdd(&PEPMonitorCreateList,key,create);CHKERRQ(ierr); }
+  if (destroy) { ierr = PetscFunctionListAdd(&PEPMonitorDestroyList,key,destroy);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
 /*@
    PEPReset - Resets the PEP context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
@@ -269,7 +322,7 @@ PetscErrorCode PEPReset(PEP pep)
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    PEPDestroy - Destroys the PEP context.
 
    Collective on pep
@@ -398,8 +451,10 @@ PetscErrorCode PEPSetRG(PEP pep,RG rg)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidHeaderSpecific(rg,RG_CLASSID,2);
-  PetscCheckSameComm(pep,1,rg,2);
+  if (rg) {
+    PetscValidHeaderSpecific(rg,RG_CLASSID,2);
+    PetscCheckSameComm(pep,1,rg,2);
+  }
   ierr = PetscObjectReference((PetscObject)rg);CHKERRQ(ierr);
   ierr = RGDestroy(&pep->rg);CHKERRQ(ierr);
   pep->rg = rg;
@@ -607,7 +662,7 @@ PetscErrorCode PEPRefineGetKSP(PEP pep,KSP *ksp)
     ierr = PetscObjectSetOptions((PetscObject)pep->refineksp,((PetscObject)pep)->options);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(*ksp,((PetscObject)pep)->prefix);CHKERRQ(ierr);
     ierr = KSPAppendOptionsPrefix(*ksp,"pep_refine_");CHKERRQ(ierr);
-    ierr = KSPSetTolerances(pep->refineksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
+    ierr = KSPSetTolerances(pep->refineksp,SlepcDefaultTol(pep->rtol),PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
   }
   *ksp = pep->refineksp;
   PetscFunctionReturn(0);
@@ -711,7 +766,7 @@ PetscErrorCode PEPSetInterval(PEP pep,PetscReal inta,PetscReal intb)
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidLogicalCollectiveReal(pep,inta,2);
   PetscValidLogicalCollectiveReal(pep,intb,3);
-  if (inta>intb) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
+  if (inta>=intb) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
   if (pep->inta != inta || pep->intb != intb) {
     pep->inta = inta;
     pep->intb = intb;
