@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -127,7 +127,7 @@ PetscErrorCode PEPSetUp(PEP pep)
         ierr = PetscObjectTypeCompareAny((PetscObject)pc,&flg,PCLU,PCCHOLESKY,"");CHKERRQ(ierr);
       }
       if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"The MBE scheme for refinement requires a direct solver in KSP");
-      ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pc),&size);CHKERRQ(ierr);
+      ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pc),&size);CHKERRMPI(ierr);
       if (size>1) {   /* currently selected PC is a factorization */
         ierr = PCFactorGetMatSolverType(pc,&stype);CHKERRQ(ierr);
         ierr = PetscStrcmp(stype,MATSOLVERPETSC,&flg);CHKERRQ(ierr);
@@ -219,7 +219,7 @@ PetscErrorCode PEPSetUp(PEP pep)
   /* compute matrix coefficients */
   ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
   if (!flg) {
-    if (pep->solvematcoeffs) { ierr = STMatSetUp(pep->st,1.0,pep->solvematcoeffs);CHKERRQ(ierr); }
+    if (pep->which!=PEP_ALL && pep->solvematcoeffs) { ierr = STMatSetUp(pep->st,1.0,pep->solvematcoeffs);CHKERRQ(ierr); }
   } else {
     if (pep->basis!=PEP_BASIS_MONOMIAL) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Cannot use ST-transform with non-monomial basis in PEP");
   }
@@ -243,7 +243,7 @@ PetscErrorCode PEPSetUp(PEP pep)
   /* process initial vectors */
   if (pep->nini<0) {
     k = -pep->nini;
-    if (k>pep->ncv) SETERRQ(PetscObjectComm((PetscObject)pep),1,"The number of initial vectors is larger than ncv");
+    if (k>pep->ncv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_USER_INPUT,"The number of initial vectors is larger than ncv");
     ierr = BVInsertVecs(pep->V,0,&k,pep->IS,PETSC_TRUE);CHKERRQ(ierr);
     ierr = SlepcBasisDestroy_Private(&pep->nini,&pep->IS);CHKERRQ(ierr);
     pep->nini = k;
@@ -277,7 +277,7 @@ PetscErrorCode PEPSetUp(PEP pep)
 PetscErrorCode PEPSetOperators(PEP pep,PetscInt nmat,Mat A[])
 {
   PetscErrorCode ierr;
-  PetscInt       i,n=0,m,m0=0;
+  PetscInt       i,n=0,m,m0=0,mloc,nloc,mloc0=0;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
@@ -290,13 +290,16 @@ PetscErrorCode PEPSetOperators(PEP pep,PetscInt nmat,Mat A[])
     PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
     PetscCheckSameComm(pep,1,A[i],3);
     ierr = MatGetSize(A[i],&m,&n);CHKERRQ(ierr);
-    if (m!=n) SETERRQ1(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"A[%D] is a non-square matrix",i);
-    if (!i) m0 = m;
-    if (m!=m0) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_INCOMP,"Dimensions of matrices do not match with each other");
+    ierr = MatGetLocalSize(A[i],&mloc,&nloc);CHKERRQ(ierr);
+    if (m!=n) SETERRQ3(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"A[%D] is a non-square matrix (%D rows, %D cols)",i,m,n);
+    if (mloc!=nloc) SETERRQ3(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"A[%D] does not have equal row and column local sizes (%D, %D)",i,mloc,nloc);
+    if (!i) { m0 = m; mloc0 = mloc; }
+    if (m!=m0) SETERRQ3(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_INCOMP,"Dimensions of A[%D] do not match with previous matrices (%D, %D)",i,m,m0);
+    if (mloc!=mloc0) SETERRQ3(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_INCOMP,"Local dimensions of A[%D] do not match with previous matrices (%D, %D)",i,mloc,mloc0);
     ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
   }
 
-  if (pep->state && n!=pep->n) { ierr = PEPReset(pep);CHKERRQ(ierr); }
+  if (pep->state && (n!=pep->n || nloc!=pep->nloc)) { ierr = PEPReset(pep);CHKERRQ(ierr); }
   else if (pep->nmat) {
     ierr = MatDestroyMatrices(pep->nmat,&pep->A);CHKERRQ(ierr);
     ierr = PetscFree2(pep->pbc,pep->nrma);CHKERRQ(ierr);
@@ -371,7 +374,7 @@ PetscErrorCode PEPGetNumMatrices(PEP pep,PetscInt *nmat)
 
    Collective on pep
 
-   Input Parameter:
+   Input Parameters:
 +  pep   - the polynomial eigensolver context
 .  n     - number of vectors
 -  is    - set of basis vectors of the initial space
@@ -421,13 +424,13 @@ PetscErrorCode PEPSetDimensions_Default(PEP pep,PetscInt nev,PetscInt *ncv,Petsc
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompareAny((PetscObject)pep,&krylov,PEPTOAR,PEPSTOAR,PEPQARNOLDI,"");CHKERRQ(ierr);
   dim = (pep->nmat-1)*pep->n;
-  if (*ncv) { /* ncv set */
+  if (*ncv!=PETSC_DEFAULT) { /* ncv set */
     if (krylov) {
-      if (*ncv<nev+1 && !(*ncv==nev && *ncv==dim)) SETERRQ(PetscObjectComm((PetscObject)pep),1,"The value of ncv must be at least nev+1");
+      if (*ncv<nev+1 && !(*ncv==nev && *ncv==dim)) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_USER_INPUT,"The value of ncv must be at least nev+1");
     } else {
-      if (*ncv<nev) SETERRQ(PetscObjectComm((PetscObject)pep),1,"The value of ncv must be at least nev");
+      if (*ncv<nev) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_USER_INPUT,"The value of ncv must be at least nev");
     }
-  } else if (*mpd) { /* mpd set */
+  } else if (*mpd!=PETSC_DEFAULT) { /* mpd set */
     *ncv = PetscMin(dim,nev+(*mpd));
   } else { /* neither set: defaults depend on nev being small or large */
     if (nev<500) *ncv = PetscMin(dim,PetscMax(2*nev,nev+15));
@@ -436,7 +439,7 @@ PetscErrorCode PEPSetDimensions_Default(PEP pep,PetscInt nev,PetscInt *ncv,Petsc
       *ncv = PetscMin(dim,nev+(*mpd));
     }
   }
-  if (!*mpd) *mpd = *ncv;
+  if (*mpd==PETSC_DEFAULT) *mpd = *ncv;
   PetscFunctionReturn(0);
 }
 
