@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -9,6 +9,12 @@
 */
 /*
    PEP routines related to the solution process
+
+   References:
+
+       [1] C. Campos and J.E. Roman, "Parallel iterative refinement in
+           polynomial eigenvalue problems", Numer. Linear Algebra Appl.
+           23(4):730-745, 2016.
 */
 
 #include <slepc/private/pepimpl.h>       /*I "slepcpep.h" I*/
@@ -309,7 +315,8 @@ PetscErrorCode PEPGetEigenpair(PEP pep,PetscInt i,PetscScalar *eigr,PetscScalar
   if (Vr) { PetscValidHeaderSpecific(Vr,VEC_CLASSID,5); PetscCheckSameComm(pep,1,Vr,5); }
   if (Vi) { PetscValidHeaderSpecific(Vi,VEC_CLASSID,6); PetscCheckSameComm(pep,1,Vi,6); }
   PEPCheckSolved(pep,1);
-  if (i<0 || i>=pep->nconv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  if (i<0) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  if (i>=pep->nconv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see PEPGetConverged()");
 
   ierr = PEPComputeVectors(pep);CHKERRQ(ierr);
   k = pep->perm[i];
@@ -334,7 +341,7 @@ PetscErrorCode PEPGetEigenpair(PEP pep,PetscInt i,PetscScalar *eigr,PetscScalar
 
    Not Collective
 
-   Input Parameter:
+   Input Parameters:
 +  pep - polynomial eigensolver context
 -  i   - index of eigenpair
 
@@ -419,7 +426,7 @@ PetscErrorCode PEPComputeResidualNorm_Private(PEP pep,PetscScalar kr,PetscScalar
           ierr = MatMult(A[i],xr,w);CHKERRQ(ierr);
         }
       }
-      if (ivals[i]!=0){
+      if (ivals[i]!=0) {
         ierr = VecAXPY(u,-ivals[i],wi);CHKERRQ(ierr);
         ierr = VecAXPY(ui,ivals[i],w);CHKERRQ(ierr);
       }
@@ -451,7 +458,7 @@ PetscErrorCode PEPComputeResidualNorm_Private(PEP pep,PetscScalar kr,PetscScalar
 
    Collective on pep
 
-   Input Parameter:
+   Input Parameters:
 +  pep  - the polynomial eigensolver context
 .  i    - the solution index
 -  type - the type of error to compute
