@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,6 +12,7 @@
 */
 
 #include <slepc/private/pepimpl.h>      /*I "slepcpep.h" I*/
+#include <slepc/private/bvimpl.h>       /*I "slepcbv.h" I*/
 #include <petscdraw.h>
 
 /*@C
@@ -56,11 +57,6 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(pep,1,viewer,2);
 
-#if defined(PETSC_USE_COMPLEX)
-#define HERM "hermitian"
-#else
-#define HERM "symmetric"
-#endif
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)pep,viewer);CHKERRQ(ierr);
@@ -72,7 +68,7 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
     if (pep->problem_type) {
       switch (pep->problem_type) {
         case PEP_GENERAL:    type = "general polynomial eigenvalue problem"; break;
-        case PEP_HERMITIAN:  type = HERM " polynomial eigenvalue problem"; break;
+        case PEP_HERMITIAN:  type = SLEPC_STRING_HERMITIAN " polynomial eigenvalue problem"; break;
         case PEP_HYPERBOLIC: type = "hyperbolic polynomial eigenvalue problem"; break;
         case PEP_GYROSCOPIC: type = "gyroscopic polynomial eigenvalue problem"; break;
       }
@@ -94,7 +90,7 @@ PetscErrorCode PEPView(PEP pep,PetscViewer viewer)
     }
     ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
-    ierr = SlepcSNPrintfScalar(str,50,pep->target,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = SlepcSNPrintfScalar(str,sizeof(str),pep->target,PETSC_FALSE);CHKERRQ(ierr);
     if (!pep->which) {
       ierr = PetscViewerASCIIPrintf(viewer,"not yet set\n");CHKERRQ(ierr);
     } else switch (pep->which) {
@@ -233,7 +229,7 @@ PetscErrorCode PEPViewFromOptions(PEP pep,PetscObject obj,const char name[])
 }
 
 /*@C
-   PEPReasonView - Displays the reason a PEP solve converged or diverged.
+   PEPConvergedReasonView - Displays the reason a PEP solve converged or diverged.
 
    Collective on pep
 
@@ -244,22 +240,31 @@ PetscErrorCode PEPViewFromOptions(PEP pep,PetscObject obj,const char name[])
    Options Database Keys:
 .  -pep_converged_reason - print reason for convergence, and number of iterations
 
+   Note:
+   To change the format of the output call PetscViewerPushFormat(viewer,format) before
+   this call. Use PETSC_VIEWER_DEFAULT for the default, use PETSC_VIEWER_FAILED to only
+   display a reason if it fails. The latter can be set in the command line with
+   -pep_converged_reason ::failed
+
    Level: intermediate
 
-.seealso: PEPSetConvergenceTest(), PEPSetTolerances(), PEPGetIterationNumber()
+.seealso: PEPSetConvergenceTest(), PEPSetTolerances(), PEPGetIterationNumber(), PEPConvergedReasonViewFromOptions()
 @*/
-PetscErrorCode PEPReasonView(PEP pep,PetscViewer viewer)
+PetscErrorCode PEPConvergedReasonView(PEP pep,PetscViewer viewer)
 {
-  PetscErrorCode ierr;
-  PetscBool      isAscii;
+  PetscErrorCode    ierr;
+  PetscBool         isAscii;
+  PetscViewerFormat format;
 
   PetscFunctionBegin;
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)pep));
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isAscii);CHKERRQ(ierr);
   if (isAscii) {
+    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)pep)->tablevel);CHKERRQ(ierr);
-    if (pep->reason > 0) {
+    if (pep->reason > 0 && format != PETSC_VIEWER_FAILED) {
       ierr = PetscViewerASCIIPrintf(viewer,"%s Polynomial eigensolve converged (%D eigenpair%s) due to %s; iterations %D\n",((PetscObject)pep)->prefix?((PetscObject)pep)->prefix:"",pep->nconv,(pep->nconv>1)?"s":"",PEPConvergedReasons[pep->reason],pep->its);CHKERRQ(ierr);
-    } else {
+    } else if (pep->reason <= 0) {
       ierr = PetscViewerASCIIPrintf(viewer,"%s Polynomial eigensolve did not converge due to %s; iterations %D\n",((PetscObject)pep)->prefix?((PetscObject)pep)->prefix:"",PEPConvergedReasons[pep->reason],pep->its);CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)pep)->tablevel);CHKERRQ(ierr);
@@ -268,7 +273,7 @@ PetscErrorCode PEPReasonView(PEP pep,PetscViewer viewer)
 }
 
 /*@
-   PEPReasonViewFromOptions - Processes command line options to determine if/how
+   PEPConvergedReasonViewFromOptions - Processes command line options to determine if/how
    the PEP converged reason is to be viewed.
 
    Collective on pep
@@ -277,8 +282,10 @@ PetscErrorCode PEPReasonView(PEP pep,PetscViewer viewer)
 .  pep - the eigensolver context
 
    Level: developer
+
+.seealso: PEPConvergedReasonView()
 @*/
-PetscErrorCode PEPReasonViewFromOptions(PEP pep)
+PetscErrorCode PEPConvergedReasonViewFromOptions(PEP pep)
 {
   PetscErrorCode    ierr;
   PetscViewer       viewer;
@@ -292,7 +299,7 @@ PetscErrorCode PEPReasonViewFromOptions(PEP pep)
   ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,"-pep_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
-    ierr = PEPReasonView(pep,viewer);CHKERRQ(ierr);
+    ierr = PEPConvergedReasonView(pep,viewer);CHKERRQ(ierr);
     ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
@@ -303,28 +310,37 @@ PetscErrorCode PEPReasonViewFromOptions(PEP pep)
 static PetscErrorCode PEPErrorView_ASCII(PEP pep,PEPErrorType etype,PetscViewer viewer)
 {
   PetscReal      error;
-  PetscInt       i,j,k;
+  PetscInt       i,j,k,nvals;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (pep->nconv<pep->nev) {
+  nvals = (pep->which==PEP_ALL)? pep->nconv: pep->nev;
+  if (pep->which!=PEP_ALL && pep->nconv<pep->nev) {
     ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D eigenvalues converged\n\n",pep->nev);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
-  for (i=0;i<pep->nev;i++) {
+  if (pep->which==PEP_ALL && !nvals) {
+    ierr = PetscViewerASCIIPrintf(viewer," No eigenvalues have been found\n\n");CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
+  for (i=0;i<nvals;i++) {
     ierr = PEPComputeError(pep,i,etype,&error);CHKERRQ(ierr);
     if (error>=5.0*pep->tol) {
-      ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",pep->nev);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",nvals);CHKERRQ(ierr);
       PetscFunctionReturn(0);
     }
   }
-  ierr = PetscViewerASCIIPrintf(viewer," All requested eigenvalues computed up to the required tolerance:");CHKERRQ(ierr);
-  for (i=0;i<=(pep->nev-1)/8;i++) {
+  if (pep->which==PEP_ALL) {
+    ierr = PetscViewerASCIIPrintf(viewer," Found %D eigenvalues, all of them computed up to the required tolerance:",nvals);CHKERRQ(ierr);
+  } else {
+    ierr = PetscViewerASCIIPrintf(viewer," All requested eigenvalues computed up to the required tolerance:");CHKERRQ(ierr);
+  }
+  for (i=0;i<=(nvals-1)/8;i++) {
     ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
-    for (j=0;j<PetscMin(8,pep->nev-8*i);j++) {
+    for (j=0;j<PetscMin(8,nvals-8*i);j++) {
       k = pep->perm[8*i+j];
       ierr = SlepcPrintEigenvalueASCII(viewer,pep->eigr[k],pep->eigi[k]);CHKERRQ(ierr);
-      if (8*i+j+1<pep->nev) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
+      if (8*i+j+1<nvals) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
     }
   }
   ierr = PetscViewerASCIIPrintf(viewer,"\n\n");CHKERRQ(ierr);
@@ -337,20 +353,19 @@ static PetscErrorCode PEPErrorView_DETAIL(PEP pep,PEPErrorType etype,PetscViewer
   PetscReal      error,re,im;
   PetscScalar    kr,ki;
   PetscInt       i;
-#define EXLEN 30
-  char           ex[EXLEN],sep[]=" ---------------------- --------------------\n";
+  char           ex[30],sep[]=" ---------------------- --------------------\n";
 
   PetscFunctionBegin;
   if (!pep->nconv) PetscFunctionReturn(0);
   switch (etype) {
     case PEP_ERROR_ABSOLUTE:
-      ierr = PetscSNPrintf(ex,EXLEN,"   ||P(k)x||");CHKERRQ(ierr);
+      ierr = PetscSNPrintf(ex,sizeof(ex),"   ||P(k)x||");CHKERRQ(ierr);
       break;
     case PEP_ERROR_RELATIVE:
-      ierr = PetscSNPrintf(ex,EXLEN,"||P(k)x||/||kx||");CHKERRQ(ierr);
+      ierr = PetscSNPrintf(ex,sizeof(ex),"||P(k)x||/||kx||");CHKERRQ(ierr);
       break;
     case PEP_ERROR_BACKWARD:
-      ierr = PetscSNPrintf(ex,EXLEN,"    eta(x,k)");CHKERRQ(ierr);
+      ierr = PetscSNPrintf(ex,sizeof(ex),"    eta(x,k)");CHKERRQ(ierr);
       break;
   }
   ierr = PetscViewerASCIIPrintf(viewer,"%s            k             %s\n%s",sep,ex,sep);CHKERRQ(ierr);
@@ -370,7 +385,7 @@ static PetscErrorCode PEPErrorView_DETAIL(PEP pep,PEPErrorType etype,PetscViewer
       ierr = PetscViewerASCIIPrintf(viewer,"    % 12f           %12g\n",(double)re,(double)error);CHKERRQ(ierr);
     }
   }
-  ierr = PetscViewerASCIIPrintf(viewer,sep);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"%s",sep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -429,8 +444,8 @@ PetscErrorCode PEPErrorView(PEP pep,PEPErrorType etype,PetscViewer viewer)
   if (!viewer) {
     ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)pep),&viewer);CHKERRQ(ierr);
   }
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
-  PetscCheckSameComm(pep,1,viewer,2);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,3);
+  PetscCheckSameComm(pep,1,viewer,3);
   PEPCheckSolved(pep,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) PetscFunctionReturn(0);
@@ -530,6 +545,78 @@ static PetscErrorCode PEPValuesView_DRAW(PEP pep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode PEPValuesView_BINARY(PEP pep,PetscViewer viewer)
+{
+#if defined(PETSC_HAVE_COMPLEX)
+  PetscInt       i,k;
+  PetscComplex   *ev;
+  PetscErrorCode ierr;
+#endif
+
+  PetscFunctionBegin;
+#if defined(PETSC_HAVE_COMPLEX)
+  ierr = PetscMalloc1(pep->nconv,&ev);CHKERRQ(ierr);
+  for (i=0;i<pep->nconv;i++) {
+    k = pep->perm[i];
+#if defined(PETSC_USE_COMPLEX)
+    ev[i] = pep->eigr[k];
+#else
+    ev[i] = PetscCMPLX(pep->eigr[k],pep->eigi[k]);
+#endif
+  }
+  ierr = PetscViewerBinaryWrite(viewer,ev,pep->nconv,PETSC_COMPLEX);CHKERRQ(ierr);
+  ierr = PetscFree(ev);CHKERRQ(ierr);
+#endif
+  PetscFunctionReturn(0);
+}
+
+#if defined(PETSC_HAVE_HDF5)
+static PetscErrorCode PEPValuesView_HDF5(PEP pep,PetscViewer viewer)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,k,n,N;
+  PetscMPIInt    rank;
+  Vec            v;
+  char           vname[30];
+  const char     *ename;
+
+  PetscFunctionBegin;
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)pep),&rank);CHKERRMPI(ierr);
+  N = pep->nconv;
+  n = rank? 0: N;
+  /* create a vector containing the eigenvalues */
+  ierr = VecCreateMPI(PetscObjectComm((PetscObject)pep),n,N,&v);CHKERRQ(ierr);
+  ierr = PetscObjectGetName((PetscObject)pep,&ename);CHKERRQ(ierr);
+  ierr = PetscSNPrintf(vname,sizeof(vname),"eigr_%s",ename);CHKERRQ(ierr);
+  ierr = PetscObjectSetName((PetscObject)v,vname);CHKERRQ(ierr);
+  if (!rank) {
+    for (i=0;i<pep->nconv;i++) {
+      k = pep->perm[i];
+      ierr = VecSetValue(v,i,pep->eigr[k],INSERT_VALUES);CHKERRQ(ierr);
+    }
+  }
+  ierr = VecAssemblyBegin(v);CHKERRQ(ierr);
+  ierr = VecAssemblyEnd(v);CHKERRQ(ierr);
+  ierr = VecView(v,viewer);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+  /* in real scalars write the imaginary part as a separate vector */
+  ierr = PetscSNPrintf(vname,sizeof(vname),"eigi_%s",ename);CHKERRQ(ierr);
+  ierr = PetscObjectSetName((PetscObject)v,vname);CHKERRQ(ierr);
+  if (!rank) {
+    for (i=0;i<pep->nconv;i++) {
+      k = pep->perm[i];
+      ierr = VecSetValue(v,i,pep->eigi[k],INSERT_VALUES);CHKERRQ(ierr);
+    }
+  }
+  ierr = VecAssemblyBegin(v);CHKERRQ(ierr);
+  ierr = VecAssemblyEnd(v);CHKERRQ(ierr);
+  ierr = VecView(v,viewer);CHKERRQ(ierr);
+#endif
+  ierr = VecDestroy(&v);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+#endif
+
 static PetscErrorCode PEPValuesView_ASCII(PEP pep,PetscViewer viewer)
 {
   PetscInt       i,k;
@@ -594,9 +681,12 @@ static PetscErrorCode PEPValuesView_MATLAB(PEP pep,PetscViewer viewer)
 @*/
 PetscErrorCode PEPValuesView(PEP pep,PetscViewer viewer)
 {
-  PetscBool         isascii,isdraw;
+  PetscBool         isascii,isdraw,isbinary;
   PetscViewerFormat format;
   PetscErrorCode    ierr;
+#if defined(PETSC_HAVE_HDF5)
+  PetscBool         ishdf5;
+#endif
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
@@ -607,9 +697,19 @@ PetscErrorCode PEPValuesView(PEP pep,PetscViewer viewer)
   PetscCheckSameComm(pep,1,viewer,2);
   PEPCheckSolved(pep,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_HDF5)
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERHDF5,&ishdf5);CHKERRQ(ierr);
+#endif
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isdraw) {
     ierr = PEPValuesView_DRAW(pep,viewer);CHKERRQ(ierr);
+  } else if (isbinary) {
+    ierr = PEPValuesView_BINARY(pep,viewer);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_HDF5)
+  } else if (ishdf5) {
+    ierr = PEPValuesView_HDF5(pep,viewer);CHKERRQ(ierr);
+#endif
   } else if (isascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     switch (format) {
@@ -686,10 +786,7 @@ PetscErrorCode PEPVectorsView(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscInt       i,k;
-  Vec            x;
-#define NMLEN 30
-  char           vname[NMLEN];
-  const char     *ename;
+  Vec            xr,xi=NULL;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
@@ -700,16 +797,20 @@ PetscErrorCode PEPVectorsView(PEP pep,PetscViewer viewer)
   PetscCheckSameComm(pep,1,viewer,2);
   PEPCheckSolved(pep,1);
   if (pep->nconv) {
-    ierr = PetscObjectGetName((PetscObject)pep,&ename);CHKERRQ(ierr);
     ierr = PEPComputeVectors(pep);CHKERRQ(ierr);
+    ierr = BVCreateVec(pep->V,&xr);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+    ierr = BVCreateVec(pep->V,&xi);CHKERRQ(ierr);
+#endif
     for (i=0;i<pep->nconv;i++) {
       k = pep->perm[i];
-      ierr = PetscSNPrintf(vname,NMLEN,"V%d_%s",(int)i,ename);CHKERRQ(ierr);
-      ierr = BVGetColumn(pep->V,k,&x);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject)x,vname);CHKERRQ(ierr);
-      ierr = VecView(x,viewer);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(pep->V,k,&x);CHKERRQ(ierr);
+      ierr = BV_GetEigenvector(pep->V,k,pep->eigi[k],xr,xi);CHKERRQ(ierr);
+      ierr = SlepcViewEigenvector(viewer,xr,xi,"X",i,(PetscObject)pep);CHKERRQ(ierr);
     }
+    ierr = VecDestroy(&xr);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+    ierr = VecDestroy(&xi);CHKERRQ(ierr);
+#endif
   }
   PetscFunctionReturn(0);
 }
