@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,9 +12,10 @@
 
 static PetscBool SVDPackageInitialized = PETSC_FALSE;
 
+const char *SVDTRLanczosGBidiags[] = {"SINGLE","UPPER","LOWER","SVDTRLanczosGBidiag","SVD_TRLANCZOS_GBIDIAG_",0};
 const char *SVDErrorTypes[] = {"ABSOLUTE","RELATIVE","SVDErrorType","SVD_ERROR_",0};
-const char *SVDPRIMMEMethods[] = {"HYBRID","NORMALEQUATIONS","AUGMENTED","SVDPRIMMEMethod","SVD_PRIMME_",0};
-const char *const SVDConvergedReasons_Shifted[] = {"","","DIVERGED_BREAKDOWN","DIVERGED_ITS","CONVERGED_ITERATING","CONVERGED_TOL","CONVERGED_USER","SVDConvergedReason","SVD_",0};
+const char *SVDPRIMMEMethods[] = {"","HYBRID","NORMALEQUATIONS","AUGMENTED","SVDPRIMMEMethod","SVD_PRIMME_",0};
+const char *const SVDConvergedReasons_Shifted[] = {"","","DIVERGED_BREAKDOWN","DIVERGED_ITS","CONVERGED_ITERATING","CONVERGED_TOL","CONVERGED_USER","CONVERGED_MAXIT","SVDConvergedReason","SVD_",0};
 const char *const*SVDConvergedReasons = SVDConvergedReasons_Shifted + 4;
 
 /*@C
@@ -31,8 +32,12 @@ PetscErrorCode SVDFinalizePackage(void)
 
   PetscFunctionBegin;
   ierr = PetscFunctionListDestroy(&SVDList);CHKERRQ(ierr);
-  SVDPackageInitialized = PETSC_FALSE;
-  SVDRegisterAllCalled  = PETSC_FALSE;
+  ierr = PetscFunctionListDestroy(&SVDMonitorList);CHKERRQ(ierr);
+  ierr = PetscFunctionListDestroy(&SVDMonitorCreateList);CHKERRQ(ierr);
+  ierr = PetscFunctionListDestroy(&SVDMonitorDestroyList);CHKERRQ(ierr);
+  SVDPackageInitialized       = PETSC_FALSE;
+  SVDRegisterAllCalled        = PETSC_FALSE;
+  SVDMonitorRegisterAllCalled = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
@@ -48,8 +53,8 @@ PetscErrorCode SVDFinalizePackage(void)
 PetscErrorCode SVDInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
+  PetscClassId   classids[1];
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -59,25 +64,21 @@ PetscErrorCode SVDInitializePackage(void)
   ierr = PetscClassIdRegister("SVD Solver",&SVD_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = SVDRegisterAll();CHKERRQ(ierr);
+  /* Register Monitors */
+  ierr = SVDMonitorRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("SVDSetUp",SVD_CLASSID,&SVD_SetUp);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("SVDSolve",SVD_CLASSID,&SVD_Solve);CHKERRQ(ierr);
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrstr(logList,"svd",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(SVD_CLASSID);CHKERRQ(ierr);
-    }
-  }
+  /* Process Info */
+  classids[0] = SVD_CLASSID;
+  ierr = PetscInfoProcessClass("svd",1,&classids[0]);CHKERRQ(ierr);
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"svd",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(SVD_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("svd",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscLogEventDeactivateClass(SVD_CLASSID);CHKERRQ(ierr); }
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(SVDFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -90,7 +91,7 @@ PetscErrorCode SVDInitializePackage(void)
   This one registers all the SVD methods that are in the basic SLEPc libslepcsvd
   library.
  */
-PETSC_EXTERN PetscErrorCode PetscDLLibraryRegister_slepcsvd()
+SLEPC_EXTERN PetscErrorCode PetscDLLibraryRegister_slepcsvd()
 {
   PetscErrorCode ierr;
 
