@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,11 +13,20 @@
 
 #include <slepc/private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
-PetscFunctionList SVDList = 0;
-PetscBool         SVDRegisterAllCalled = PETSC_FALSE;
+/* Logging support */
 PetscClassId      SVD_CLASSID = 0;
 PetscLogEvent     SVD_SetUp = 0,SVD_Solve = 0;
 
+/* List of registered SVD routines */
+PetscFunctionList SVDList = 0;
+PetscBool         SVDRegisterAllCalled = PETSC_FALSE;
+
+/* List of registered SVD monitors */
+PetscFunctionList SVDMonitorList              = NULL;
+PetscFunctionList SVDMonitorCreateList        = NULL;
+PetscFunctionList SVDMonitorDestroyList       = NULL;
+PetscBool         SVDMonitorRegisterAllCalled = PETSC_FALSE;
+
 /*@
    SVDCreate - Creates the default SVD context.
 
@@ -48,16 +57,18 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   ierr = SlepcHeaderCreate(svd,SVD_CLASSID,"SVD","Singular Value Decomposition","SVD",comm,SVDDestroy,SVDView);CHKERRQ(ierr);
 
   svd->OP               = NULL;
-  svd->max_it           = 0;
+  svd->OPb              = NULL;
+  svd->max_it           = PETSC_DEFAULT;
   svd->nsv              = 1;
-  svd->ncv              = 0;
-  svd->mpd              = 0;
+  svd->ncv              = PETSC_DEFAULT;
+  svd->mpd              = PETSC_DEFAULT;
   svd->nini             = 0;
   svd->ninil            = 0;
   svd->tol              = PETSC_DEFAULT;
   svd->conv             = SVD_CONV_REL;
   svd->stop             = SVD_STOP_BASIC;
   svd->which            = SVD_LARGEST;
+  svd->problem_type     = (SVDProblemType)0;
   svd->impltrans        = PETSC_FALSE;
   svd->trackall         = PETSC_FALSE;
 
@@ -75,18 +86,29 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   svd->U                = NULL;
   svd->V                = NULL;
   svd->A                = NULL;
+  svd->B                = NULL;
   svd->AT               = NULL;
+  svd->BT               = NULL;
   svd->IS               = NULL;
   svd->ISL              = NULL;
   svd->sigma            = NULL;
-  svd->perm             = NULL;
   svd->errest           = NULL;
+  svd->perm             = NULL;
+  svd->nworkl           = 0;
+  svd->nworkr           = 0;
+  svd->workl            = NULL;
+  svd->workr            = NULL;
   svd->data             = NULL;
 
   svd->state            = SVD_STATE_INITIAL;
   svd->nconv            = 0;
   svd->its              = 0;
   svd->leftbasis        = PETSC_FALSE;
+  svd->swapped          = PETSC_FALSE;
+  svd->expltrans        = PETSC_FALSE;
+  svd->nrma             = 0.0;
+  svd->nrmb             = 0.0;
+  svd->isgeneralized    = PETSC_FALSE;
   svd->reason           = SVD_CONVERGED_ITERATING;
 
   ierr = PetscNewLog(svd,&svd->sc);CHKERRQ(ierr);
@@ -116,15 +138,22 @@ PetscErrorCode SVDReset(SVD svd)
   if (!svd) PetscFunctionReturn(0);
   if (svd->ops->reset) { ierr = (svd->ops->reset)(svd);CHKERRQ(ierr); }
   ierr = MatDestroy(&svd->OP);CHKERRQ(ierr);
+  ierr = MatDestroy(&svd->OPb);CHKERRQ(ierr);
   ierr = MatDestroy(&svd->A);CHKERRQ(ierr);
+  ierr = MatDestroy(&svd->B);CHKERRQ(ierr);
   ierr = MatDestroy(&svd->AT);CHKERRQ(ierr);
+  ierr = MatDestroy(&svd->BT);CHKERRQ(ierr);
   ierr = BVDestroy(&svd->U);CHKERRQ(ierr);
   ierr = BVDestroy(&svd->V);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(svd->nworkl,&svd->workl);CHKERRQ(ierr);
+  svd->nworkl = 0;
+  ierr = VecDestroyVecs(svd->nworkr,&svd->workr);CHKERRQ(ierr);
+  svd->nworkr = 0;
   svd->state = SVD_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    SVDDestroy - Destroys the SVD context.
 
    Collective on svd
@@ -272,6 +301,50 @@ PetscErrorCode SVDRegister(const char *name,PetscErrorCode (*function)(SVD))
   PetscFunctionReturn(0);
 }
 
+/*@C
+   SVDMonitorRegister - Adds SVD monitor routine.
+
+   Not Collective
+
+   Input Parameters:
++  name    - name of a new monitor routine
+.  vtype   - a PetscViewerType for the output
+.  format  - a PetscViewerFormat for the output
+.  monitor - monitor routine
+.  create  - creation routine, or NULL
+-  destroy - destruction routine, or NULL
+
+   Notes:
+   SVDMonitorRegister() may be called multiple times to add several user-defined monitors.
+
+   Sample usage:
+.vb
+   SVDMonitorRegister("my_monitor",PETSCVIEWERASCII,PETSC_VIEWER_ASCII_INFO_DETAIL,MyMonitor,NULL,NULL);
+.ve
+
+   Then, your monitor can be chosen with the procedural interface via
+$      SVDMonitorSetFromOptions(svd,"-svd_monitor_my_monitor","my_monitor",NULL)
+   or at runtime via the option
+$      -svd_monitor_my_monitor
+
+   Level: advanced
+
+.seealso: SVDMonitorRegisterAll()
+@*/
+PetscErrorCode SVDMonitorRegister(const char name[],PetscViewerType vtype,PetscViewerFormat format,PetscErrorCode (*monitor)(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscErrorCode (*create)(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**),PetscErrorCode (*destroy)(PetscViewerAndFormat**))
+{
+  char           key[PETSC_MAX_PATH_LEN];
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = SVDInitializePackage();CHKERRQ(ierr);
+  ierr = SlepcMonitorMakeKey_Internal(name,vtype,format,key);CHKERRQ(ierr);
+  ierr = PetscFunctionListAdd(&SVDMonitorList,key,monitor);CHKERRQ(ierr);
+  if (create)  { ierr = PetscFunctionListAdd(&SVDMonitorCreateList,key,create);CHKERRQ(ierr); }
+  if (destroy) { ierr = PetscFunctionListAdd(&SVDMonitorDestroyList,key,destroy);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
 /*@
    SVDSetBV - Associates basis vectors objects to the singular value solver.
 
@@ -321,10 +394,10 @@ PetscErrorCode SVDSetBV(SVD svd,BV V,BV U)
 
    Not Collective
 
-   Input Parameters:
+   Input Parameter:
 .  svd - singular value solver context obtained from SVDCreate()
 
-   Output Parameter:
+   Output Parameters:
 +  V - basis vectors context for right singular vectors
 -  U - basis vectors context for left singular vectors
 
