@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -21,73 +21,77 @@ typedef struct {
 PetscErrorCode BVMult_Mat(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
 {
   PetscErrorCode ierr;
-  BV_MAT         *y = (BV_MAT*)Y->data,*x = (BV_MAT*)X->data;
-  PetscScalar    *px,*py,*q;
-  PetscInt       ldq;
+  BV_MAT            *y = (BV_MAT*)Y->data,*x = (BV_MAT*)X->data;
+  PetscScalar       *py;
+  const PetscScalar *px,*q;
+  PetscInt          ldq;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(x->A,&px);CHKERRQ(ierr);
   ierr = MatDenseGetArray(y->A,&py);CHKERRQ(ierr);
   if (Q) {
     ierr = MatGetSize(Q,&ldq,NULL);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(Q,&q);CHKERRQ(ierr);
+    ierr = MatDenseGetArrayRead(Q,&q);CHKERRQ(ierr);
     ierr = BVMult_BLAS_Private(Y,Y->n,Y->k-Y->l,X->k-X->l,ldq,alpha,px+(X->nc+X->l)*X->n,q+Y->l*ldq+X->l,beta,py+(Y->nc+Y->l)*Y->n);CHKERRQ(ierr);
-    ierr = MatDenseRestoreArray(Q,&q);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayRead(Q,&q);CHKERRQ(ierr);
   } else {
     ierr = BVAXPY_BLAS_Private(Y,Y->n,Y->k-Y->l,alpha,px+(X->nc+X->l)*X->n,beta,py+(Y->nc+Y->l)*Y->n);CHKERRQ(ierr);
   }
-  ierr = MatDenseRestoreArray(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(x->A,&px);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(y->A,&py);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode BVMultVec_Mat(BV X,PetscScalar alpha,PetscScalar beta,Vec y,PetscScalar *q)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *x = (BV_MAT*)X->data;
-  PetscScalar    *px,*py,*qq=q;
+  PetscErrorCode    ierr;
+  BV_MAT            *x = (BV_MAT*)X->data;
+  PetscScalar       *py,*qq=q;
+  const PetscScalar *px;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(x->A,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
   if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = BVMultVec_BLAS_Private(X,X->n,X->k-X->l,alpha,px+(X->nc+X->l)*X->n,qq,beta,py);CHKERRQ(ierr);
   if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
-  ierr = MatDenseRestoreArray(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(x->A,&px);CHKERRQ(ierr);
   ierr = VecRestoreArray(y,&py);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode BVMultInPlace_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *ctx = (BV_MAT*)V->data;
-  PetscScalar    *pv,*q;
-  PetscInt       ldq;
+  PetscErrorCode    ierr;
+  BV_MAT            *ctx = (BV_MAT*)V->data;
+  PetscScalar       *pv;
+  const PetscScalar *q;
+  PetscInt          ldq;
 
   PetscFunctionBegin;
   ierr = MatGetSize(Q,&ldq,NULL);CHKERRQ(ierr);
   ierr = MatDenseGetArray(ctx->A,&pv);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(Q,&q);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(Q,&q);CHKERRQ(ierr);
   ierr = BVMultInPlace_BLAS_Private(V,V->n,V->k-V->l,ldq,s-V->l,e-V->l,pv+(V->nc+V->l)*V->n,q+V->l*ldq+V->l,PETSC_FALSE);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(Q,&q);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(Q,&q);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(ctx->A,&pv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode BVMultInPlaceTranspose_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
+PetscErrorCode BVMultInPlaceHermitianTranspose_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *ctx = (BV_MAT*)V->data;
-  PetscScalar    *pv,*q;
-  PetscInt       ldq;
+  PetscErrorCode    ierr;
+  BV_MAT            *ctx = (BV_MAT*)V->data;
+  PetscScalar       *pv;
+  const PetscScalar *q;
+  PetscInt          ldq;
 
   PetscFunctionBegin;
   ierr = MatGetSize(Q,&ldq,NULL);CHKERRQ(ierr);
   ierr = MatDenseGetArray(ctx->A,&pv);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(Q,&q);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(Q,&q);CHKERRQ(ierr);
   ierr = BVMultInPlace_BLAS_Private(V,V->n,V->k-V->l,ldq,s-V->l,e-V->l,pv+(V->nc+V->l)*V->n,q+V->l*ldq+V->l,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(Q,&q);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(Q,&q);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(ctx->A,&pv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -95,19 +99,20 @@ PetscErrorCode BVMultInPlaceTranspose_Mat(BV V,Mat Q,PetscInt s,PetscInt e)
 PetscErrorCode BVDot_Mat(BV X,BV Y,Mat M)
 {
   PetscErrorCode ierr;
-  BV_MAT         *x = (BV_MAT*)X->data,*y = (BV_MAT*)Y->data;
-  PetscScalar    *px,*py,*m;
-  PetscInt       ldm;
+  BV_MAT            *x = (BV_MAT*)X->data,*y = (BV_MAT*)Y->data;
+  PetscScalar       *m;
+  const PetscScalar *px,*py;
+  PetscInt          ldm;
 
   PetscFunctionBegin;
   ierr = MatGetSize(M,&ldm,NULL);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(x->A,&px);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(y->A,&py);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(y->A,&py);CHKERRQ(ierr);
   ierr = MatDenseGetArray(M,&m);CHKERRQ(ierr);
   ierr = BVDot_BLAS_Private(X,Y->k-Y->l,X->k-X->l,X->n,ldm,py+(Y->nc+Y->l)*Y->n,px+(X->nc+X->l)*X->n,m+X->l*ldm+Y->l,x->mpi);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(M,&m);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(x->A,&px);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(y->A,&py);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(y->A,&py);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -115,8 +120,8 @@ PetscErrorCode BVDotVec_Mat(BV X,Vec y,PetscScalar *q)
 {
   PetscErrorCode    ierr;
   BV_MAT            *x = (BV_MAT*)X->data;
-  PetscScalar       *px,*qq=q;
-  const PetscScalar *py;
+  PetscScalar       *qq=q;
+  const PetscScalar *px,*py;
   Vec               z = y;
 
   PetscFunctionBegin;
@@ -124,33 +129,33 @@ PetscErrorCode BVDotVec_Mat(BV X,Vec y,PetscScalar *q)
     ierr = BV_IPMatMult(X,y);CHKERRQ(ierr);
     z = X->Bx;
   }
-  ierr = MatDenseGetArray(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(x->A,&px);CHKERRQ(ierr);
   ierr = VecGetArrayRead(z,&py);CHKERRQ(ierr);
   if (!q) { ierr = VecGetArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = BVDotVec_BLAS_Private(X,X->n,X->k-X->l,px+(X->nc+X->l)*X->n,py,qq,x->mpi);CHKERRQ(ierr);
   if (!q) { ierr = VecRestoreArray(X->buffer,&qq);CHKERRQ(ierr); }
   ierr = VecRestoreArrayRead(z,&py);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(x->A,&px);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(x->A,&px);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode BVDotVec_Local_Mat(BV X,Vec y,PetscScalar *m)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *x = (BV_MAT*)X->data;
-  PetscScalar    *px,*py;
-  Vec            z = y;
+  PetscErrorCode    ierr;
+  BV_MAT            *x = (BV_MAT*)X->data;
+  const PetscScalar *px,*py;
+  Vec               z = y;
 
   PetscFunctionBegin;
   if (X->matrix) {
     ierr = BV_IPMatMult(X,y);CHKERRQ(ierr);
     z = X->Bx;
   }
-  ierr = MatDenseGetArray(x->A,&px);CHKERRQ(ierr);
-  ierr = VecGetArray(z,&py);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(x->A,&px);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(z,&py);CHKERRQ(ierr);
   ierr = BVDotVec_BLAS_Private(X,X->n,X->k-X->l,px+(X->nc+X->l)*X->n,py,m,PETSC_FALSE);CHKERRQ(ierr);
-  ierr = VecRestoreArray(z,&py);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(x->A,&px);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(z,&py);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(x->A,&px);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -173,34 +178,48 @@ PetscErrorCode BVScale_Mat(BV bv,PetscInt j,PetscScalar alpha)
 
 PetscErrorCode BVNorm_Mat(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *ctx = (BV_MAT*)bv->data;
-  PetscScalar    *array;
+  PetscErrorCode    ierr;
+  BV_MAT            *ctx = (BV_MAT*)bv->data;
+  const PetscScalar *array;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(ctx->A,&array);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(ctx->A,&array);CHKERRQ(ierr);
   if (j<0) {
     ierr = BVNorm_LAPACK_Private(bv,bv->n,bv->k-bv->l,array+(bv->nc+bv->l)*bv->n,type,val,ctx->mpi);CHKERRQ(ierr);
   } else {
     ierr = BVNorm_LAPACK_Private(bv,bv->n,1,array+(bv->nc+j)*bv->n,type,val,ctx->mpi);CHKERRQ(ierr);
   }
-  ierr = MatDenseRestoreArray(ctx->A,&array);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(ctx->A,&array);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode BVNorm_Local_Mat(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *ctx = (BV_MAT*)bv->data;
-  PetscScalar    *array;
+  PetscErrorCode    ierr;
+  BV_MAT            *ctx = (BV_MAT*)bv->data;
+  const PetscScalar *array;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(ctx->A,&array);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(ctx->A,&array);CHKERRQ(ierr);
   if (j<0) {
     ierr = BVNorm_LAPACK_Private(bv,bv->n,bv->k-bv->l,array+(bv->nc+bv->l)*bv->n,type,val,PETSC_FALSE);CHKERRQ(ierr);
   } else {
     ierr = BVNorm_LAPACK_Private(bv,bv->n,1,array+(bv->nc+j)*bv->n,type,val,PETSC_FALSE);CHKERRQ(ierr);
   }
+  ierr = MatDenseRestoreArrayRead(ctx->A,&array);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode BVNormalize_Mat(BV bv,PetscScalar *eigi)
+{
+  PetscErrorCode ierr;
+  BV_MAT         *ctx = (BV_MAT*)bv->data;
+  PetscScalar    *array,*wi=NULL;
+
+  PetscFunctionBegin;
+  ierr = MatDenseGetArray(ctx->A,&array);CHKERRQ(ierr);
+  if (eigi) wi = eigi+bv->l;
+  ierr = BVNormalize_LAPACK_Private(bv,bv->n,bv->k-bv->l,array+(bv->nc+bv->l)*bv->n,wi,ctx->mpi);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(ctx->A,&array);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -209,13 +228,11 @@ PetscErrorCode BVMatMult_Mat(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
   PetscInt       j;
-  PetscBool      flg;
   Mat            Vmat,Wmat;
   Vec            vv,ww;
 
   PetscFunctionBegin;
-  ierr = MatHasOperation(A,MATOP_MAT_MULT,&flg);CHKERRQ(ierr);
-  if (V->vmm && flg) {
+  if (V->vmm) {
     ierr = BVGetMat(V,&Vmat);CHKERRQ(ierr);
     ierr = BVGetMat(W,&Wmat);CHKERRQ(ierr);
     ierr = MatProductCreateWithMat(A,Vmat,NULL,Wmat);CHKERRQ(ierr);
@@ -223,6 +240,7 @@ PetscErrorCode BVMatMult_Mat(BV V,Mat A,BV W)
     ierr = MatProductSetFromOptions(Wmat);CHKERRQ(ierr);
     ierr = MatProductSymbolic(Wmat);CHKERRQ(ierr);
     ierr = MatProductNumeric(Wmat);CHKERRQ(ierr);
+    ierr = MatProductClear(Wmat);CHKERRQ(ierr);
     ierr = BVRestoreMat(V,&Vmat);CHKERRQ(ierr);
     ierr = BVRestoreMat(W,&Wmat);CHKERRQ(ierr);
   } else {
@@ -239,17 +257,18 @@ PetscErrorCode BVMatMult_Mat(BV V,Mat A,BV W)
 
 PetscErrorCode BVCopy_Mat(BV V,BV W)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *v = (BV_MAT*)V->data,*w = (BV_MAT*)W->data;
-  PetscScalar    *pv,*pw,*pvc,*pwc;
+  PetscErrorCode    ierr;
+  BV_MAT            *v = (BV_MAT*)V->data,*w = (BV_MAT*)W->data;
+  PetscScalar       *pw,*pwc;
+  const PetscScalar *pv,*pvc;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(v->A,&pv);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(v->A,&pv);CHKERRQ(ierr);
   ierr = MatDenseGetArray(w->A,&pw);CHKERRQ(ierr);
   pvc = pv+(V->nc+V->l)*V->n;
   pwc = pw+(W->nc+W->l)*W->n;
   ierr = PetscArraycpy(pwc,pvc,(V->k-V->l)*V->n);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(v->A,&pv);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(v->A,&pv);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(w->A,&pw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -269,41 +288,31 @@ PetscErrorCode BVCopyColumn_Mat(BV V,PetscInt j,PetscInt i)
 
 PetscErrorCode BVResize_Mat(BV bv,PetscInt m,PetscBool copy)
 {
-  PetscErrorCode ierr;
-  BV_MAT         *ctx = (BV_MAT*)bv->data;
-  PetscScalar    *pA,*pnew;
-  PetscInt       lsplit;
-  Mat            A;
-  char           str[50];
-  PetscScalar    *array;
-  BV             parent;
+  PetscErrorCode    ierr;
+  BV_MAT            *ctx = (BV_MAT*)bv->data;
+  PetscScalar       *pnew;
+  const PetscScalar *pA;
+  Mat               A;
+  char              str[50];
 
   PetscFunctionBegin;
-  if (bv->issplit==2) {
-    parent = bv->splitparent;
-    lsplit = parent->lsplit;
-    ierr = MatDenseGetArray(((BV_MAT*)parent->data)->A,&array);CHKERRQ(ierr);
-    ierr = MatDensePlaceArray(ctx->A,array+lsplit*bv->n);CHKERRQ(ierr);
-    ierr = MatDenseRestoreArray(((BV_MAT*)parent->data)->A,&array);CHKERRQ(ierr);
-  } else if (!bv->issplit) {
-    ierr = MatCreateDense(PetscObjectComm((PetscObject)bv->t),bv->n,PETSC_DECIDE,PETSC_DECIDE,m,NULL,&A);CHKERRQ(ierr);
-    ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)A);CHKERRQ(ierr);
-    if (((PetscObject)bv)->name) {
-      ierr = PetscSNPrintf(str,50,"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject)A,str);CHKERRQ(ierr);
-    }
-    if (copy) {
-      ierr = MatDenseGetArray(ctx->A,&pA);CHKERRQ(ierr);
-      ierr = MatDenseGetArray(A,&pnew);CHKERRQ(ierr);
-      ierr = PetscArraycpy(pnew,pA,PetscMin(m,bv->m)*bv->n);CHKERRQ(ierr);
-      ierr = MatDenseRestoreArray(ctx->A,&pA);CHKERRQ(ierr);
-      ierr = MatDenseRestoreArray(A,&pnew);CHKERRQ(ierr);
-    }
-    ierr = MatDestroy(&ctx->A);CHKERRQ(ierr);
-    ctx->A = A;
+  ierr = MatCreateDense(PetscObjectComm((PetscObject)bv->t),bv->n,PETSC_DECIDE,PETSC_DECIDE,m,NULL,&A);CHKERRQ(ierr);
+  ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)A);CHKERRQ(ierr);
+  if (((PetscObject)bv)->name) {
+    ierr = PetscSNPrintf(str,sizeof(str),"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
+    ierr = PetscObjectSetName((PetscObject)A,str);CHKERRQ(ierr);
   }
+  if (copy) {
+    ierr = MatDenseGetArrayRead(ctx->A,&pA);CHKERRQ(ierr);
+    ierr = MatDenseGetArrayWrite(A,&pnew);CHKERRQ(ierr);
+    ierr = PetscArraycpy(pnew,pA,PetscMin(m,bv->m)*bv->n);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayRead(ctx->A,&pA);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayWrite(A,&pnew);CHKERRQ(ierr);
+  }
+  ierr = MatDestroy(&ctx->A);CHKERRQ(ierr);
+  ctx->A = A;
   PetscFunctionReturn(0);
 }
 
@@ -361,7 +370,7 @@ PetscErrorCode BVGetArrayRead_Mat(BV bv,const PetscScalar **a)
   BV_MAT         *ctx = (BV_MAT*)bv->data;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(ctx->A,(PetscScalar**)a);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(ctx->A,a);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -371,7 +380,7 @@ PetscErrorCode BVRestoreArrayRead_Mat(BV bv,const PetscScalar **a)
   BV_MAT         *ctx = (BV_MAT*)bv->data;
 
   PetscFunctionBegin;
-  if (a) { ierr = MatDenseRestoreArray(ctx->A,(PetscScalar**)a);CHKERRQ(ierr); }
+  if (a) { ierr = MatDenseRestoreArrayRead(ctx->A,a);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
@@ -455,7 +464,7 @@ SLEPC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
   ierr = MatAssemblyEnd(ctx->A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)ctx->A);CHKERRQ(ierr);
   if (((PetscObject)bv)->name) {
-    ierr = PetscSNPrintf(str,50,"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
+    ierr = PetscSNPrintf(str,sizeof(str),"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
     ierr = PetscObjectSetName((PetscObject)ctx->A,str);CHKERRQ(ierr);
   }
 
@@ -470,13 +479,14 @@ SLEPC_EXTERN PetscErrorCode BVCreate_Mat(BV bv)
   bv->ops->mult             = BVMult_Mat;
   bv->ops->multvec          = BVMultVec_Mat;
   bv->ops->multinplace      = BVMultInPlace_Mat;
-  bv->ops->multinplacetrans = BVMultInPlaceTranspose_Mat;
+  bv->ops->multinplacetrans = BVMultInPlaceHermitianTranspose_Mat;
   bv->ops->dot              = BVDot_Mat;
   bv->ops->dotvec           = BVDotVec_Mat;
   bv->ops->dotvec_local     = BVDotVec_Local_Mat;
   bv->ops->scale            = BVScale_Mat;
   bv->ops->norm             = BVNorm_Mat;
   bv->ops->norm_local       = BVNorm_Local_Mat;
+  bv->ops->normalize        = BVNormalize_Mat;
   bv->ops->matmult          = BVMatMult_Mat;
   bv->ops->copy             = BVCopy_Mat;
   bv->ops->copycolumn       = BVCopyColumn_Mat;
