@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -14,14 +14,15 @@
 #include <slepc/private/bvimpl.h>            /*I "slepcbv.h" I*/
 
 PetscClassId     BV_CLASSID = 0;
-PetscLogEvent    BV_Create = 0,BV_Copy = 0,BV_Mult = 0,BV_MultVec = 0,BV_MultInPlace = 0,BV_Dot = 0,BV_DotVec = 0,BV_Orthogonalize = 0,BV_OrthogonalizeVec = 0,BV_Scale = 0,BV_Norm = 0,BV_NormVec = 0,BV_SetRandom = 0,BV_MatMult = 0,BV_MatMultVec = 0,BV_MatProject = 0;
+PetscLogEvent    BV_Create = 0,BV_Copy = 0,BV_Mult = 0,BV_MultVec = 0,BV_MultInPlace = 0,BV_Dot = 0,BV_DotVec = 0,BV_Orthogonalize = 0,BV_OrthogonalizeVec = 0,BV_Scale = 0,BV_Norm = 0,BV_NormVec = 0,BV_Normalize = 0,BV_SetRandom = 0,BV_MatMult = 0,BV_MatMultVec = 0,BV_MatProject = 0,BV_SVDAndRank = 0;
 static PetscBool BVPackageInitialized = PETSC_FALSE;
-MPI_Op MPIU_TSQR = 0;
+MPI_Op MPIU_TSQR = 0,MPIU_LAPY2;
 
 const char *BVOrthogTypes[] = {"CGS","MGS","BVOrthogType","BV_ORTHOG_",0};
 const char *BVOrthogRefineTypes[] = {"IFNEEDED","NEVER","ALWAYS","BVOrthogRefineType","BV_ORTHOG_REFINE_",0};
 const char *BVOrthogBlockTypes[] = {"GS","CHOL","TSQR","TSQRCHOL","SVQB","BVOrthogBlockType","BV_ORTHOG_BLOCK_",0};
 const char *BVMatMultTypes[] = {"VECS","MAT","MAT_SAVE","BVMatMultType","BV_MATMULT_",0};
+const char *BVSVDMethods[] = {"REFINE","QR","QR_CAA","BVSVDMethod","BV_SVD_METHOD_",0};
 
 /*@C
    BVFinalizePackage - This function destroys everything in the Slepc interface
@@ -37,7 +38,8 @@ PetscErrorCode BVFinalizePackage(void)
 
   PetscFunctionBegin;
   ierr = PetscFunctionListDestroy(&BVList);CHKERRQ(ierr);
-  ierr = MPI_Op_free(&MPIU_TSQR);CHKERRQ(ierr);
+  ierr = MPI_Op_free(&MPIU_TSQR);CHKERRMPI(ierr);
+  ierr = MPI_Op_free(&MPIU_LAPY2);CHKERRMPI(ierr);
   BVPackageInitialized = PETSC_FALSE;
   BVRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
@@ -79,12 +81,15 @@ PetscErrorCode BVInitializePackage(void)
   ierr = PetscLogEventRegister("BVScale",BV_CLASSID,&BV_Scale);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVNorm",BV_CLASSID,&BV_Norm);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVNormVec",BV_CLASSID,&BV_NormVec);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVNormalize",BV_CLASSID,&BV_Normalize);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVSetRandom",BV_CLASSID,&BV_SetRandom);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVMatMult",BV_CLASSID,&BV_MatMult);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVMatMultVec",BV_CLASSID,&BV_MatMultVec);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVMatProject",BV_CLASSID,&BV_MatProject);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVSVDAndRank",BV_CLASSID,&BV_SVDAndRank);CHKERRQ(ierr);
   /* MPI reduction operation used in BVOrthogonalize */
-  ierr = MPI_Op_create(SlepcGivensPacked,PETSC_TRUE,&MPIU_TSQR);CHKERRQ(ierr);
+  ierr = MPI_Op_create(SlepcGivensPacked,PETSC_FALSE,&MPIU_TSQR);CHKERRMPI(ierr);
+  ierr = MPI_Op_create(SlepcPythag,PETSC_TRUE,&MPIU_LAPY2);CHKERRMPI(ierr);
   /* Process Info */
   classids[0] = BV_CLASSID;
   ierr = PetscInfoProcessClass("bv",1,&classids[0]);CHKERRQ(ierr);
@@ -99,7 +104,7 @@ PetscErrorCode BVInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    BVDestroy - Destroys BV context that was created with BVCreate().
 
    Collective on bv
@@ -179,6 +184,8 @@ PetscErrorCode BVCreate(MPI_Comm comm,BV *newbv)
   bv->matrix       = NULL;
   bv->indef        = PETSC_FALSE;
   bv->vmm          = BV_MATMULT_MAT;
+  bv->rrandom      = PETSC_FALSE;
+  bv->deftol       = 10*PETSC_MACHINE_EPSILON;
 
   bv->Bx           = NULL;
   bv->buffer       = NULL;
@@ -211,6 +218,7 @@ PetscErrorCode BVCreate(MPI_Comm comm,BV *newbv)
   bv->Acreate      = NULL;
   bv->Aget         = NULL;
   bv->cuda         = PETSC_FALSE;
+  bv->sfocalled    = PETSC_FALSE;
   bv->work         = NULL;
   bv->lwork        = 0;
   bv->data         = NULL;
@@ -243,13 +251,11 @@ PetscErrorCode BVCreate(MPI_Comm comm,BV *newbv)
 PetscErrorCode BVCreateFromMat(Mat A,BV *bv)
 {
   PetscErrorCode ierr;
-  PetscBool      match;
   PetscInt       n,N,k;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(A,MAT_CLASSID,1);
-  ierr = PetscObjectTypeCompareAny((PetscObject)A,&match,MATSEQDENSE,MATMPIDENSE,"");CHKERRQ(ierr);
-  if (!match) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"Matrix must be of type dense");
+  PetscCheckTypeNames(A,MATSEQDENSE,MATMPIDENSE);
 
   ierr = MatGetSize(A,&N,&k);CHKERRQ(ierr);
   ierr = MatGetLocalSize(A,&n,NULL);CHKERRQ(ierr);
@@ -393,7 +399,7 @@ PetscErrorCode BVInsertVecs(BV V,PetscInt s,PetscInt *m,Vec *W,PetscBool orth)
    operation, so they are intended for removing those directions from the
    orthogonal basis computed in regular BV columns.
 
-   Constraints are not stored in regular BV colums, but in a special part of
+   Constraints are not stored in regular BV columns, but in a special part of
    the storage. They can be accessed with negative indices in BVGetColumn().
 
    This operation is DESTRUCTIVE, meaning that all data contained in the
@@ -623,6 +629,7 @@ PetscErrorCode BVView(BV bv,PetscViewer viewer)
         } else {
           ierr = PetscViewerASCIIPrintf(viewer,"  non-standard inner product\n");CHKERRQ(ierr);
         }
+        ierr = PetscViewerASCIIPrintf(viewer,"  tolerance for definite inner product: %g\n",(double)bv->deftol);CHKERRQ(ierr);
         ierr = PetscViewerASCIIPrintf(viewer,"  inner product matrix:\n");CHKERRQ(ierr);
         ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
         ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
