@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -28,6 +28,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVDot_Private(BV X,BV Y,Mat M)
   Vec            z;
 
   PetscFunctionBegin;
+  BVCheckOp(Y,1,dotvec);
   ierr = MatGetSize(M,&m,NULL);CHKERRQ(ierr);
   ierr = MatDenseGetArray(M,&marray);CHKERRQ(ierr);
   ierr = PetscObjectGetId((PetscObject)X,&idx);CHKERRQ(ierr);
@@ -54,14 +55,14 @@ PETSC_STATIC_INLINE PetscErrorCode BVDot_Private(BV X,BV Y,Mat M)
 /*@
    BVDot - Computes the 'block-dot' product of two basis vectors objects.
 
-   Collective on BV
+   Collective on X
 
    Input Parameters:
-+  X, Y - basis vectors
--  M    - Mat object where the result must be placed
++  X - first basis vectors
+-  Y - second basis vectors
 
    Output Parameter:
-.  M    - the resulting matrix
+.  M - the resulting matrix
 
    Notes:
    This is the generalization of VecDot() for a collection of vectors, M = Y^H*X.
@@ -86,7 +87,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVDot_Private(BV X,BV Y,Mat M)
 PetscErrorCode BVDot(BV X,BV Y,Mat M)
 {
   PetscErrorCode ierr;
-  PetscBool      match;
   PetscInt       m,n;
 
   PetscFunctionBegin;
@@ -99,8 +99,7 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
   BVCheckSizes(Y,2);
   PetscValidType(M,3);
   PetscCheckSameTypeAndComm(X,1,Y,2);
-  ierr = PetscObjectTypeCompare((PetscObject)M,MATSEQDENSE,&match);CHKERRQ(ierr);
-  if (!match) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_SUP,"Mat argument must be of type seqdense");
+  PetscCheckTypeNames(M,MATSEQDENSE,MATSEQDENSECUDA);
 
   ierr = MatGetSize(M,&m,&n);CHKERRQ(ierr);
   if (m<Y->k) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_SIZ,"Mat argument has %D rows, should have at least %D",m,Y->k);
@@ -130,7 +129,7 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
    BVDotVec - Computes multiple dot products of a vector against all the
    column vectors of a BV.
 
-   Collective on BV and Vec
+   Collective on X
 
    Input Parameters:
 +  X - basis vectors
@@ -165,6 +164,7 @@ PetscErrorCode BVDotVec(BV X,Vec y,PetscScalar m[])
   PetscValidHeaderSpecific(y,VEC_CLASSID,2);
   PetscValidType(X,1);
   BVCheckSizes(X,1);
+  BVCheckOp(X,1,dotvec);
   PetscValidType(y,2);
   PetscCheckSameTypeAndComm(X,1,y,2);
 
@@ -213,6 +213,7 @@ PetscErrorCode BVDotVecBegin(BV X,Vec y,PetscScalar *m)
   if (X->ops->dotvec_begin) {
     ierr = (*X->ops->dotvec_begin)(X,y,m);CHKERRQ(ierr);
   } else {
+    BVCheckOp(X,1,dotvec_local);
     nv = X->k-X->l;
     ierr = PetscObjectGetComm((PetscObject)X,&comm);CHKERRQ(ierr);
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
@@ -221,7 +222,7 @@ PetscErrorCode BVDotVecBegin(BV X,Vec y,PetscScalar *m)
       if (sr->numopsbegin+i >= sr->maxops) {
         ierr = PetscSplitReductionExtend(sr);CHKERRQ(ierr);
       }
-      sr->reducetype[sr->numopsbegin+i] = REDUCE_SUM;
+      sr->reducetype[sr->numopsbegin+i] = PETSC_SR_REDUCE_SUM;
       sr->invecs[sr->numopsbegin+i]     = (void*)X;
     }
     ierr = PetscLogEventBegin(BV_DotVec,X,y,0,0);CHKERRQ(ierr);
@@ -269,7 +270,7 @@ PetscErrorCode BVDotVecEnd(BV X,Vec y,PetscScalar *m)
 
     if (sr->numopsend >= sr->numopsbegin) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times than VecxxxBegin()");
     if ((void*)X != sr->invecs[sr->numopsend]) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called BVxxxEnd() in a different order or with a different BV than BVxxxBegin()");
-    if (sr->reducetype[sr->numopsend] != REDUCE_SUM) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
+    if (sr->reducetype[sr->numopsend] != PETSC_SR_REDUCE_SUM) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
     for (i=0;i<nv;i++) m[i] = sr->gvalues[sr->numopsend++];
 
     /* Finished getting all the results so reset to no outstanding requests */
@@ -286,7 +287,7 @@ PetscErrorCode BVDotVecEnd(BV X,Vec y,PetscScalar *m)
    BVDotColumn - Computes multiple dot products of a column against all the
    previous columns of a BV.
 
-   Collective on BV
+   Collective on X
 
    Input Parameters:
 +  X - basis vectors
@@ -322,6 +323,7 @@ PetscErrorCode BVDotColumn(BV X,PetscInt j,PetscScalar *q)
   PetscValidLogicalCollectiveInt(X,j,2);
   PetscValidType(X,1);
   BVCheckSizes(X,1);
+  BVCheckOp(X,1,dotvec);
 
   if (j<0) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_OUTOFRANGE,"Index j must be non-negative");
   if (j>=X->m) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_OUTOFRANGE,"Index j=%D but BV only has %D columns",j,X->m);
@@ -376,6 +378,7 @@ PetscErrorCode BVDotColumnBegin(BV X,PetscInt j,PetscScalar *m)
   if (X->ops->dotvec_begin) {
     ierr = (*X->ops->dotvec_begin)(X,y,m);CHKERRQ(ierr);
   } else {
+    BVCheckOp(X,1,dotvec_local);
     nv = X->k-X->l;
     ierr = PetscObjectGetComm((PetscObject)X,&comm);CHKERRQ(ierr);
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
@@ -384,7 +387,7 @@ PetscErrorCode BVDotColumnBegin(BV X,PetscInt j,PetscScalar *m)
       if (sr->numopsbegin+i >= sr->maxops) {
         ierr = PetscSplitReductionExtend(sr);CHKERRQ(ierr);
       }
-      sr->reducetype[sr->numopsbegin+i] = REDUCE_SUM;
+      sr->reducetype[sr->numopsbegin+i] = PETSC_SR_REDUCE_SUM;
       sr->invecs[sr->numopsbegin+i]     = (void*)X;
     }
     ierr = PetscLogEventBegin(BV_DotVec,X,0,0,0);CHKERRQ(ierr);
@@ -443,7 +446,7 @@ PetscErrorCode BVDotColumnEnd(BV X,PetscInt j,PetscScalar *m)
 
     if (sr->numopsend >= sr->numopsbegin) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times than VecxxxBegin()");
     if ((void*)X != sr->invecs[sr->numopsend]) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Called BVxxxEnd() in a different order or with a different BV than BVxxxBegin()");
-    if (sr->reducetype[sr->numopsend] != REDUCE_SUM) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
+    if (sr->reducetype[sr->numopsend] != PETSC_SR_REDUCE_SUM) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_WRONGSTATE,"Wrong type of reduction");
     for (i=0;i<nv;i++) m[i] = sr->gvalues[sr->numopsend++];
 
     /* Finished getting all the results so reset to no outstanding requests */
@@ -494,7 +497,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVNorm_End_Private(BV bv,Vec z,NormType type,
 /*@
    BVNorm - Computes the matrix norm of the BV.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv   - basis vectors
@@ -512,7 +515,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVNorm_End_Private(BV bv,Vec z,NormType type,
 
    Level: intermediate
 
-.seealso: BVNormVec(), BVNormColumn(), BVSetActiveColumns(), BVSetMatrix()
+.seealso: BVNormVec(), BVNormColumn(), BVNormalize(), BVSetActiveColumns(), BVSetMatrix()
 @*/
 PetscErrorCode BVNorm(BV bv,NormType type,PetscReal *val)
 {
@@ -521,7 +524,7 @@ PetscErrorCode BVNorm(BV bv,NormType type,PetscReal *val)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveEnum(bv,type,2);
-  PetscValidPointer(val,3);
+  PetscValidRealPointer(val,3);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -537,7 +540,7 @@ PetscErrorCode BVNorm(BV bv,NormType type,PetscReal *val)
 /*@
    BVNormVec - Computes the norm of a given vector.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv   - basis vectors
@@ -566,7 +569,7 @@ PetscErrorCode BVNormVec(BV bv,Vec v,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidHeaderSpecific(v,VEC_CLASSID,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidPointer(val,4);
+  PetscValidRealPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
   PetscValidType(v,2);
@@ -611,7 +614,7 @@ PetscErrorCode BVNormVecBegin(BV bv,Vec v,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidHeaderSpecific(v,VEC_CLASSID,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidPointer(val,4);
+  PetscValidRealPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
   PetscValidType(v,2);
@@ -654,7 +657,7 @@ PetscErrorCode BVNormVecEnd(BV bv,Vec v,NormType type,PetscReal *val)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidPointer(val,4);
+  PetscValidRealPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -671,7 +674,7 @@ PetscErrorCode BVNormVecEnd(BV bv,Vec v,NormType type,PetscReal *val)
 /*@
    BVNormColumn - Computes the vector norm of a selected column.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv   - basis vectors
@@ -689,7 +692,7 @@ PetscErrorCode BVNormVecEnd(BV bv,Vec v,NormType type,PetscReal *val)
 
    Level: intermediate
 
-.seealso: BVNorm(), BVNormVec(), BVSetActiveColumns(), BVSetMatrix()
+.seealso: BVNorm(), BVNormVec(), BVNormalize(), BVSetActiveColumns(), BVSetMatrix()
 @*/
 PetscErrorCode BVNormColumn(BV bv,PetscInt j,NormType type,PetscReal *val)
 {
@@ -700,7 +703,7 @@ PetscErrorCode BVNormColumn(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(bv,j,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidPointer(val,4);
+  PetscValidRealPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -747,7 +750,7 @@ PetscErrorCode BVNormColumnBegin(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(bv,j,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidPointer(val,4);
+  PetscValidRealPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -761,6 +764,7 @@ PetscErrorCode BVNormColumnBegin(BV bv,PetscInt j,NormType type,PetscReal *val)
   } else if (bv->ops->norm_begin) {
     ierr = (*bv->ops->norm_begin)(bv,j,type,val);CHKERRQ(ierr);
   } else {
+    BVCheckOp(bv,1,norm_local);
     ierr = PetscObjectGetComm((PetscObject)z,&comm);CHKERRQ(ierr);
     ierr = PetscSplitReductionGet(comm,&sr);CHKERRQ(ierr);
     if (sr->state != STATE_BEGIN) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ORDER,"Called before all VecxxxEnd() called");
@@ -770,8 +774,8 @@ PetscErrorCode BVNormColumnBegin(BV bv,PetscInt j,NormType type,PetscReal *val)
     sr->invecs[sr->numopsbegin] = (void*)bv;
     ierr = (*bv->ops->norm_local)(bv,j,type,&lresult);CHKERRQ(ierr);
     if (type == NORM_2) lresult = lresult*lresult;
-    if (type == NORM_MAX) sr->reducetype[sr->numopsbegin] = REDUCE_MAX;
-    else sr->reducetype[sr->numopsbegin] = REDUCE_SUM;
+    if (type == NORM_MAX) sr->reducetype[sr->numopsbegin] = PETSC_SR_REDUCE_MAX;
+    else sr->reducetype[sr->numopsbegin] = PETSC_SR_REDUCE_SUM;
     sr->lvalues[sr->numopsbegin++] = lresult;
   }
   ierr = BVRestoreColumn(bv,j,&z);CHKERRQ(ierr);
@@ -806,7 +810,7 @@ PetscErrorCode BVNormColumnEnd(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(bv,j,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidPointer(val,4);
+  PetscValidRealPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -824,7 +828,7 @@ PetscErrorCode BVNormColumnEnd(BV bv,PetscInt j,NormType type,PetscReal *val)
 
     if (sr->numopsend >= sr->numopsbegin) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() more times then VecxxxBegin()");
     if ((void*)bv != sr->invecs[sr->numopsend]) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Called VecxxxEnd() in a different order or with a different vector than VecxxxBegin()");
-    if (sr->reducetype[sr->numopsend] != REDUCE_MAX && type == NORM_MAX) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Called BVNormEnd(,NORM_MAX,) on a reduction started with VecDotBegin() or NORM_1 or NORM_2");
+    if (sr->reducetype[sr->numopsend] != PETSC_SR_REDUCE_MAX && type == NORM_MAX) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Called BVNormEnd(,NORM_MAX,) on a reduction started with VecDotBegin() or NORM_1 or NORM_2");
     *val = PetscRealPart(sr->gvalues[sr->numopsend++]);
     if (type == NORM_2) *val = PetscSqrtReal(*val);
     if (sr->numopsend == sr->numopsbegin) {
@@ -837,6 +841,84 @@ PetscErrorCode BVNormColumnEnd(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscFunctionReturn(0);
 }
 
+/*@
+   BVNormalize - Normalize all columns (starting from the leading ones).
+
+   Collective on bv
+
+   Input Parameters:
++  bv   - basis vectors
+-  eigi - (optional) imaginary parts of eigenvalues
+
+   Notes:
+   On output, all columns will have unit norm. The normalization is done with
+   respect to the 2-norm, or to the B-norm if a non-standard inner product has
+   been specified with BVSetMatrix(), see BVNormColumn().
+
+   If the optional argument eigi is passed (taken into account only in real
+   scalars) it is interpreted as the imaginary parts of the eigenvalues and
+   the BV is supposed to contain the corresponding eigenvectors. Suppose the
+   first three values are eigi = { 0, alpha, -alpha }, then the first column
+   is normalized as usual, but the second and third ones are normalized assuming
+   that they contain the real and imaginary parts of a complex conjugate pair of
+   eigenvectors.
+
+   If eigi is passed, the inner-product matrix is ignored.
+
+   If there are leading columns, they are not modified (are assumed to be already
+   normalized).
+
+   Level: intermediate
+
+.seealso: BVNormColumn()
+@*/
+PetscErrorCode BVNormalize(BV bv,PetscScalar *eigi)
+{
+  PetscErrorCode ierr;
+  PetscReal      norm;
+  PetscInt       i;
+  Vec            v;
+#if !defined(PETSC_USE_COMPLEX)
+  Vec            v1;
+#endif
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  PetscValidType(bv,1);
+  BVCheckSizes(bv,1);
+
+  ierr = PetscLogEventBegin(BV_Normalize,bv,0,0,0);CHKERRQ(ierr);
+  if (bv->matrix && !eigi) {
+    for (i=bv->l;i<bv->k;i++) {
+      ierr = BVNormColumn(bv,i,NORM_2,&norm);CHKERRQ(ierr);
+      ierr = BVScaleColumn(bv,i,1.0/norm);CHKERRQ(ierr);
+    }
+  } else if (bv->ops->normalize) {
+    ierr = (*bv->ops->normalize)(bv,eigi);CHKERRQ(ierr);
+  } else {
+    for (i=bv->l;i<bv->k;i++) {
+#if !defined(PETSC_USE_COMPLEX)
+      if (eigi && eigi[i] != 0.0) {
+        ierr = BVGetColumn(bv,i,&v);CHKERRQ(ierr);
+        ierr = BVGetColumn(bv,i+1,&v1);CHKERRQ(ierr);
+        ierr = VecNormalizeComplex(v,v1,PETSC_TRUE,NULL);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(bv,i,&v);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(bv,i+1,&v1);CHKERRQ(ierr);
+        i++;
+      } else
+#endif
+      {
+        ierr = BVGetColumn(bv,i,&v);CHKERRQ(ierr);
+        ierr = VecNormalize(v,NULL);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(bv,i,&v);CHKERRQ(ierr);
+      }
+    }
+  }
+  ierr = PetscLogEventEnd(BV_Normalize,bv,0,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)bv);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*
   Compute Y^H*A*X: right part column by column (with MatMult) and bottom
   part row by row (with MatMultHermitianTranspose); result placed in marray[*,ldm]
@@ -851,6 +933,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Vec(BV X,Mat A,BV Y,PetscScalar
   lx = X->l; kx = X->k;
   ly = Y->l; ky = Y->k;
   ierr = BVCreateVec(X,&f);CHKERRQ(ierr);
+  BVCheckOp(Y,3,dotvec);
   for (j=lx;j<kx;j++) {
     ierr = BVGetColumn(X,j,&z);CHKERRQ(ierr);
     ierr = MatMult(A,z,f);CHKERRQ(ierr);
@@ -863,6 +946,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Vec(BV X,Mat A,BV Y,PetscScalar
     }
   }
   if (!symm) {
+    BVCheckOp(X,1,dotvec);
     ierr = BV_AllocateCoeffs(Y);CHKERRQ(ierr);
     for (j=ly;j<ky;j++) {
       ierr = BVGetColumn(Y,j,&z);CHKERRQ(ierr);
@@ -889,11 +973,11 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Vec(BV X,Mat A,BV Y,PetscScalar
 */
 PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult(BV X,Mat A,BV Y,PetscScalar *marray,PetscInt ldm)
 {
-  PetscErrorCode ierr;
-  PetscInt       j,lx,ly,kx,ky;
-  PetscScalar    *harray;
-  Mat            H;
-  BV             W;
+  PetscErrorCode    ierr;
+  PetscInt          j,lx,ly,kx,ky;
+  const PetscScalar *harray;
+  Mat               H;
+  BV                W;
 
   PetscFunctionBegin;
   lx = X->l; kx = X->k;
@@ -909,11 +993,11 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult(BV X,Mat A,BV Y,PetscSca
     W->l = lx; W->k = kx;
     Y->l = 0;  Y->k = ly;
     ierr = BVDot(W,Y,H);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseGetArrayRead(H,&harray);CHKERRQ(ierr);
     for (j=lx;j<kx;j++) {
-      ierr = PetscMemcpy(marray+j*ldm,harray+j*ly,ly*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(marray+j*ldm,harray+j*ly,ly);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayRead(H,&harray);CHKERRQ(ierr);
     ierr = MatDestroy(&H);CHKERRQ(ierr);
   }
 
@@ -923,11 +1007,11 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult(BV X,Mat A,BV Y,PetscSca
     W->l = 0;  W->k = kx;
     Y->l = ly; Y->k = ky;
     ierr = BVDot(W,Y,H);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseGetArrayRead(H,&harray);CHKERRQ(ierr);
     for (j=0;j<kx;j++) {
-      ierr = PetscMemcpy(marray+j*ldm+ly,harray+j*ky+ly,(ky-ly)*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(marray+j*ldm+ly,harray+j*ky+ly,ky-ly);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayRead(H,&harray);CHKERRQ(ierr);
     ierr = MatDestroy(&H);CHKERRQ(ierr);
   }
   ierr = BVDestroy(&W);CHKERRQ(ierr);
@@ -940,16 +1024,16 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult(BV X,Mat A,BV Y,PetscSca
   Compute Y^H*A*X= [   --   | Y0'*W1 ]
                    [ Y1'*W0 | Y1'*W1 ]
   First stage: allocate auxiliary BV to store A*X1, one BVDot for right part;
-  Second stage: resize BV to accomodate A'*Y1, then call BVDot for transpose of
+  Second stage: resize BV to accommodate A'*Y1, then call BVDot for transpose of
   bottom-left part; result placed in marray[*,ldm]
 */
 PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult_2(BV X,Mat A,BV Y,PetscScalar *marray,PetscInt ldm,PetscBool symm)
 {
-  PetscErrorCode ierr;
-  PetscInt       i,j,lx,ly,kx,ky;
-  PetscScalar    *harray;
-  Mat            H;
-  BV             W;
+  PetscErrorCode    ierr;
+  PetscInt          i,j,lx,ly,kx,ky;
+  const PetscScalar *harray;
+  Mat               H;
+  BV                W;
 
   PetscFunctionBegin;
   lx = X->l; kx = X->k;
@@ -962,11 +1046,11 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult_2(BV X,Mat A,BV Y,PetscS
     ierr = MatCreateSeqDense(PETSC_COMM_SELF,ky,kx-lx,NULL,&H);CHKERRQ(ierr);
     Y->l = 0; Y->k = ky;
     ierr = BVDot(W,Y,H);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseGetArrayRead(H,&harray);CHKERRQ(ierr);
     for (j=lx;j<kx;j++) {
-      ierr = PetscMemcpy(marray+j*ldm,harray+(j-lx)*ky,ky*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(marray+j*ldm,harray+(j-lx)*ky,ky);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayRead(H,&harray);CHKERRQ(ierr);
     ierr = MatDestroy(&H);CHKERRQ(ierr);
   }
 
@@ -984,13 +1068,13 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult_2(BV X,Mat A,BV Y,PetscS
       ierr = MatCreateSeqDense(PETSC_COMM_SELF,lx,ky-ly,NULL,&H);CHKERRQ(ierr);
       X->l = 0; X->k = lx;
       ierr = BVDot(W,X,H);CHKERRQ(ierr);
-      ierr = MatDenseGetArray(H,&harray);CHKERRQ(ierr);
+      ierr = MatDenseGetArrayRead(H,&harray);CHKERRQ(ierr);
       for (i=0;i<ky-ly;i++) {
         for (j=0;j<lx;j++) {
           marray[i+j*ldm+ly] = PetscConj(harray[j+i*(ky-ly)]);
         }
       }
-      ierr = MatDenseRestoreArray(H,&harray);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArrayRead(H,&harray);CHKERRQ(ierr);
       ierr = MatDestroy(&H);CHKERRQ(ierr);
     }
   }
@@ -1008,10 +1092,10 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_MatMult_2(BV X,Mat A,BV Y,PetscS
 */
 PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Dot(BV X,BV Y,PetscScalar *marray,PetscInt ldm)
 {
-  PetscErrorCode ierr;
-  PetscInt       j,lx,ly,kx,ky;
-  PetscScalar    *harray;
-  Mat            H;
+  PetscErrorCode    ierr;
+  PetscInt          j,lx,ly,kx,ky;
+  const PetscScalar *harray;
+  Mat               H;
 
   PetscFunctionBegin;
   lx = X->l; kx = X->k;
@@ -1023,11 +1107,11 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Dot(BV X,BV Y,PetscScalar *marra
     X->l = lx; X->k = kx;
     Y->l = 0;  Y->k = ly;
     ierr = BVDot(X,Y,H);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseGetArrayRead(H,&harray);CHKERRQ(ierr);
     for (j=lx;j<kx;j++) {
-      ierr = PetscMemcpy(marray+j*ldm,harray+j*ly,ly*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(marray+j*ldm,harray+j*ly,ly);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayRead(H,&harray);CHKERRQ(ierr);
     ierr = MatDestroy(&H);CHKERRQ(ierr);
   }
 
@@ -1037,11 +1121,11 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Dot(BV X,BV Y,PetscScalar *marra
     X->l = 0;  X->k = kx;
     Y->l = ly; Y->k = ky;
     ierr = BVDot(X,Y,H);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseGetArrayRead(H,&harray);CHKERRQ(ierr);
     for (j=0;j<kx;j++) {
-      ierr = PetscMemcpy(marray+j*ldm+ly,harray+j*ky+ly,(ky-ly)*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(marray+j*ldm+ly,harray+j*ky+ly,ky-ly);CHKERRQ(ierr);
     }
-    ierr = MatDenseRestoreArray(H,&harray);CHKERRQ(ierr);
+    ierr = MatDenseRestoreArrayRead(H,&harray);CHKERRQ(ierr);
     ierr = MatDestroy(&H);CHKERRQ(ierr);
   }
   X->l = lx; X->k = kx;
@@ -1052,7 +1136,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Dot(BV X,BV Y,PetscScalar *marra
 /*@
    BVMatProject - Computes the projection of a matrix onto a subspace.
 
-   Collective on BV
+   Collective on X
 
    Input Parameters:
 +  X - basis vectors
@@ -1095,7 +1179,7 @@ PETSC_STATIC_INLINE PetscErrorCode BVMatProject_Dot(BV X,BV Y,PetscScalar *marra
 PetscErrorCode BVMatProject(BV X,Mat A,BV Y,Mat M)
 {
   PetscErrorCode ierr;
-  PetscBool      match,set,flg,symm=PETSC_FALSE;
+  PetscBool      set,flg,symm=PETSC_FALSE;
   PetscInt       m,n;
   PetscScalar    *marray;
   Mat            Xmatrix,Ymatrix;
@@ -1116,8 +1200,7 @@ PetscErrorCode BVMatProject(BV X,Mat A,BV Y,Mat M)
   BVCheckSizes(Y,3);
   PetscValidType(M,4);
   PetscCheckSameTypeAndComm(X,1,Y,3);
-  ierr = PetscObjectTypeCompare((PetscObject)M,MATSEQDENSE,&match);CHKERRQ(ierr);
-  if (!match) SETERRQ(PetscObjectComm((PetscObject)X),PETSC_ERR_SUP,"Matrix M must be of type seqdense");
+  PetscCheckTypeNames(M,MATSEQDENSE,MATSEQDENSECUDA);
 
   ierr = MatGetSize(M,&m,&n);CHKERRQ(ierr);
   if (m<Y->k) SETERRQ2(PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_SIZ,"Matrix M has %D rows, should have at least %D",m,Y->k);
