@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -14,6 +14,23 @@
 #include <slepc/private/bvimpl.h>
 #include <slepcblaslapack.h>
 
+/*
+    Reduction operation to compute sqrt(x**2+y**2) when normalizing vectors
+*/
+SLEPC_EXTERN void MPIAPI SlepcPythag(void *in,void *inout,PetscMPIInt *len,MPI_Datatype *datatype)
+{
+  PetscBLASInt i,n=*len;
+  PetscReal    *x = (PetscReal*)in,*y = (PetscReal*)inout;
+
+  PetscFunctionBegin;
+  if (PetscUnlikely(*datatype!=MPIU_REAL)) {
+    (*PetscErrorPrintf)("Only implemented for MPIU_REAL data type");
+    MPI_Abort(MPI_COMM_WORLD,1);
+  }
+  for (i=0;i<n;i++) y[i] = SlepcAbs(x[i],y[i]);
+  PetscFunctionReturnVoid();
+}
+
 /*
     Compute ||A|| for an mxn matrix
 */
@@ -31,9 +48,7 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
   if (type==NORM_FROBENIUS || type==NORM_2) {
     lnrm = LAPACKlange_("F",&m,&n,(PetscScalar*)A,&m,rwork);
     if (mpi) {
-      lnrm = lnrm*lnrm;
-      ierr = MPI_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-      *nrm = PetscSqrtReal(*nrm);
+      ierr = MPIU_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_LAPY2,PetscObjectComm((PetscObject)bv));CHKERRMPI(ierr);
     } else *nrm = lnrm;
     ierr = PetscLogFlops(2.0*m*n);CHKERRQ(ierr);
   } else if (type==NORM_1) {
@@ -49,7 +64,7 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
         }
       }
       ierr = PetscMPIIntCast(n_,&len);CHKERRQ(ierr);
-      ierr = MPI_Allreduce(rwork,rwork2,len,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+      ierr = MPIU_Allreduce(rwork,rwork2,len,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRMPI(ierr);
       *nrm = 0.0;
       for (j=0;j<n_;j++) if (rwork2[j] > *nrm) *nrm = rwork2[j];
     } else {
@@ -61,7 +76,7 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
     rwork = (PetscReal*)bv->work;
     lnrm = LAPACKlange_("I",&m,&n,(PetscScalar*)A,&m,rwork);
     if (mpi) {
-      ierr = MPI_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+      ierr = MPIU_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)bv));CHKERRMPI(ierr);
     } else *nrm = lnrm;
     ierr = PetscLogFlops(1.0*m*n);CHKERRQ(ierr);
   }
@@ -69,6 +84,53 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
   PetscFunctionReturn(0);
 }
 
+/*
+    Normalize the columns of an mxn matrix A
+*/
+PetscErrorCode BVNormalize_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscScalar *A,PetscScalar *eigi,PetscBool mpi)
+{
+  PetscErrorCode ierr;
+  PetscBLASInt   m,j,k,info,zero=0;
+  PetscMPIInt    len;
+  PetscReal      *norms,*rwork=NULL,*rwork2=NULL,done=1.0;
+
+  PetscFunctionBegin;
+  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(m_,&m);CHKERRQ(ierr);
+  ierr = BVAllocateWork_Private(bv,2*n_);CHKERRQ(ierr);
+  rwork = (PetscReal*)bv->work;
+  rwork2 = rwork+n_;
+  /* compute local norms */
+  for (j=0;j<n_;j++) {
+    k = 1;
+#if !defined(PETSC_USE_COMPLEX)
+    if (eigi && eigi[j] != 0.0) k = 2;
+#endif
+    rwork[j] = LAPACKlange_("F",&m,&k,(PetscScalar*)(A+j*m_),&m,rwork2);
+    if (k==2) { rwork[j+1] = rwork[j]; j++; }
+  }
+  /* reduction to get global norms */
+  if (mpi) {
+    ierr = PetscMPIIntCast(n_,&len);CHKERRQ(ierr);
+    ierr = PetscArrayzero(rwork2,n_);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(rwork,rwork2,len,MPIU_REAL,MPIU_LAPY2,PetscObjectComm((PetscObject)bv));CHKERRMPI(ierr);
+    norms = rwork2;
+  } else norms = rwork;
+  /* scale columns */
+  for (j=0;j<n_;j++) {
+    k = 1;
+#if !defined(PETSC_USE_COMPLEX)
+    if (eigi && eigi[j] != 0.0) k = 2;
+#endif
+    PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,norms+j,&done,&m,&k,(PetscScalar*)(A+j*m_),&m,&info));
+    SlepcCheckLapackInfo("lascl",info);
+    if (k==2) j++;
+  }
+  ierr = PetscLogFlops(3.0*m*n_);CHKERRQ(ierr);
+  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*
    Compute the upper Cholesky factor in R and its inverse in S.
    If S == R then the inverse overwrites the Cholesky factor.
@@ -78,7 +140,7 @@ PetscErrorCode BVMatCholInv_LAPACK_Private(BV bv,Mat R,Mat S)
   PetscErrorCode ierr;
   PetscInt       i,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS;
-  PetscBLASInt   info,n_,m_,ld_,lds_;
+  PetscBLASInt   info,n_ = 0,m_ = 0,ld_,lds_;
 
   PetscFunctionBegin;
   l = bv->l;
@@ -108,6 +170,7 @@ PetscErrorCode BVMatCholInv_LAPACK_Private(BV bv,Mat R,Mat S)
   }
 
   /* compute upper Cholesky factor in R */
+  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKpotrf",LAPACKpotrf_("U",&n_,pR+l*ld+l,&ld_,&info));
   ierr = PetscLogFlops((1.0*n*n*n)/3.0);CHKERRQ(ierr);
 
@@ -132,6 +195,7 @@ PetscErrorCode BVMatCholInv_LAPACK_Private(BV bv,Mat R,Mat S)
     PetscStackCallBLAS("LAPACKtrtri",LAPACKtrtri_("U","N",&n_,pS+l*lds+l,&lds_,&info));
   }
   SlepcCheckLapackInfo("trtri",info);
+  ierr = PetscFPTrapPop();CHKERRQ(ierr);
   ierr = PetscLogFlops(0.33*n*n*n);CHKERRQ(ierr);
 
   /* Zero out entries below the diagonal */
@@ -153,7 +217,7 @@ PetscErrorCode BVMatTriInv_LAPACK_Private(BV bv,Mat R,Mat S)
   PetscErrorCode ierr;
   PetscInt       i,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS;
-  PetscBLASInt   info,n_,m_,ld_,lds_;
+  PetscBLASInt   info,n_,m_ = 0,ld_,lds_;
 
   PetscFunctionBegin;
   l = bv->l;
@@ -178,6 +242,7 @@ PetscErrorCode BVMatTriInv_LAPACK_Private(BV bv,Mat R,Mat S)
   }
 
   /* compute S = inv(R) */
+  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
   if (S==R) {
     PetscStackCallBLAS("LAPACKtrtri",LAPACKtrtri_("U","N",&n_,pR+l*ld+l,&ld_,&info));
   } else {
@@ -188,6 +253,7 @@ PetscErrorCode BVMatTriInv_LAPACK_Private(BV bv,Mat R,Mat S)
     PetscStackCallBLAS("LAPACKtrtri",LAPACKtrtri_("U","N",&n_,pS+l*lds+l,&lds_,&info));
   }
   SlepcCheckLapackInfo("trtri",info);
+  ierr = PetscFPTrapPop();CHKERRQ(ierr);
   ierr = PetscLogFlops(0.33*n*n*n);CHKERRQ(ierr);
 
   ierr = MatDenseRestoreArray(R,&pR);CHKERRQ(ierr);
@@ -208,7 +274,7 @@ PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
   PetscInt       i,j,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS,*D,*work,a;
   PetscReal      *eig,dummy;
-  PetscBLASInt   info,lwork,n_,m_,ld_,lds_;
+  PetscBLASInt   info,lwork,n_,m_ = 0,ld_,lds_;
 #if defined(PETSC_USE_COMPLEX)
   PetscReal      *rwork,rdummy;
 #endif
@@ -233,6 +299,7 @@ PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
     ierr = MatGetSize(S,&lds,NULL);CHKERRQ(ierr);
     ierr = PetscBLASIntCast(lds,&lds_);CHKERRQ(ierr);
   }
+  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
 
   /* workspace query and memory allocation */
   lwork = -1;
@@ -242,7 +309,7 @@ PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
   ierr = PetscMalloc4(n,&eig,n,&D,lwork,&work,PetscMax(1,3*n-2),&rwork);CHKERRQ(ierr);
 #else
   PetscStackCallBLAS("LAPACKsyev",LAPACKsyev_("V","L",&n_,pS,&lds_,&dummy,&a,&lwork,&info));
-  ierr = PetscBLASIntCast((PetscInt)PetscRealPart(a),&lwork);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast((PetscInt)a,&lwork);CHKERRQ(ierr);
   ierr = PetscMalloc3(n,&eig,n,&D,lwork,&work);CHKERRQ(ierr);
 #endif
 
@@ -278,6 +345,7 @@ PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
   ierr = PetscFree3(eig,D,work);CHKERRQ(ierr);
 #endif
   ierr = PetscLogFlops(9.0*n*n*n);CHKERRQ(ierr);
+  ierr = PetscFPTrapPop();CHKERRQ(ierr);
 
   ierr = MatDenseRestoreArray(R,&pR);CHKERRQ(ierr);
   if (S!=R) { ierr = MatDenseRestoreArray(S,&pS);CHKERRQ(ierr); }
@@ -291,7 +359,7 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscSc
 {
   PetscErrorCode ierr;
   PetscInt       level,plevel,nlevels,powtwo,lda,worklen;
-  PetscBLASInt   m,n,i,j,k,l,s,nb,sz,lwork,info;
+  PetscBLASInt   m,n,i,j,k,l,s = 0,nb,sz,lwork,info;
   PetscScalar    *tau,*work,*A=NULL,*QQ=NULL,*Qhalf,*C=NULL,one=1.0,zero=0.0;
   PetscMPIInt    rank,size,count,stride;
   MPI_Datatype   tmat;
@@ -303,8 +371,8 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscSc
   k  = PetscMin(m,n);
   nb = 16;
   lda = 2*n;
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)bv),&size);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)bv),&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)bv),&size);CHKERRMPI(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)bv),&rank);CHKERRMPI(ierr);
   nlevels = (PetscInt)PetscCeilReal(PetscLog2Real((PetscReal)size));
   powtwo  = PetscPowInt(2,(PetscInt)PetscFloorReal(PetscLog2Real((PetscReal)size)));
   worklen = n+n*nb;
@@ -346,8 +414,8 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscSc
     ierr = PetscMPIIntCast(n,&count);CHKERRQ(ierr);
     ierr = PetscMPIIntCast(lda,&stride);CHKERRQ(ierr);
     ierr = PetscBLASIntCast(lda,&l);CHKERRQ(ierr);
-    ierr = MPI_Type_vector(count,count,stride,MPIU_SCALAR,&tmat);CHKERRQ(ierr);
-    ierr = MPI_Type_commit(&tmat);CHKERRQ(ierr);
+    ierr = MPI_Type_vector(count,count,stride,MPIU_SCALAR,&tmat);CHKERRMPI(ierr);
+    ierr = MPI_Type_commit(&tmat);CHKERRMPI(ierr);
 
     for (level=nlevels;level>=1;level--) {
 
@@ -356,16 +424,16 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscSc
 
       /* Stack triangular matrices */
       if (rank<s && s<size) {  /* send top part, receive bottom part */
-        ierr = MPI_Sendrecv(A,1,tmat,s,111,A+n,1,tmat,s,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRQ(ierr);
+        ierr = MPI_Sendrecv(A,1,tmat,s,111,A+n,1,tmat,s,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRMPI(ierr);
       } else if (s<size) {  /* copy top to bottom, receive top part */
-        ierr = MPI_Sendrecv(A,1,tmat,rank,111,A+n,1,tmat,rank,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRQ(ierr);
-        ierr = MPI_Sendrecv(A+n,1,tmat,s,111,A,1,tmat,s,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRQ(ierr);
+        ierr = MPI_Sendrecv(A,1,tmat,rank,111,A+n,1,tmat,rank,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRMPI(ierr);
+        ierr = MPI_Sendrecv(A+n,1,tmat,s,111,A,1,tmat,s,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRMPI(ierr);
       }
       if (level<nlevels && size!=powtwo) {  /* for cases when size is not a power of 2 */
         if (rank<size-powtwo) {  /* send bottom part */
-          ierr = MPI_Send(A+n,1,tmat,rank+powtwo,111,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+          ierr = MPI_Send(A+n,1,tmat,rank+powtwo,111,PetscObjectComm((PetscObject)bv));CHKERRMPI(ierr);
         } else if (rank>=powtwo) {  /* receive bottom part */
-          ierr = MPI_Recv(A+n,1,tmat,rank-powtwo,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRQ(ierr);
+          ierr = MPI_Recv(A+n,1,tmat,rank-powtwo,111,PetscObjectComm((PetscObject)bv),MPI_STATUS_IGNORE);CHKERRMPI(ierr);
         }
       }
       /* Compute QR and build orthogonal matrix */
@@ -413,7 +481,7 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscSc
       }
     }
 
-    ierr = MPI_Type_free(&tmat);CHKERRQ(ierr);
+    ierr = MPI_Type_free(&tmat);CHKERRMPI(ierr);
   }
 
   ierr = PetscLogFlops(3.0*m*n*n);CHKERRQ(ierr);
@@ -466,7 +534,7 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR_OnlyR(BV bv,PetscInt m_,PetscInt n_,P
   ierr = PetscBLASIntCast(n_,&n);CHKERRQ(ierr);
   nb = 16;
   s  = n+n*(n-1)/2;  /* length of packed triangular matrix */
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)bv),&size);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)bv),&size);CHKERRMPI(ierr);
   worklen = n+n*nb+2*s+m*n;
   ierr = BVAllocateWork_Private(bv,worklen);CHKERRQ(ierr);
   tau  = bv->work;
@@ -490,22 +558,17 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR_OnlyR(BV bv,PetscInt m_,PetscInt n_,P
   } else {
     /* Use MPI reduction operation to obtain global R */
     ierr = PetscMPIIntCast(s,&count);CHKERRQ(ierr);
-    ierr = MPI_Type_contiguous(count,MPIU_SCALAR,&tmat);CHKERRQ(ierr);
-    ierr = MPI_Type_commit(&tmat);CHKERRQ(ierr);
+    ierr = MPI_Type_contiguous(count,MPIU_SCALAR,&tmat);CHKERRMPI(ierr);
+    ierr = MPI_Type_commit(&tmat);CHKERRMPI(ierr);
     for (i=0;i<n;i++) {
       for (j=i;j<n;j++) R1[(2*n-i-1)*i/2+j] = (i<m)?A[i+j*m]:0.0;
     }
-#if defined(PETSC_HAVE_I_MPI_NUMVERSION)
-    ierr = MPI_Reduce(R1,R2,1,tmat,MPIU_TSQR,0,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-    ierr = MPI_Bcast(R2,1,tmat,0,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-#else
-    ierr = MPI_Allreduce(R1,R2,1,tmat,MPIU_TSQR,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-#endif
+    ierr = MPIU_Allreduce(R1,R2,1,tmat,MPIU_TSQR,PetscObjectComm((PetscObject)bv));CHKERRMPI(ierr);
     for (i=0;i<n;i++) {
       for (j=0;j<i;j++) R[i+j*ldr] = 0.0;
       for (j=i;j<n;j++) R[i+j*ldr] = R2[(2*n-i-1)*i/2+j];
     }
-    ierr = MPI_Type_free(&tmat);CHKERRQ(ierr);
+    ierr = MPI_Type_free(&tmat);CHKERRMPI(ierr);
   }
 
   ierr = PetscLogFlops(3.0*m*n*n);CHKERRQ(ierr);
