@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,142 +12,6 @@
 */
 
 #include <slepc/private/bvimpl.h>          /*I   "slepcbv.h"   I*/
-#include <slepcblaslapack.h>
-
-/*
-   BV_CleanCoefficients_Default - Sets to zero all entries of column j of the bv buffer
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_CleanCoefficients_Default(BV bv,PetscInt j,PetscScalar *h)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *hh=h,*a;
-  PetscInt       i;
-
-  PetscFunctionBegin;
-  if (!h) {
-    ierr = VecGetArray(bv->buffer,&a);CHKERRQ(ierr);
-    hh = a + j*(bv->nc+bv->m);
-  }
-  for (i=0;i<bv->nc+j;i++) hh[i] = 0.0;
-  if (!h) { ierr = VecRestoreArray(bv->buffer,&a);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-/*
-   BV_AddCoefficients_Default - Add the contents of the scratch (0-th column) of the bv buffer
-   into column j of the bv buffer
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_AddCoefficients_Default(BV bv,PetscInt j,PetscScalar *h,PetscScalar *c)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *hh=h,*cc=c;
-  PetscInt       i;
-
-  PetscFunctionBegin;
-  if (!h) {
-    ierr = VecGetArray(bv->buffer,&cc);CHKERRQ(ierr);
-    hh = cc + j*(bv->nc+bv->m);
-  }
-  for (i=0;i<bv->nc+j;i++) hh[i] += cc[i];
-  if (!h) { ierr = VecRestoreArray(bv->buffer,&cc);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-/*
-   BV_SetValue_Default - Sets value in row j (counted after the constraints) of column k
-   of the coefficients array
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_SetValue_Default(BV bv,PetscInt j,PetscInt k,PetscScalar *h,PetscScalar value)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *hh=h,*a;
-
-  PetscFunctionBegin;
-  if (!h) {
-    ierr = VecGetArray(bv->buffer,&a);CHKERRQ(ierr);
-    hh = a + k*(bv->nc+bv->m);
-  }
-  hh[bv->nc+j] = value;
-  if (!h) { ierr = VecRestoreArray(bv->buffer,&a);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-/*
-   BV_SquareSum_Default - Returns the value h'*h, where h represents the contents of the
-   coefficients array (up to position j)
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_SquareSum_Default(BV bv,PetscInt j,PetscScalar *h,PetscReal *sum)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *hh=h;
-  PetscInt       i;
-
-  PetscFunctionBegin;
-  *sum = 0.0;
-  if (!h) { ierr = VecGetArray(bv->buffer,&hh);CHKERRQ(ierr); }
-  for (i=0;i<bv->nc+j;i++) *sum += PetscRealPart(hh[i]*PetscConj(hh[i]));
-  if (!h) { ierr = VecRestoreArray(bv->buffer,&hh);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-/*
-   BV_ApplySignature_Default - Computes the pointwise product h*omega, where h represents
-   the contents of the coefficients array (up to position j) and omega is the signature;
-   if inverse=TRUE then the operation is h/omega
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_ApplySignature_Default(BV bv,PetscInt j,PetscScalar *h,PetscBool inverse)
-{
-  PetscErrorCode    ierr;
-  PetscScalar       *hh=h;
-  PetscInt          i;
-  const PetscScalar *omega;
-
-  PetscFunctionBegin;
-  if (!(bv->nc+j)) PetscFunctionReturn(0);
-  if (!h) { ierr = VecGetArray(bv->buffer,&hh);CHKERRQ(ierr); }
-  ierr = VecGetArrayRead(bv->omega,&omega);CHKERRQ(ierr);
-  if (inverse) for (i=0;i<bv->nc+j;i++) hh[i] /= PetscRealPart(omega[i]);
-  else for (i=0;i<bv->nc+j;i++) hh[i] *= PetscRealPart(omega[i]);
-  ierr = VecRestoreArrayRead(bv->omega,&omega);CHKERRQ(ierr);
-  if (!h) { ierr = VecRestoreArray(bv->buffer,&hh);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-/*
-   BV_SquareRoot_Default - Returns the square root of position j (counted after the constraints)
-   of the coefficients array
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_SquareRoot_Default(BV bv,PetscInt j,PetscScalar *h,PetscReal *beta)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *hh=h;
-
-  PetscFunctionBegin;
-  if (!h) { ierr = VecGetArray(bv->buffer,&hh);CHKERRQ(ierr); }
-  ierr = BV_SafeSqrt(bv,hh[bv->nc+j],beta);CHKERRQ(ierr);
-  if (!h) { ierr = VecRestoreArray(bv->buffer,&hh);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
-
-/*
-   BV_StoreCoefficients_Default - Copy the contents of the coefficients array to an array dest
-   provided by the caller (only values from l to j are copied)
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_StoreCoefficients_Default(BV bv,PetscInt j,PetscScalar *h,PetscScalar *dest)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *hh=h,*a;
-  PetscInt       i;
-
-  PetscFunctionBegin;
-  if (!h) {
-    ierr = VecGetArray(bv->buffer,&a);CHKERRQ(ierr);
-    hh = a + j*(bv->nc+bv->m);
-  }
-  for (i=bv->l;i<j;i++) dest[i-bv->l] = hh[bv->nc+i];
-  if (!h) { ierr = VecRestoreArray(bv->buffer,&a);CHKERRQ(ierr); }
-  PetscFunctionReturn(0);
-}
 
 /*
    BV_NormVecOrColumn - Compute the 2-norm of the working vector, irrespective of
@@ -185,25 +49,6 @@ PETSC_STATIC_INLINE PetscErrorCode BVDotColumnInc(BV X,PetscInt j,PetscScalar *q
   PetscFunctionReturn(0);
 }
 
-
-#if defined(PETSC_HAVE_VECCUDA)
-#define BV_CleanCoefficients(a,b,c)   ((a)->cuda?BV_CleanCoefficients_CUDA:BV_CleanCoefficients_Default)((a),(b),(c))
-#define BV_AddCoefficients(a,b,c,d)   ((a)->cuda?BV_AddCoefficients_CUDA:BV_AddCoefficients_Default)((a),(b),(c),(d))
-#define BV_SetValue(a,b,c,d,e)        ((a)->cuda?BV_SetValue_CUDA:BV_SetValue_Default)((a),(b),(c),(d),(e))
-#define BV_SquareSum(a,b,c,d)         ((a)->cuda?BV_SquareSum_CUDA:BV_SquareSum_Default)((a),(b),(c),(d))
-#define BV_ApplySignature(a,b,c,d)    ((a)->cuda?BV_ApplySignature_CUDA:BV_ApplySignature_Default)((a),(b),(c),(d))
-#define BV_SquareRoot(a,b,c,d)        ((a)->cuda?BV_SquareRoot_CUDA:BV_SquareRoot_Default)((a),(b),(c),(d))
-#define BV_StoreCoefficients(a,b,c,d) ((a)->cuda?BV_StoreCoefficients_CUDA:BV_StoreCoefficients_Default)((a),(b),(c),(d))
-#else
-#define BV_CleanCoefficients(a,b,c)   BV_CleanCoefficients_Default((a),(b),(c))
-#define BV_AddCoefficients(a,b,c,d)   BV_AddCoefficients_Default((a),(b),(c),(d))
-#define BV_SetValue(a,b,c,d,e)        BV_SetValue_Default((a),(b),(c),(d),(e))
-#define BV_SquareSum(a,b,c,d)         BV_SquareSum_Default((a),(b),(c),(d))
-#define BV_ApplySignature(a,b,c,d)    BV_ApplySignature_Default((a),(b),(c),(d))
-#define BV_SquareRoot(a,b,c,d)        BV_SquareRoot_Default((a),(b),(c),(d))
-#define BV_StoreCoefficients(a,b,c,d) BV_StoreCoefficients_Default((a),(b),(c),(d))
-#endif /* PETSC_HAVE_VECCUDA */
-
 /*
    BVOrthogonalizeMGS1 - Compute one step of Modified Gram-Schmidt
 */
@@ -225,7 +70,7 @@ static PetscErrorCode BVOrthogonalizeMGS1(BV bv,PetscInt j,Vec v,PetscBool *whic
   for (i=-bv->nc;i<j;i++) {
     if (which && i>=0 && !which[i]) continue;
     ierr = BVGetColumn(bv,i,&vi);CHKERRQ(ierr);
-    /* h_i = ( v, v_i ) */
+    /* h_i = (v, v_i) */
     if (bv->matrix) {
       ierr = BV_IPMatMult(bv,w);CHKERRQ(ierr);
       z = bv->Bx;
@@ -296,7 +141,7 @@ static PetscErrorCode BVOrthogonalizeCGS1(BV bv,PetscInt j,Vec v,PetscBool *whic
   PetscFunctionReturn(0);
 }
 
-#define BVOrthogonalizeGS1(a,b,c,d,e,f,g,h) (mgs?BVOrthogonalizeMGS1:BVOrthogonalizeCGS1)(a,b,c,d,e,f,g,h)
+#define BVOrthogonalizeGS1(a,b,c,d,e,f,g,h) ((bv->ops->gramschmidt)?(*bv->ops->gramschmidt):(mgs?BVOrthogonalizeMGS1:BVOrthogonalizeCGS1))(a,b,c,d,e,f,g,h)
 
 /*
    BVOrthogonalizeGS - Orthogonalize with (classical or modified) Gram-Schmidt
@@ -388,7 +233,7 @@ static PetscErrorCode BVOrthogonalizeGS(BV bv,PetscInt j,Vec v,PetscBool *which,
    BVOrthogonalizeVec - Orthogonalize a given vector with respect to all
    active columns.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv     - the basis vectors context
@@ -445,7 +290,7 @@ PetscErrorCode BVOrthogonalizeVec(BV bv,Vec v,PetscScalar *H,PetscReal *norm,Pet
    BVOrthogonalizeColumn - Orthogonalize one of the column vectors with respect to
    the previous ones.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv     - the basis vectors context
@@ -459,7 +304,7 @@ PetscErrorCode BVOrthogonalizeVec(BV bv,Vec v,PetscScalar *H,PetscReal *norm,Pet
 
    Notes:
    This function applies an orthogonal projector to project vector V[j] onto
-   the orthogonal complement of the span of the columns of V[0..j-1],
+   the orthogonal complement of the span of the columns V[0..j-1],
    where V[.] are the vectors of BV. The columns V[0..j-1] are assumed to be
    mutually orthonormal.
 
@@ -514,7 +359,7 @@ PetscErrorCode BVOrthogonalizeColumn(BV bv,PetscInt j,PetscScalar *H,PetscReal *
    the previous ones. This is equivalent to a call to BVOrthogonalizeColumn()
    followed by a call to BVScaleColumn() with the reciprocal of the norm.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv      - the basis vectors context
@@ -536,7 +381,7 @@ PetscErrorCode BVOrthogonalizeColumn(BV bv,PetscInt j,PetscScalar *H,PetscReal *
    because norm=0. In that case, it could be left as zero or replaced by a random
    vector that is then orthonormalized. The latter is achieved by setting the
    argument replace to TRUE. The vector will be replaced by a random vector also
-   if lindep was set to TRUE, even if the norm is not exaclty zero.
+   if lindep was set to TRUE, even if the norm is not exactly zero.
 
    If the vector has been replaced by a random vector, the output arguments norm and
    lindep will be set according to the orthogonalization of this new vector.
@@ -601,7 +446,7 @@ PetscErrorCode BVOrthonormalizeColumn(BV bv,PetscInt j,PetscBool replace,PetscRe
    BVOrthogonalizeSomeColumn - Orthogonalize one of the column vectors with
    respect to some of the previous ones.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv     - the basis vectors context
@@ -636,7 +481,7 @@ PetscErrorCode BVOrthogonalizeSomeColumn(BV bv,PetscInt j,PetscBool *which,Petsc
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(bv,j,2);
-  PetscValidPointer(which,3);
+  PetscValidBoolPointer(which,3);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
   if (j<0) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_OUTOFRANGE,"Index j must be non-negative");
@@ -658,6 +503,22 @@ PetscErrorCode BVOrthogonalizeSomeColumn(BV bv,PetscInt j,PetscBool *which,Petsc
   PetscFunctionReturn(0);
 }
 
+/*
+   Block Gram-Schmidt: V2 = V2 - V1*R12, where R12 = V1'*V2
+ */
+static PetscErrorCode BVOrthogonalize_BlockGS(BV V,Mat R)
+{
+  PetscErrorCode ierr;
+  BV             V1;
+
+  PetscFunctionBegin;
+  ierr = BVGetSplit(V,&V1,NULL);CHKERRQ(ierr);
+  ierr = BVDot(V,V1,R);CHKERRQ(ierr);
+  ierr = BVMult(V,-1.0,1.0,V1,R);CHKERRQ(ierr);
+  ierr = BVRestoreSplit(V,&V1,NULL);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*
    Orthogonalize a set of vectors with Gram-Schmidt, column by column.
  */
@@ -666,7 +527,7 @@ static PetscErrorCode BVOrthogonalize_GS(BV V,Mat R)
   PetscErrorCode ierr;
   PetscScalar    *r=NULL;
   PetscReal      norm;
-  PetscInt       j,ldr;
+  PetscInt       j,ldr,lsave;
   Vec            v,w;
 
   PetscFunctionBegin;
@@ -687,12 +548,16 @@ static PetscErrorCode BVOrthogonalize_GS(BV V,Mat R)
       ierr = BVRestoreColumn(V->cached,j,&v);CHKERRQ(ierr);
     }
     if (R) {
-      ierr = BVOrthogonalizeColumn(V,j,r+j*ldr+V->l,&norm,NULL);CHKERRQ(ierr);
+      ierr = BVOrthogonalizeColumn(V,j,NULL,&norm,NULL);CHKERRQ(ierr);
+      lsave = V->l;
+      V->l = -V->nc;
+      ierr = BV_StoreCoefficients(V,j,NULL,r+j*ldr);CHKERRQ(ierr);
+      V->l = lsave;
       r[j+j*ldr] = norm;
     } else {
       ierr = BVOrthogonalizeColumn(V,j,NULL,&norm,NULL);CHKERRQ(ierr);
     }
-    if (!norm) SETERRQ(PetscObjectComm((PetscObject)V),1,"Breakdown in BVOrthogonalize due to a linearly dependent column");
+    if (!norm) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_CONV_FAILED,"Breakdown in BVOrthogonalize due to a linearly dependent column");
     if (V->matrix && V->orthog_type==BV_ORTHOG_CGS) {  /* fill cached BV */
       ierr = BVGetColumn(V->cached,j,&v);CHKERRQ(ierr);
       ierr = VecCopy(V->Bx,v);CHKERRQ(ierr);
@@ -705,68 +570,49 @@ static PetscErrorCode BVOrthogonalize_GS(BV V,Mat R)
 }
 
 /*
-   Compute the upper Cholesky factor in R and its inverse in S.
- */
-static PetscErrorCode MatCholeskyFactorInvert(Mat R,PetscInt l,Mat *S)
+  BV_GetBufferMat - Create auxiliary seqdense matrix that wraps the bv->buffer.
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_GetBufferMat(BV bv)
 {
-#if defined(PETSC_MISSING_LAPACK_POTRF) || defined(SLEPC_MISSING_LAPACK_TRTRI)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"POTRF/TRTRI - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
-  PetscInt       i,n,m,ld;
-  PetscScalar    *pR,*pS;
-  PetscBLASInt   info,n_,l_,m_,ld_;
+  PetscInt       ld;
+  PetscScalar    *array;
 
   PetscFunctionBegin;
-  ierr = MatGetSize(R,&m,NULL);CHKERRQ(ierr);
-  n = m-l;
-  ierr = PetscBLASIntCast(m,&m_);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(l,&l_);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(n,&n_);CHKERRQ(ierr);
-  ld  = m;
-  ld_ = m_;
-  ierr = MatCreateSeqDense(PETSC_COMM_SELF,ld,ld,NULL,S);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(R,&pR);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(*S,&pS);CHKERRQ(ierr);
-
-  /* save a copy of matrix in S */
-  for (i=l;i<m;i++) {
-    ierr = PetscMemcpy(pS+i*ld+l,pR+i*ld+l,n*sizeof(PetscScalar));CHKERRQ(ierr);
+  if (!bv->Abuffer) {
+    if (!bv->buffer) { ierr = BVGetBufferVec(bv,&bv->buffer);CHKERRQ(ierr); }
+    ld = bv->m+bv->nc;
+    ierr = VecGetArray(bv->buffer,&array);CHKERRQ(ierr);
+    ierr = MatCreateSeqDense(PETSC_COMM_SELF,ld,bv->m,array,&bv->Abuffer);CHKERRQ(ierr);
+    ierr = VecRestoreArray(bv->buffer,&array);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->Abuffer);CHKERRQ(ierr);
   }
-
-  /* compute upper Cholesky factor in R */
-  PetscStackCallBLAS("LAPACKpotrf",LAPACKpotrf_("U",&n_,pR+l*ld+l,&ld_,&info));
-  ierr = PetscLogFlops((1.0*n*n*n)/3.0);CHKERRQ(ierr);
-
-  if (info) {  /* LAPACKpotrf failed, retry on diagonally perturbed matrix */
-    for (i=l;i<m;i++) {
-      ierr = PetscMemcpy(pR+i*ld+l,pS+i*ld+l,n*sizeof(PetscScalar));CHKERRQ(ierr);
-      pR[i+i*ld] += 50.0*PETSC_MACHINE_EPSILON;
-    }
-    PetscStackCallBLAS("LAPACKpotrf",LAPACKpotrf_("U",&n_,pR+l*ld+l,&ld_,&info));
-    SlepcCheckLapackInfo("potrf",info);
-    ierr = PetscLogFlops((1.0*n*n*n)/3.0);CHKERRQ(ierr);
-  }
-
-  /* compute S = inv(R) */
-  ierr = PetscMemzero(pS,m*m*sizeof(PetscScalar));CHKERRQ(ierr);
-  for (i=l;i<m;i++) {
-    ierr = PetscMemcpy(pS+i*ld+l,pR+i*ld+l,n*sizeof(PetscScalar));CHKERRQ(ierr);
-  }
-  PetscStackCallBLAS("LAPACKtrtri",LAPACKtrtri_("U","N",&n_,pS+l*ld+l,&ld_,&info));
-  SlepcCheckLapackInfo("trtri",info);
-  ierr = PetscLogFlops(1.0*n*n*n);CHKERRQ(ierr);
-
-  /* Zero out entries below the diagonal */
-  for (i=l;i<m-1;i++) {
-    ierr = PetscMemzero(pR+i*ld+i+1,(m-i-1)*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = PetscMemzero(pS+i*ld+i+1,(m-i-1)*sizeof(PetscScalar));CHKERRQ(ierr);
-  }
-  ierr = MatDenseRestoreArray(R,&pR);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(*S,&pS);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
+}
+
+/*
+   BV_StoreCoeffsBlock_Default - Copy the contents of the BV buffer to a dense Mat
+   provided by the caller. Only columns l:k-1 are copied, restricting to the upper
+   triangular part if tri=PETSC_TRUE.
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_StoreCoeffsBlock_Default(BV bv,Mat R,PetscBool tri)
+{
+  PetscErrorCode    ierr;
+  const PetscScalar *bb;
+  PetscScalar       *rr;
+  PetscInt          j,ldr,ldb;
+
+  PetscFunctionBegin;
+  ierr = MatGetSize(R,&ldr,NULL);CHKERRQ(ierr);
+  ierr = MatDenseGetArray(R,&rr);CHKERRQ(ierr);
+  ldb  = bv->m+bv->nc;
+  ierr = VecGetArrayRead(bv->buffer,&bb);CHKERRQ(ierr);
+  for (j=bv->l;j<bv->k;j++) {
+    ierr = PetscArraycpy(rr+j*ldr,bb+j*ldb,(tri?(j+1):bv->k)+bv->nc);CHKERRQ(ierr);
+  }
+  ierr = VecRestoreArrayRead(bv->buffer,&bb);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArray(R,&rr);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
 }
 
 /*
@@ -775,65 +621,127 @@ static PetscErrorCode MatCholeskyFactorInvert(Mat R,PetscInt l,Mat *S)
 static PetscErrorCode BVOrthogonalize_Chol(BV V,Mat Rin)
 {
   PetscErrorCode ierr;
-  Mat            S,R=Rin;
+  Mat            R,S;
 
   PetscFunctionBegin;
-  if (!Rin) {
-    ierr = MatCreateSeqDense(PETSC_COMM_SELF,V->k,V->k,NULL,&R);CHKERRQ(ierr);
-  }
+  ierr = BV_GetBufferMat(V);CHKERRQ(ierr);
+  R = V->Abuffer;
+  if (Rin) S = Rin;   /* use Rin as a workspace for S */
+  else S = R;
+  if (V->l) { ierr = BVOrthogonalize_BlockGS(V,R);CHKERRQ(ierr); }
   ierr = BVDot(V,V,R);CHKERRQ(ierr);
-  ierr = MatCholeskyFactorInvert(R,V->l,&S);CHKERRQ(ierr);
+  ierr = BVMatCholInv_LAPACK_Private(V,R,S);CHKERRQ(ierr);
   ierr = BVMultInPlace(V,S,V->l,V->k);CHKERRQ(ierr);
-  ierr = MatDestroy(&S);CHKERRQ(ierr);
-  if (!Rin) {
-    ierr = MatDestroy(&R);CHKERRQ(ierr);
-  }
+  if (Rin) { ierr = BV_StoreCoeffsBlock_Default(V,Rin,PETSC_TRUE);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
 /*
    Orthogonalize a set of vectors with the Tall-Skinny QR method
  */
-static PetscErrorCode BVOrthogonalize_TSQR(BV V,Mat R)
+static PetscErrorCode BVOrthogonalize_TSQR(BV V,Mat Rin)
 {
   PetscErrorCode ierr;
   PetscScalar    *pv,*r=NULL;
+  PetscInt       ldr;
+  Mat            R;
 
   PetscFunctionBegin;
-  if (R) { ierr = MatDenseGetArray(R,&r);CHKERRQ(ierr); }
+  ierr = BV_GetBufferMat(V);CHKERRQ(ierr);
+  R = V->Abuffer;
+  if (V->l) { ierr = BVOrthogonalize_BlockGS(V,R);CHKERRQ(ierr); }
+  ierr = MatGetSize(R,&ldr,NULL);CHKERRQ(ierr);
+  ierr = MatDenseGetArray(R,&r);CHKERRQ(ierr);
   ierr = BVGetArray(V,&pv);CHKERRQ(ierr);
-  ierr = BVOrthogonalize_LAPACK_Private(V,V->n,V->k,pv+V->nc*V->n,r);CHKERRQ(ierr);
+  ierr = BVOrthogonalize_LAPACK_TSQR(V,V->n,V->k-V->l,pv+(V->nc+V->l)*V->n,r+V->l*ldr+V->l,ldr);CHKERRQ(ierr);
   ierr = BVRestoreArray(V,&pv);CHKERRQ(ierr);
-  if (R) { ierr = MatDenseRestoreArray(R,&r);CHKERRQ(ierr); }
+  ierr = MatDenseRestoreArray(R,&r);CHKERRQ(ierr);
+  if (Rin) { ierr = BV_StoreCoeffsBlock_Default(V,Rin,PETSC_TRUE);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   Orthogonalize a set of vectors with TSQR, but computing R only, then doing Q=V*inv(R)
+ */
+static PetscErrorCode BVOrthogonalize_TSQRCHOL(BV V,Mat Rin)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *pv,*r=NULL;
+  PetscInt       ldr;
+  Mat            R,S;
+
+  PetscFunctionBegin;
+  ierr = BV_GetBufferMat(V);CHKERRQ(ierr);
+  R = V->Abuffer;
+  if (Rin) S = Rin;   /* use Rin as a workspace for S */
+  else S = R;
+  if (V->l) { ierr = BVOrthogonalize_BlockGS(V,R);CHKERRQ(ierr); }
+  ierr = MatGetSize(R,&ldr,NULL);CHKERRQ(ierr);
+  ierr = MatDenseGetArray(R,&r);CHKERRQ(ierr);
+  ierr = BVGetArray(V,&pv);CHKERRQ(ierr);
+  ierr = BVOrthogonalize_LAPACK_TSQR_OnlyR(V,V->n,V->k-V->l,pv+(V->nc+V->l)*V->n,r+V->l*ldr+V->l,ldr);CHKERRQ(ierr);
+  ierr = BVRestoreArray(V,&pv);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArray(R,&r);CHKERRQ(ierr);
+  ierr = BVMatTriInv_LAPACK_Private(V,R,S);CHKERRQ(ierr);
+  ierr = BVMultInPlace(V,S,V->l,V->k);CHKERRQ(ierr);
+  if (Rin) { ierr = BV_StoreCoeffsBlock_Default(V,Rin,PETSC_TRUE);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   Orthogonalize a set of vectors with SVQB
+ */
+static PetscErrorCode BVOrthogonalize_SVQB(BV V,Mat Rin)
+{
+  PetscErrorCode ierr;
+  Mat            R,S;
+
+  PetscFunctionBegin;
+  ierr = BV_GetBufferMat(V);CHKERRQ(ierr);
+  R = V->Abuffer;
+  if (Rin) S = Rin;   /* use Rin as a workspace for S */
+  else S = R;
+  if (V->l) { ierr = BVOrthogonalize_BlockGS(V,R);CHKERRQ(ierr); }
+  ierr = BVDot(V,V,R);CHKERRQ(ierr);
+  ierr = BVMatSVQB_LAPACK_Private(V,R,S);CHKERRQ(ierr);
+  ierr = BVMultInPlace(V,S,V->l,V->k);CHKERRQ(ierr);
+  if (Rin) { ierr = BV_StoreCoeffsBlock_Default(V,Rin,PETSC_FALSE);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
 /*@
-   BVOrthogonalize - Orthogonalize all columns (except leading ones), that is,
-   compute the QR decomposition.
+   BVOrthogonalize - Orthogonalize all columns (starting from the leading ones),
+   that is, compute the QR decomposition.
 
-   Collective on BV
+   Collective on V
 
-   Input Parameter:
-.  V - basis vectors
-
-   Output Parameters:
-+  V - the modified basis vectors
--  R - a sequential dense matrix (or NULL)
+   Input/Outpu Parameters:
++  V - basis vectors to be orthogonalized (or B-orthogonalized)
+-  R - a sequential dense matrix (or NULL), on output the triangular factor of
+       the QR decomposition
 
    Notes:
-   On input, matrix R must be a sequential dense Mat, with at least as many rows
-   and columns as the number of active columns of V. The output satisfies
-   V0 = V*R (where V0 represent the input V) and V'*V = I.
+   On input, matrix R must be a square sequential dense Mat, with at least as many
+   rows and columns as the number of active columns of V. The output satisfies
+   V0 = V*R (where V0 represent the input V) and V'*V = I (or V'*B*V = I if an
+   inner product matrix B has been specified with BVSetMatrix()).
 
    If V has leading columns, then they are not modified (are assumed to be already
-   orthonormal) and the corresponding part of R is not referenced.
+   orthonormal) and the leading columns of R are not referenced. Let the
+   decomposition be
+.vb
+   [ V01 V02 ] = [ V1 V2 ] [ R11 R12 ]
+                           [  0  R22 ]
+.ve
+   then V1 is left unchanged (equal to V01) as well as R11 (it should satisfy
+   V01 = V1*R11).
 
    Can pass NULL if R is not required.
 
    The method to be used for block orthogonalization can be set with
    BVSetOrthogonalization(). If set to GS, the computation is done column by
-   column with successive calls to BVOrthogonalizeColumn().
+   column with successive calls to BVOrthogonalizeColumn(). Note that in the
+   SVQB method the R factor is not upper triangular.
 
    If V is rank-deficient or very ill-conditioned, that is, one or more columns are
    (almost) linearly dependent with respect to the rest, then the algorithm may
@@ -843,12 +751,11 @@ static PetscErrorCode BVOrthogonalize_TSQR(BV V,Mat R)
 
    Level: intermediate
 
-.seealso: BVOrthogonalizeColumn(), BVOrthogonalizeVec(), BVSetActiveColumns(), BVSetOrthogonalization(), BVOrthogBlockType
+.seealso: BVOrthogonalizeColumn(), BVOrthogonalizeVec(), BVSetMatrix(), BVSetActiveColumns(), BVSetOrthogonalization(), BVOrthogBlockType
 @*/
 PetscErrorCode BVOrthogonalize(BV V,Mat R)
 {
   PetscErrorCode ierr;
-  PetscBool      match;
   PetscInt       m,n;
 
   PetscFunctionBegin;
@@ -858,9 +765,7 @@ PetscErrorCode BVOrthogonalize(BV V,Mat R)
   if (R) {
     PetscValidHeaderSpecific(R,MAT_CLASSID,2);
     PetscValidType(R,2);
-    if (V->l>0 && V->orthog_block==BV_ORTHOG_BLOCK_GS) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_SUP,"Cannot request matrix R in Gram-Schmidt orthogonalization if l>0");
-    ierr = PetscObjectTypeCompare((PetscObject)R,MATSEQDENSE,&match);CHKERRQ(ierr);
-    if (!match) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_SUP,"Mat argument must be of type seqdense");
+    PetscCheckTypeName(R,MATSEQDENSE);
     ierr = MatGetSize(R,&m,&n);CHKERRQ(ierr);
     if (m!=n) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Mat argument is not square, it has %D rows and %D columns",m,n);
     if (n<V->k) SETERRQ2(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_SIZ,"Mat size %D is smaller than the number of BV active columns %D",n,V->k);
@@ -879,6 +784,13 @@ PetscErrorCode BVOrthogonalize(BV V,Mat R)
     if (V->matrix) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_SUP,"Orthogonalization method not available for non-standard inner product");
     ierr = BVOrthogonalize_TSQR(V,R);CHKERRQ(ierr);
     break;
+  case BV_ORTHOG_BLOCK_TSQRCHOL:
+    if (V->matrix) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_SUP,"Orthogonalization method not available for non-standard inner product");
+    ierr = BVOrthogonalize_TSQRCHOL(V,R);CHKERRQ(ierr);
+    break;
+  case BV_ORTHOG_BLOCK_SVQB:
+    ierr = BVOrthogonalize_SVQB(V,R);CHKERRQ(ierr);
+    break;
   }
   ierr = PetscLogEventEnd(BV_Orthogonalize,V,R,0,0);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)V);CHKERRQ(ierr);
