@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -20,8 +20,8 @@ PetscLogEvent     DS_Solve = 0,DS_Vectors = 0,DS_Synchronize = 0,DS_Other = 0;
 static PetscBool  DSPackageInitialized = PETSC_FALSE;
 
 const char *DSStateTypes[] = {"RAW","INTERMEDIATE","CONDENSED","TRUNCATED","DSStateType","DS_STATE_",0};
-const char *DSParallelTypes[] = {"REDUNDANT","SYNCHRONIZED","DSParallelType","DS_PARALLEL_",0};
-const char *DSMatName[DS_NUM_MAT] = {"A","B","C","T","D","Q","Z","X","Y","U","VT","W","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9"};
+const char *DSParallelTypes[] = {"REDUNDANT","SYNCHRONIZED","DISTRIBUTED","DSParallelType","DS_PARALLEL_",0};
+const char *DSMatName[DS_NUM_MAT] = {"A","B","C","T","D","Q","Z","X","Y","U","V","W","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9"};
 DSMatType  DSMatExtra[DS_NUM_EXTRA] = {DS_MAT_E0,DS_MAT_E1,DS_MAT_E2,DS_MAT_E3,DS_MAT_E4,DS_MAT_E5,DS_MAT_E6,DS_MAT_E7,DS_MAT_E8,DS_MAT_E9};
 
 /*@C
@@ -124,7 +124,6 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
   ds->ld            = 0;
   ds->l             = 0;
   ds->n             = 0;
-  ds->m             = 0;
   ds->k             = 0;
   ds->t             = 0;
   ds->bs            = 1;
@@ -243,7 +242,7 @@ PetscErrorCode DSGetOptionsPrefix(DS ds,const char *prefix[])
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds   - the direct solver context
 -  type - a known type
 
@@ -325,10 +324,11 @@ PetscErrorCode DSDuplicate(DS ds,DS *dsnew)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  PetscValidType(ds,1);
   PetscValidPointer(dsnew,2);
   ierr = DSCreate(PetscObjectComm((PetscObject)ds),dsnew);CHKERRQ(ierr);
-  ierr = DSSetType(*dsnew,((PetscObject)ds)->type_name);CHKERRQ(ierr);
+  if (((PetscObject)ds)->type_name) {
+    ierr = DSSetType(*dsnew,((PetscObject)ds)->type_name);CHKERRQ(ierr);
+  }
   (*dsnew)->method   = ds->method;
   (*dsnew)->compact  = ds->compact;
   (*dsnew)->refined  = ds->refined;
@@ -343,9 +343,9 @@ PetscErrorCode DSDuplicate(DS ds,DS *dsnew)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds   - the direct solver context
--  meth - an index indentifying the method
+-  meth - an index identifying the method
 
    Options Database Key:
 .  -ds_method <meth> - Sets the method
@@ -394,12 +394,13 @@ PetscErrorCode DSGetMethod(DS ds,PetscInt *meth)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds    - the direct solver context
 -  pmode - the parallel mode
 
    Options Database Key:
-.  -ds_parallel <mode> - Sets the parallel mode, either 'redundant' or 'synchronized'
+.  -ds_parallel <mode> - Sets the parallel mode, 'redundant', 'synchronized'
+   or 'distributed'
 
    Notes:
    In the 'redundant' parallel mode, all processes will make the computation
@@ -412,6 +413,10 @@ PetscErrorCode DSGetMethod(DS ds,PetscInt *meth)
    processes in the communicator. This communication is not done automatically,
    an explicit call to DSSynchronize() is required.
 
+   The 'distributed' parallel mode can be used in some DS types only, such
+   as the contour integral method of DSNEP. In this case, every MPI process
+   will be in charge of part of the computation.
+
    Level: advanced
 
 .seealso: DSSynchronize(), DSGetParallel()
@@ -454,7 +459,7 @@ PetscErrorCode DSGetParallel(DS ds,DSParallelType *pmode)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds   - the direct solver context
 -  comp - a boolean flag
 
@@ -509,7 +514,7 @@ PetscErrorCode DSGetCompact(DS ds,PetscBool *comp)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds  - the direct solver context
 -  ext - a boolean flag
 
@@ -565,7 +570,7 @@ PetscErrorCode DSGetExtraRow(DS ds,PetscBool *ext)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds  - the direct solver context
 -  ref - a boolean flag
 
@@ -621,7 +626,7 @@ PetscErrorCode DSGetRefined(DS ds,PetscBool *ref)
 
    Logically Collective on ds
 
-   Input Parameter:
+   Input Parameters:
 +  ds - the direct solver context
 -  bs - the block size
 
@@ -761,6 +766,9 @@ PetscErrorCode DSSetFromOptions(DS ds)
     ierr = PetscOptionsEnum("-ds_parallel","Operation mode in parallel runs","DSSetParallel",DSParallelTypes,(PetscEnum)ds->pmode,(PetscEnum*)&pmode,&flag);CHKERRQ(ierr);
     if (flag) { ierr = DSSetParallel(ds,pmode);CHKERRQ(ierr); }
 
+    if (ds->ops->setfromoptions) {
+      ierr = (*ds->ops->setfromoptions)(PetscOptionsObject,ds);CHKERRQ(ierr);
+    }
     ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)ds);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -792,7 +800,7 @@ PetscErrorCode DSSetFromOptions(DS ds)
 @*/
 PetscErrorCode DSView(DS ds,PetscViewer viewer)
 {
-  PetscBool         isascii,issvd;
+  PetscBool         isascii;
   PetscViewerFormat format;
   PetscErrorCode    ierr;
   PetscMPIInt       size;
@@ -808,18 +816,13 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
   if (isascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ds,viewer);CHKERRQ(ierr);
-    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRQ(ierr);
+    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRMPI(ierr);
+    if (size>1) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",DSParallelTypes[ds->pmode]);CHKERRQ(ierr);
+    }
     if (format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
-      if (size>1) {
-        ierr = PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",DSParallelTypes[ds->pmode]);CHKERRQ(ierr);
-      }
       ierr = PetscViewerASCIIPrintf(viewer,"  current state: %s\n",DSStateTypes[ds->state]);CHKERRQ(ierr);
-      ierr = PetscObjectTypeCompare((PetscObject)ds,DSSVD,&issvd);CHKERRQ(ierr);
-      if (issvd) {
-        ierr = PetscViewerASCIIPrintf(viewer,"  dimensions: ld=%D, n=%D, m=%D, l=%D, k=%D",ds->ld,ds->n,ds->m,ds->l,ds->k);CHKERRQ(ierr);
-      } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"  dimensions: ld=%D, n=%D, l=%D, k=%D",ds->ld,ds->n,ds->l,ds->k);CHKERRQ(ierr);
-      }
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimensions: ld=%D, n=%D, l=%D, k=%D",ds->ld,ds->n,ds->l,ds->k);CHKERRQ(ierr);
       if (ds->state==DS_STATE_TRUNCATED) {
         ierr = PetscViewerASCIIPrintf(viewer,", t=%D\n",ds->t);CHKERRQ(ierr);
       } else {
@@ -888,6 +891,7 @@ PetscErrorCode DSAllocate(DS ds,PetscInt ld)
   PetscValidType(ds,1);
   if (ld<1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Leading dimension should be at least one");
   if (ld!=ds->ld) {
+    ierr = PetscInfo1(ds,"Allocating memory with leading dimension=%D\n",ld);CHKERRQ(ierr);
     ierr = DSReset(ds);CHKERRQ(ierr);
     ds->ld = ld;
     ierr = (*ds->ops->allocate)(ds,ld);CHKERRQ(ierr);
@@ -923,7 +927,6 @@ PetscErrorCode DSReset(DS ds)
   ds->ld       = 0;
   ds->l        = 0;
   ds->n        = 0;
-  ds->m        = 0;
   ds->k        = 0;
   for (i=0;i<DS_NUM_MAT;i++) {
     ierr = PetscFree(ds->mat[i]);CHKERRQ(ierr);
@@ -934,7 +937,7 @@ PetscErrorCode DSReset(DS ds)
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    DSDestroy - Destroys DS context that was created with DSCreate().
 
    Collective on ds
@@ -996,7 +999,9 @@ SLEPC_EXTERN PetscErrorCode DSCreate_NHEP(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_GHEP(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_GHIEP(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_GNHEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_NHEPTS(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_SVD(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_GSVD(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_PEP(DS);
 SLEPC_EXTERN PetscErrorCode DSCreate_NEP(DS);
 
@@ -1019,7 +1024,9 @@ PetscErrorCode DSRegisterAll(void)
   ierr = DSRegister(DSGHEP,DSCreate_GHEP);CHKERRQ(ierr);
   ierr = DSRegister(DSGHIEP,DSCreate_GHIEP);CHKERRQ(ierr);
   ierr = DSRegister(DSGNHEP,DSCreate_GNHEP);CHKERRQ(ierr);
+  ierr = DSRegister(DSNHEPTS,DSCreate_NHEPTS);CHKERRQ(ierr);
   ierr = DSRegister(DSSVD,DSCreate_SVD);CHKERRQ(ierr);
+  ierr = DSRegister(DSGSVD,DSCreate_GSVD);CHKERRQ(ierr);
   ierr = DSRegister(DSPEP,DSCreate_PEP);CHKERRQ(ierr);
   ierr = DSRegister(DSNEP,DSCreate_NEP);CHKERRQ(ierr);
   PetscFunctionReturn(0);
