@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -44,7 +44,7 @@ PetscErrorCode FNEvaluateFunction_Combine(FN fn,PetscScalar x,PetscScalar *y)
       break;
     case FN_COMBINE_DIVIDE:
       ierr = FNEvaluateFunction(ctx->f2,x,&b);CHKERRQ(ierr);
-      if (b==0.0) SETERRQ(PETSC_COMM_SELF,1,"Function not defined in the requested value");
+      if (b==0.0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Function not defined in the requested value");
       *y = a/b;
       break;
     case FN_COMBINE_COMPOSE:
@@ -79,7 +79,7 @@ PetscErrorCode FNEvaluateDerivative_Combine(FN fn,PetscScalar x,PetscScalar *yp)
       ierr = FNEvaluateDerivative(ctx->f2,x,&bp);CHKERRQ(ierr);
       ierr = FNEvaluateFunction(ctx->f1,x,&a);CHKERRQ(ierr);
       ierr = FNEvaluateFunction(ctx->f2,x,&b);CHKERRQ(ierr);
-      if (b==0.0) SETERRQ(PETSC_COMM_SELF,1,"Derivative not defined in the requested value");
+      if (b==0.0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Derivative not defined in the requested value");
       *yp = (ap*b-a*bp)/(b*b);
       break;
     case FN_COMBINE_COMPOSE:
@@ -94,22 +94,16 @@ PetscErrorCode FNEvaluateDerivative_Combine(FN fn,PetscScalar x,PetscScalar *yp)
 
 PetscErrorCode FNEvaluateFunctionMat_Combine(FN fn,Mat A,Mat B)
 {
-#if defined(PETSC_MISSING_LAPACK_GESV)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GESV - Lapack routines are unavailable");
-#else
-  PetscErrorCode ierr;
-  FN_COMBINE     *ctx = (FN_COMBINE*)fn->data;
-  PetscScalar    *Aa,*Ba,*Wa,*Za,one=1.0,zero=0.0;
-  PetscBLASInt   n,ld,ld2,inc=1,*ipiv,info;
-  PetscInt       m;
-  Mat            W,Z;
+  PetscErrorCode    ierr;
+  FN_COMBINE        *ctx = (FN_COMBINE*)fn->data;
+  PetscScalar       *Ba,*Wa,one=1.0,zero=0.0;
+  const PetscScalar *Za;
+  PetscBLASInt      n,ld,ld2,inc=1,*ipiv,info;
+  PetscInt          m;
+  Mat               W,Z;
 
   PetscFunctionBegin;
   ierr = FN_AllocateWorkMat(fn,A,&W);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(A,&Aa);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(W,&Wa);CHKERRQ(ierr);
   ierr = MatGetSize(A,&m,NULL);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
   ld  = n;
@@ -119,26 +113,38 @@ PetscErrorCode FNEvaluateFunctionMat_Combine(FN fn,Mat A,Mat B)
     case FN_COMBINE_ADD:
       ierr = FNEvaluateFunctionMat_Private(ctx->f1,A,W,PETSC_FALSE);CHKERRQ(ierr);
       ierr = FNEvaluateFunctionMat_Private(ctx->f2,A,B,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
+      ierr = MatDenseGetArray(W,&Wa);CHKERRQ(ierr);
       PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&ld2,&one,Wa,&inc,Ba,&inc));
       ierr = PetscLogFlops(1.0*n*n);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(W,&Wa);CHKERRQ(ierr);
       break;
     case FN_COMBINE_MULTIPLY:
       ierr = FN_AllocateWorkMat(fn,A,&Z);CHKERRQ(ierr);
-      ierr = MatDenseGetArray(Z,&Za);CHKERRQ(ierr);
       ierr = FNEvaluateFunctionMat_Private(ctx->f1,A,W,PETSC_FALSE);CHKERRQ(ierr);
       ierr = FNEvaluateFunctionMat_Private(ctx->f2,A,Z,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
+      ierr = MatDenseGetArray(W,&Wa);CHKERRQ(ierr);
+      ierr = MatDenseGetArrayRead(Z,&Za);CHKERRQ(ierr);
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&one,Wa,&ld,Za,&ld,&zero,Ba,&ld));
       ierr = PetscLogFlops(2.0*n*n*n);CHKERRQ(ierr);
-      ierr = MatDenseRestoreArray(Z,&Za);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(W,&Wa);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArrayRead(Z,&Za);CHKERRQ(ierr);
       ierr = FN_FreeWorkMat(fn,&Z);CHKERRQ(ierr);
       break;
     case FN_COMBINE_DIVIDE:
       ierr = FNEvaluateFunctionMat_Private(ctx->f2,A,W,PETSC_FALSE);CHKERRQ(ierr);
       ierr = FNEvaluateFunctionMat_Private(ctx->f1,A,B,PETSC_FALSE);CHKERRQ(ierr);
       ierr = PetscMalloc1(ld,&ipiv);CHKERRQ(ierr);
+      ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
+      ierr = MatDenseGetArray(W,&Wa);CHKERRQ(ierr);
       PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&n,&n,Wa,&ld,ipiv,Ba,&ld,&info));
       SlepcCheckLapackInfo("gesv",info);
       ierr = PetscLogFlops(2.0*n*n*n/3.0+2.0*n*n*n);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
+      ierr = MatDenseRestoreArray(W,&Wa);CHKERRQ(ierr);
       ierr = PetscFree(ipiv);CHKERRQ(ierr);
       break;
     case FN_COMBINE_COMPOSE:
@@ -147,20 +153,12 @@ PetscErrorCode FNEvaluateFunctionMat_Combine(FN fn,Mat A,Mat B)
       break;
   }
 
-  ierr = MatDenseRestoreArray(A,&Aa);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(W,&Wa);CHKERRQ(ierr);
   ierr = FN_FreeWorkMat(fn,&W);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode FNEvaluateFunctionMatVec_Combine(FN fn,Mat A,Vec v)
 {
-#if defined(PETSC_MISSING_LAPACK_GESV)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GESV - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   FN_COMBINE     *ctx = (FN_COMBINE*)fn->data;
   PetscScalar    *va,*Za;
@@ -216,7 +214,6 @@ PetscErrorCode FNEvaluateFunctionMatVec_Combine(FN fn,Mat A,Vec v)
       break;
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode FNView_Combine(FN fn,PetscViewer viewer)
@@ -272,7 +269,7 @@ static PetscErrorCode FNCombineSetChildren_Combine(FN fn,FNCombineType comb,FN f
    FNCombineSetChildren - Sets the two child functions that constitute this
    combined function, and the way they must be combined.
 
-   Logically Collective on FN
+   Logically Collective on fn
 
    Input Parameters:
 +  fn   - the math function context
@@ -375,7 +372,7 @@ PetscErrorCode FNDestroy_Combine(FN fn)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode FNCreate_Combine(FN fn)
+SLEPC_EXTERN PetscErrorCode FNCreate_Combine(FN fn)
 {
   PetscErrorCode ierr;
   FN_COMBINE     *ctx;
