@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -185,7 +185,7 @@ PetscErrorCode RGView_Ring(RG rg,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = SlepcSNPrintfScalar(str,50,ctx->center,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = SlepcSNPrintfScalar(str,sizeof(str),ctx->center,PETSC_FALSE);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  center: %s, radius: %g, vscale: %g, start angle: %g, end angle: %g, ring width: %g\n",str,RGShowReal(ctx->radius),RGShowReal(ctx->vscale),(double)ctx->start_ang,(double)ctx->end_ang,(double)ctx->width);CHKERRQ(ierr);
   } else if (isdraw) {
     ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
@@ -258,16 +258,16 @@ PetscErrorCode RGComputeContour_Ring(RG rg,PetscInt n,PetscScalar *cr,PetscScala
 #if defined(PETSC_USE_COMPLEX)
       cr[i] = ctx->center + (ctx->radius+ctx->width/2.0)*PetscCMPLX(PetscCosReal(theta),ctx->vscale*PetscSinReal(theta));
 #else
-      cr[i] = ctx->center + (ctx->radius+ctx->width/2.0)*PetscCosReal(theta);
-      ci[i] = (ctx->radius+ctx->width/2.0)*ctx->vscale*PetscSinReal(theta);
+      if (cr) cr[i] = ctx->center + (ctx->radius+ctx->width/2.0)*PetscCosReal(theta);
+      if (ci) ci[i] = (ctx->radius+ctx->width/2.0)*ctx->vscale*PetscSinReal(theta);
 #endif
     } else {
       theta = ((ctx->end_ang-start_ang)*(n-i)/n2 + start_ang)*2.0*PETSC_PI;
 #if defined(PETSC_USE_COMPLEX)
       cr[i] = ctx->center + (ctx->radius-ctx->width/2.0)*PetscCMPLX(PetscCosReal(theta),ctx->vscale*PetscSinReal(theta));
 #else
-      cr[i] = ctx->center + (ctx->radius-ctx->width/2.0)*PetscCosReal(theta);
-      ci[i] = (ctx->radius-ctx->width/2.0)*ctx->vscale*PetscSinReal(theta);
+      if (cr) cr[i] = ctx->center + (ctx->radius-ctx->width/2.0)*PetscCosReal(theta);
+      if (ci) ci[i] = (ctx->radius-ctx->width/2.0)*ctx->vscale*PetscSinReal(theta);
 #endif
     }
   }
@@ -287,6 +287,43 @@ PetscErrorCode RGComputeBoundingBox_Ring(RG rg,PetscReal *a,PetscReal *b,PetscRe
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode RGComputeQuadrature_Ring(RG rg,RGQuadRule quad,PetscInt n,PetscScalar *z,PetscScalar *zn,PetscScalar *w)
+{
+  RG_RING     *ctx = (RG_RING*)rg->data;
+  PetscReal   max_w=0.0;
+  PetscScalar tmp,tmp2;
+  PetscInt    i,j;
+
+  PetscFunctionBegin;
+  if (quad == RG_QUADRULE_CHEBYSHEV) {
+#if defined(PETSC_USE_COMPLEX)
+    PetscReal theta;
+    for (i=0;i<n;i++) {
+      theta = PETSC_PI*(i+0.5)/n;
+      zn[i] = PetscCosReal(theta);
+      w[i]  = PetscCosReal((n-1)*theta)/n;
+      theta = (ctx->start_ang*2.0+(ctx->end_ang-ctx->start_ang)*(PetscRealPart(zn[i])+1.0))*PETSC_PI;
+      z[i] = rg->sfactor*(ctx->center + ctx->radius*PetscCMPLX(PetscCosReal(theta),ctx->vscale*PetscSinReal(theta)));
+    }
+#else
+    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Integration points on a vertical line require complex arithmetic");
+#endif
+  } else {  /* RG_QUADRULE_TRAPEZOIDAL */
+    for (i=0;i<n;i++) {
+      zn[i] = (z[i]-rg->sfactor*ctx->center)/(rg->sfactor*ctx->radius);
+      tmp = 1.0; tmp2 = 1.0;
+      for (j=0;j<n;j++) {
+        tmp *= z[j];
+        if (i != j) tmp2 *= z[j]-z[i];
+      }
+      w[i] = tmp/tmp2;
+      max_w = PetscMax(PetscAbsScalar(w[i]),max_w);
+    }
+    for (i=0;i<n;i++) w[i] /= (PetscScalar)max_w;
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode RGCheckInside_Ring(RG rg,PetscReal px,PetscReal py,PetscInt *inside)
 {
   RG_RING   *ctx = (RG_RING*)rg->data;
@@ -339,6 +376,16 @@ PetscErrorCode RGCheckInside_Ring(RG rg,PetscReal px,PetscReal py,PetscInt *insi
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode RGIsAxisymmetric_Ring(RG rg,PetscBool vertical,PetscBool *symm)
+{
+  RG_RING *ctx = (RG_RING*)rg->data;
+
+  PetscFunctionBegin;
+  if (vertical) *symm = (PetscRealPart(ctx->center) == 0.0 && PetscAbs(ctx->start_ang+ctx->end_ang-PetscRealConstant(1.0)) == 0.5)? PETSC_TRUE: PETSC_FALSE;
+  else *symm = (PetscImaginaryPart(ctx->center) == 0.0 && ctx->start_ang+ctx->end_ang == 1.0)? PETSC_TRUE: PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode RGSetFromOptions_Ring(PetscOptionItems *PetscOptionsObject,RG rg)
 {
   PetscErrorCode ierr;
@@ -388,13 +435,15 @@ SLEPC_EXTERN PetscErrorCode RGCreate_Ring(RG rg)
   ring->width     = 0.1;
   rg->data = (void*)ring;
 
-  rg->ops->istrivial      = RGIsTrivial_Ring;
-  rg->ops->computecontour = RGComputeContour_Ring;
-  rg->ops->computebbox    = RGComputeBoundingBox_Ring;
-  rg->ops->checkinside    = RGCheckInside_Ring;
-  rg->ops->setfromoptions = RGSetFromOptions_Ring;
-  rg->ops->view           = RGView_Ring;
-  rg->ops->destroy        = RGDestroy_Ring;
+  rg->ops->istrivial         = RGIsTrivial_Ring;
+  rg->ops->computecontour    = RGComputeContour_Ring;
+  rg->ops->computebbox       = RGComputeBoundingBox_Ring;
+  rg->ops->computequadrature = RGComputeQuadrature_Ring;
+  rg->ops->checkinside       = RGCheckInside_Ring;
+  rg->ops->isaxisymmetric    = RGIsAxisymmetric_Ring;
+  rg->ops->setfromoptions    = RGSetFromOptions_Ring;
+  rg->ops->view              = RGView_Ring;
+  rg->ops->destroy           = RGDestroy_Ring;
   ierr = PetscObjectComposeFunction((PetscObject)rg,"RGRingSetParameters_C",RGRingSetParameters_Ring);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)rg,"RGRingGetParameters_C",RGRingGetParameters_Ring);CHKERRQ(ierr);
   PetscFunctionReturn(0);
