@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,34 +12,40 @@
 */
 
 #include <slepc/private/stimpl.h>         /*I "slepcst.h" I*/
-#include "./filter.h"
+#include "filter.h"
 
-PetscErrorCode STApply_Filter(ST st,Vec x,Vec y)
+/*
+   Operator (filter):
+               Op               P         M
+   if nmat=1:  p(A)             NULL      p(A)
+*/
+PetscErrorCode STComputeOperator_Filter(ST st)
 {
   PetscErrorCode ierr;
+  ST_FILTER      *ctx = (ST_FILTER*)st->data;
 
   PetscFunctionBegin;
-  ierr = STFilter_FILTLAN_Apply(st,x,y);CHKERRQ(ierr);
+  if (st->nmat>1) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_SUP,"Only implemented for standard eigenvalue problem");
+  if (ctx->intb >= PETSC_MAX_REAL && ctx->inta <= PETSC_MIN_REAL) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_ORDER,"Must pass an interval with STFilterSetInterval()");
+  if (ctx->right == 0.0 && ctx->left == 0.0) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_ORDER,"Must pass an approximate numerical range with STFilterSetRange()");
+  if (ctx->left > ctx->inta || ctx->right < ctx->intb) SETERRQ4(PetscObjectComm((PetscObject)st),PETSC_ERR_USER_INPUT,"The requested interval [%g,%g] must be contained in the numerical range [%g,%g]",(double)ctx->inta,(double)ctx->intb,(double)ctx->left,(double)ctx->right);
+  if (!ctx->polyDegree) ctx->polyDegree = 100;
+  ctx->frame[0] = ctx->left;
+  ctx->frame[1] = ctx->inta;
+  ctx->frame[2] = ctx->intb;
+  ctx->frame[3] = ctx->right;
+  ierr = STFilter_FILTLAN_setFilter(st,&st->T[0]);CHKERRQ(ierr);
+  st->M = st->T[0];
+  ierr = MatDestroy(&st->P);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode STSetUp_Filter(ST st)
 {
   PetscErrorCode ierr;
-  ST_FILTER      *ctx = (ST_FILTER*)st->data;
 
   PetscFunctionBegin;
   ierr = STSetWorkVecs(st,4);CHKERRQ(ierr);
-  if (st->nmat>1) SETERRQ(PetscObjectComm((PetscObject)st),1,"Only implemented for standard eigenvalue problem");
-  if (ctx->intb >= PETSC_MAX_REAL && ctx->inta <= PETSC_MIN_REAL) SETERRQ(PetscObjectComm((PetscObject)st),1,"Must pass an interval with STFilterSetInterval()");
-  if (ctx->right == 0.0 && ctx->left == 0.0) SETERRQ(PetscObjectComm((PetscObject)st),1,"Must pass an approximate numerical range with STFilterSetRange()");
-  if (ctx->left > ctx->inta || ctx->right < ctx->intb) SETERRQ4(PetscObjectComm((PetscObject)st),1,"The requested interval [%g,%g] must be contained in the numerical range [%g,%g]",(double)ctx->inta,(double)ctx->intb,(double)ctx->left,(double)ctx->right);
-  if (!ctx->polyDegree) ctx->polyDegree = 100;
-  ctx->frame[0] = ctx->left;
-  ctx->frame[1] = ctx->inta;
-  ctx->frame[2] = ctx->intb;
-  ctx->frame[3] = ctx->right;
-  ierr = STFilter_FILTLAN_setFilter(st);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -79,9 +85,11 @@ static PetscErrorCode STFilterSetInterval_Filter(ST st,PetscReal inta,PetscReal
   PetscFunctionBegin;
   if (inta>intb) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
   if (ctx->inta != inta || ctx->intb != intb) {
-    ctx->inta = inta;
-    ctx->intb = intb;
-    st->state = ST_STATE_INITIAL;
+    ctx->inta   = inta;
+    ctx->intb   = intb;
+    st->state   = ST_STATE_INITIAL;
+    st->opready = PETSC_FALSE;
+    ctx->filtch = PETSC_TRUE;
   }
   PetscFunctionReturn(0);
 }
@@ -89,7 +97,7 @@ static PetscErrorCode STFilterSetInterval_Filter(ST st,PetscReal inta,PetscReal
 /*@
    STFilterSetInterval - Defines the interval containing the desired eigenvalues.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st   - the spectral transformation context
@@ -99,8 +107,6 @@ static PetscErrorCode STFilterSetInterval_Filter(ST st,PetscReal inta,PetscReal
    Options Database Key:
 .  -st_filter_interval <a,b> - set [a,b] as the interval of interest
 
-   Level: intermediate
-
    Notes:
    The filter will be configured to emphasize eigenvalues contained in the given
    interval, and damp out eigenvalues outside it. If the interval is open, then
@@ -111,6 +117,8 @@ static PetscErrorCode STFilterSetInterval_Filter(ST st,PetscReal inta,PetscReal
    The interval must be contained within the numerical range of the matrix, see
    STFilterSetRange().
 
+   Level: intermediate
+
 .seealso: STFilterGetInterval(), STFilterSetRange(), EPSSetInterval()
 @*/
 PetscErrorCode STFilterSetInterval(ST st,PetscReal inta,PetscReal intb)
@@ -143,7 +151,7 @@ static PetscErrorCode STFilterGetInterval_Filter(ST st,PetscReal *inta,PetscReal
    Input Parameter:
 .  st  - the spectral transformation context
 
-   Output Parameter:
+   Output Parameters:
 +  inta - left end of the interval
 -  intb - right end of the interval
 
@@ -168,9 +176,11 @@ static PetscErrorCode STFilterSetRange_Filter(ST st,PetscReal left,PetscReal rig
   PetscFunctionBegin;
   if (left>right) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be left<right");
   if (ctx->left != left || ctx->right != right) {
-    ctx->left  = left;
-    ctx->right = right;
-    st->state  = ST_STATE_INITIAL;
+    ctx->left   = left;
+    ctx->right  = right;
+    st->state   = ST_STATE_INITIAL;
+    st->opready = PETSC_FALSE;
+    ctx->filtch = PETSC_TRUE;
   }
   PetscFunctionReturn(0);
 }
@@ -179,7 +189,7 @@ static PetscErrorCode STFilterSetRange_Filter(ST st,PetscReal left,PetscReal rig
    STFilterSetRange - Defines the numerical range (or field of values) of the matrix, that is,
    the interval containing all eigenvalues.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st    - the spectral transformation context
@@ -189,12 +199,12 @@ static PetscErrorCode STFilterSetRange_Filter(ST st,PetscReal left,PetscReal rig
    Options Database Key:
 .  -st_filter_range <a,b> - set [a,b] as the numerical range
 
-   Level: intermediate
-
    Notes:
    The filter will be most effective if the numerical range is tight, that is, left and right
    are good approximations to the leftmost and rightmost eigenvalues, respectively.
 
+   Level: intermediate
+
 .seealso: STFilterGetRange(), STFilterSetInterval()
 @*/
 PetscErrorCode STFilterSetRange(ST st,PetscReal left,PetscReal right)
@@ -227,7 +237,7 @@ static PetscErrorCode STFilterGetRange_Filter(ST st,PetscReal *left,PetscReal *r
    Input Parameter:
 .  st  - the spectral transformation context
 
-   Output Parameter:
+   Output Parameters:
 +  left  - left end of the interval
 -  right - right end of the interval
 
@@ -252,12 +262,16 @@ static PetscErrorCode STFilterSetDegree_Filter(ST st,PetscInt deg)
   PetscFunctionBegin;
   if (deg == PETSC_DEFAULT || deg == PETSC_DECIDE) {
     ctx->polyDegree = 0;
-    st->state = ST_STATE_INITIAL;
+    st->state       = ST_STATE_INITIAL;
+    st->opready     = PETSC_FALSE;
+    ctx->filtch     = PETSC_TRUE;
   } else {
     if (deg<=0) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of degree. Must be > 0");
     if (ctx->polyDegree != deg) {
       ctx->polyDegree = deg;
-      st->state = ST_STATE_INITIAL;
+      st->state       = ST_STATE_INITIAL;
+      st->opready     = PETSC_FALSE;
+      ctx->filtch     = PETSC_TRUE;
     }
   }
   PetscFunctionReturn(0);
@@ -266,7 +280,7 @@ static PetscErrorCode STFilterSetDegree_Filter(ST st,PetscInt deg)
 /*@
    STFilterSetDegree - Sets the degree of the filter polynomial.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st  - the spectral transformation context
@@ -320,7 +334,7 @@ PetscErrorCode STFilterGetDegree(ST st,PetscInt *deg)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  PetscValidPointer(deg,2);
+  PetscValidIntPointer(deg,2);
   ierr = PetscUseMethod(st,"STFilterGetDegree_C",(ST,PetscInt*),(st,deg));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -354,18 +368,20 @@ PetscErrorCode STFilterGetThreshold(ST st,PetscReal *gamma)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  PetscValidPointer(gamma,2);
+  PetscValidRealPointer(gamma,2);
   ierr = PetscUseMethod(st,"STFilterGetThreshold_C",(ST,PetscReal*),(st,gamma));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode STReset_Filter(ST st)
 {
-  ST_FILTER *ctx = (ST_FILTER*)st->data;
+  PetscErrorCode ierr;
+  ST_FILTER      *ctx = (ST_FILTER*)st->data;
 
   PetscFunctionBegin;
   ctx->left  = 0.0;
   ctx->right = 0.0;
+  ierr = MatDestroy(&ctx->T);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -408,7 +424,7 @@ PetscErrorCode STDestroy_Filter(ST st)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode STCreate_Filter(ST st)
+SLEPC_EXTERN PetscErrorCode STCreate_Filter(ST st)
 {
   PetscErrorCode ierr;
   ST_FILTER      *ctx;
@@ -418,6 +434,8 @@ PETSC_EXTERN PetscErrorCode STCreate_Filter(ST st)
   ierr = PetscNewLog(st,&ctx);CHKERRQ(ierr);
   st->data = (void*)ctx;
 
+  st->usesksp = PETSC_FALSE;
+
   ctx->inta               = PETSC_MIN_REAL;
   ctx->intb               = PETSC_MAX_REAL;
   ctx->left               = 0.0;
@@ -448,12 +466,13 @@ PETSC_EXTERN PetscErrorCode STCreate_Filter(ST st)
   ierr = PetscNewLog(st,&pfi);CHKERRQ(ierr);
   ctx->filterInfo         = pfi;
 
-  st->ops->apply          = STApply_Filter;
-  st->ops->setfromoptions = STSetFromOptions_Filter;
-  st->ops->setup          = STSetUp_Filter;
-  st->ops->destroy        = STDestroy_Filter;
-  st->ops->reset          = STReset_Filter;
-  st->ops->view           = STView_Filter;
+  st->ops->apply           = STApply_Generic;
+  st->ops->setup           = STSetUp_Filter;
+  st->ops->computeoperator = STComputeOperator_Filter;
+  st->ops->setfromoptions  = STSetFromOptions_Filter;
+  st->ops->destroy         = STDestroy_Filter;
+  st->ops->reset           = STReset_Filter;
+  st->ops->view            = STView_Filter;
 
   ierr = PetscObjectComposeFunction((PetscObject)st,"STFilterSetInterval_C",STFilterSetInterval_Filter);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)st,"STFilterGetInterval_C",STFilterGetInterval_Filter);CHKERRQ(ierr);
