@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -219,13 +219,13 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
   const PetscInt  numLookMore = 2*(PetscInt)(0.5+(PetscLogReal(2.0)/PetscLogReal(opts->shiftStepExpanRate)));
 
   PetscFunctionBegin;
-  if (a>a1 || a1>b1 || b1>b) SETERRQ(PETSC_COMM_SELF,1,"Values in the frame vector should be non-decreasing");
-  if (a1 == b1) SETERRQ(PETSC_COMM_SELF,1,"The range of wanted eigenvalues cannot be of size zero");
+  if (a>a1 || a1>b1 || b1>b) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Values in the frame vector should be non-decreasing");
+  if (a1 == b1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"The range of wanted eigenvalues cannot be of size zero");
   filterInfo->filterType = 2;      /* mid-pass filter, for interior eigenvalues */
   if (b == b1) {
-    if (a == a1) SETERRQ(PETSC_COMM_SELF,1,"A polynomial filter should not cover all eigenvalues");
+    if (a == a1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"A polynomial filter should not cover all eigenvalues");
     filterInfo->filterType = 1;    /* high-pass filter, for largest eigenvalues */
-  } else if (a == a1) SETERRQ(PETSC_COMM_SELF,1,"filterType==3 for smallest eigenvalues should be pre-converted to filterType==1 for largest eigenvalues");
+  } else if (a == a1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"filterType==3 for smallest eigenvalues should be pre-converted to filterType==1 for largest eigenvalues");
 
   /* the following recipe follows Yousef Saad (2005, 2006) with a few minor adaptations / enhancements */
   halfPlateau = 0.5*(b1-a1)*opts->initialPlateau;    /* half length of the "plateau" of the (dual) base filter */
@@ -448,7 +448,7 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
       else halfPlateau /= PetscSqrtReal(opts->plateauShrinkRate);
     }
   }
-  if (!filterInfo->filterOK) SETERRQ(PETSC_COMM_SELF,1,"STFILTER cannot get the filter specified; please adjust your filter parameters (e.g. increasing the polynomial degree)");
+  if (!filterInfo->filterOK) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_CONV_FAILED,"STFILTER cannot get the filter specified; please adjust your filter parameters (e.g. increasing the polynomial degree)");
 
   filterInfo->totalNumIter = numIter;
   ierr = PetscFree2(polyFilter,baseFilter);CHKERRQ(ierr);
@@ -969,7 +969,7 @@ PetscErrorCode MatMult_FILTLAN(Mat A,Vec x,Vec y)
   PetscInt       npoints;
 
   PetscFunctionBegin;
-  ierr = MatShellGetContext(A,(void**)&st);CHKERRQ(ierr);
+  ierr = MatShellGetContext(A,&st);CHKERRQ(ierr);
   ctx = (ST_FILTER*)st->data;
   npoints = (ctx->filterInfo->filterType == 2)? 6: 4;
   ierr = FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorProduct(ctx->T,x,y,ctx->baseFilter,2*ctx->baseDegree+2,ctx->intervals,npoints-1,ctx->opts->intervalWeights,ctx->polyDegree,st->work);CHKERRQ(ierr);
@@ -1002,16 +1002,12 @@ PetscErrorCode STFilter_FILTLAN_setFilter(ST st,Mat *G)
     alpha = ctx->frame[3];
     ierr = MatShift(ctx->T,alpha);CHKERRQ(ierr);
     for (i=0;i<4;i++) frame2[i] = ctx->frame[3] - ctx->frame[3-i];
-    ierr = FILTLAN_GetIntervals(ctx->intervals,frame2,ctx->polyDegree,ctx->baseDegree,ctx->opts,ctx->filterInfo);CHKERRQ(ierr);
-    /* translate the intervals back */
-    for (i=0;i<4;i++) ctx->intervals2[i] = ctx->frame[3] - ctx->intervals[3-i];
   } else {  /* it can be a mid-pass filter or a high-pass filter */
     if (ctx->frame[0] == 0.0) {
       ierr = PetscObjectReference((PetscObject)st->A[0]);CHKERRQ(ierr);
       ierr = MatDestroy(&ctx->T);CHKERRQ(ierr);
       ctx->T = st->A[0];
-      ierr = FILTLAN_GetIntervals(ctx->intervals,ctx->frame,ctx->polyDegree,ctx->baseDegree,ctx->opts,ctx->filterInfo);CHKERRQ(ierr);
-      for (i=0;i<6;i++) ctx->intervals2[i] = ctx->intervals[i];
+      for (i=0;i<4;i++) frame2[i] = ctx->frame[i];
     } else {
       /* T = A - frame[0]*eye(n) */
       ierr = MatDestroy(&ctx->T);CHKERRQ(ierr);
@@ -1019,23 +1015,39 @@ PetscErrorCode STFilter_FILTLAN_setFilter(ST st,Mat *G)
       alpha = -ctx->frame[0];
       ierr = MatShift(ctx->T,alpha);CHKERRQ(ierr);
       for (i=0;i<4;i++) frame2[i] = ctx->frame[i] - ctx->frame[0];
-      ierr = FILTLAN_GetIntervals(ctx->intervals,frame2,ctx->polyDegree,ctx->baseDegree,ctx->opts,ctx->filterInfo);CHKERRQ(ierr);
-      /* translate the intervals back */
-      for (i=0;i<6;i++) ctx->intervals2[i] = ctx->intervals[i] + ctx->frame[0];
     }
   }
-  npoints = (ctx->filterInfo->filterType == 2)? 6: 4;
-  ierr = PetscFree(ctx->baseFilter);CHKERRQ(ierr);
-  ierr = PetscMalloc1((2*ctx->baseDegree+2)*(npoints-1),&ctx->baseFilter);CHKERRQ(ierr);
-  ierr = FILTLAN_HermiteBaseFilterInChebyshevBasis(ctx->baseFilter,ctx->intervals,npoints,HighLowFlags,ctx->baseDegree);CHKERRQ(ierr);
-  ierr = PetscInfo1(st,"Computed value of yLimit = %g\n",ctx->filterInfo->yLimit);CHKERRQ(ierr);
+
+  /* no need to recompute filter if the parameters did not change */
+  if (st->state==ST_STATE_INITIAL || ctx->filtch) {
+    ierr = FILTLAN_GetIntervals(ctx->intervals,frame2,ctx->polyDegree,ctx->baseDegree,ctx->opts,ctx->filterInfo);CHKERRQ(ierr);
+    /* translate the intervals back */
+    if (ctx->frame[0] == ctx->frame[1]) {  /* low pass filter, convert it to high pass filter */
+      for (i=0;i<4;i++) ctx->intervals2[i] = ctx->frame[3] - ctx->intervals[3-i];
+    } else {  /* it can be a mid-pass filter or a high-pass filter */
+      if (ctx->frame[0] == 0.0) {
+        for (i=0;i<6;i++) ctx->intervals2[i] = ctx->intervals[i];
+      } else {
+        for (i=0;i<6;i++) ctx->intervals2[i] = ctx->intervals[i] + ctx->frame[0];
+      }
+    }
+
+    npoints = (ctx->filterInfo->filterType == 2)? 6: 4;
+    ierr = PetscFree(ctx->baseFilter);CHKERRQ(ierr);
+    ierr = PetscMalloc1((2*ctx->baseDegree+2)*(npoints-1),&ctx->baseFilter);CHKERRQ(ierr);
+    ierr = FILTLAN_HermiteBaseFilterInChebyshevBasis(ctx->baseFilter,ctx->intervals,npoints,HighLowFlags,ctx->baseDegree);CHKERRQ(ierr);
+    ierr = PetscInfo1(st,"Computed value of yLimit = %g\n",ctx->filterInfo->yLimit);CHKERRQ(ierr);
+  }
+  ctx->filtch = PETSC_FALSE;
 
   /* create shell matrix*/
-  ierr = MatGetSize(ctx->T,&N,&M);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->T,&n,&m);CHKERRQ(ierr);
-  ierr = MatCreateShell(PetscObjectComm((PetscObject)st),n,m,N,M,st,G);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(*G,MATOP_MULT,(void(*)(void))MatMult_FILTLAN);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)st,(PetscObject)*G);CHKERRQ(ierr);
+  if (!*G) {
+    ierr = MatGetSize(ctx->T,&N,&M);CHKERRQ(ierr);
+    ierr = MatGetLocalSize(ctx->T,&n,&m);CHKERRQ(ierr);
+    ierr = MatCreateShell(PetscObjectComm((PetscObject)st),n,m,N,M,st,G);CHKERRQ(ierr);
+    ierr = MatShellSetOperation(*G,MATOP_MULT,(void(*)(void))MatMult_FILTLAN);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)st,(PetscObject)*G);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
