@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -63,7 +63,7 @@ PetscErrorCode STInitializePackage(void)
   ierr = STRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("STSetUp",ST_CLASSID,&ST_SetUp);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("STComputeOperator",ST_CLASSID,&ST_ComputeOperator);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("STComputeOperatr",ST_CLASSID,&ST_ComputeOperator);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STApply",ST_CLASSID,&ST_Apply);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STApplyTranspose",ST_CLASSID,&ST_ApplyTranspose);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STMatSetUp",ST_CLASSID,&ST_MatSetUp);CHKERRQ(ierr);
@@ -110,9 +110,11 @@ PetscErrorCode STReset(ST st)
   if (st->ksp) { ierr = KSPReset(st->ksp);CHKERRQ(ierr); }
   ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->T);CHKERRQ(ierr);
   ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->A);CHKERRQ(ierr);
+  st->nmat = 0;
   ierr = PetscFree(st->Astate);CHKERRQ(ierr);
   ierr = MatDestroy(&st->Op);CHKERRQ(ierr);
   ierr = MatDestroy(&st->P);CHKERRQ(ierr);
+  ierr = MatDestroy(&st->Pmat);CHKERRQ(ierr);
   ierr = VecDestroyVecs(st->nwork,&st->work);CHKERRQ(ierr);
   st->nwork = 0;
   ierr = VecDestroy(&st->wb);CHKERRQ(ierr);
@@ -123,7 +125,7 @@ PetscErrorCode STReset(ST st)
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    STDestroy - Destroys ST context that was created with STCreate().
 
    Collective on st
@@ -181,9 +183,10 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
   st->sigma        = 0.0;
   st->defsigma     = 0.0;
   st->matmode      = ST_MATMODE_COPY;
-  st->str          = DIFFERENT_NONZERO_PATTERN;
+  st->str          = UNKNOWN_NONZERO_PATTERN;
   st->transform    = PETSC_FALSE;
   st->D            = NULL;
+  st->Pmat         = NULL;
 
   st->ksp          = NULL;
   st->usesksp      = PETSC_FALSE;
@@ -316,7 +319,7 @@ PetscErrorCode STSetMatrices(ST st,PetscInt n,Mat A[])
 
    Not collective, though parallel Mats are returned if the ST is parallel
 
-   Input Parameter:
+   Input Parameters:
 +  st - the spectral transformation context
 -  k  - the index of the requested matrix (starting in 0)
 
@@ -345,7 +348,7 @@ PetscErrorCode STGetMatrix(ST st,PetscInt k,Mat *A)
 
    Not collective, though parallel Mats are returned if the ST is parallel
 
-   Input Parameter:
+   Input Parameters:
 +  st - the spectral transformation context
 -  k  - the index of the requested matrix (starting in 0)
 
@@ -421,6 +424,77 @@ PetscErrorCode STResetMatrixState(ST st)
   PetscFunctionReturn(0);
 }
 
+/*@
+   STSetPreconditionerMat - Sets the matrix to be used to build the preconditioner.
+
+   Collective on st
+
+   Input Parameters:
++  st  - the spectral transformation context
+-  mat - the matrix that will be used in constructing the preconditioner
+
+   Notes:
+   This matrix will be passed to the internal KSP object (via the last argument
+   of KSPSetOperators()) as the matrix to be used when constructing the preconditioner.
+   If no matrix is set or mat is set to NULL, A-sigma*B will be used
+   to build the preconditioner, being sigma the value set by STSetShift().
+
+   More precisely, this is relevant for spectral transformations that represent
+   a rational matrix function, and use a KSP object for the denominator, called
+   K in the description of STGetOperator(). It includes also the STPRECOND case.
+   If the user has a good approximation to matrix K that can be used to build a
+   cheap preconditioner, it can be passed with this function. Note that it affects
+   only the Pmat argument of KSPSetOperators(), not the Amat argument.
+
+   If a preconditioner matrix is set, the default is to use an iterative KSP
+   rather than a direct method.
+
+   Level: advanced
+
+.seealso: STGetPreconditionerMat(), STSetShift(), STGetOperator()
+@*/
+PetscErrorCode STSetPreconditionerMat(ST st,Mat mat)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,2);
+  PetscCheckSameComm(st,1,mat,2);
+  STCheckNotSeized(st,1);
+  ierr = PetscObjectReference((PetscObject)mat);CHKERRQ(ierr);
+  ierr = MatDestroy(&st->Pmat);CHKERRQ(ierr);
+  st->Pmat     = mat;
+  st->state    = ST_STATE_INITIAL;
+  st->opready  = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   STGetPreconditionerMat - Returns the matrix previously set by STSetPreconditionerMat().
+
+   Collective on st
+
+   Input Parameter:
+.  st - the spectral transformation context
+
+   Output Parameter:
+.  mat - the matrix that will be used in constructing the preconditioner or
+   NULL if no matrix was set by STSetPreconditionerMat().
+
+   Level: advanced
+
+.seealso: STSetPreconditionerMat()
+@*/
+PetscErrorCode STGetPreconditionerMat(ST st,Mat *mat)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidPointer(mat,2);
+  *mat = st->Pmat;
+  PetscFunctionReturn(0);
+}
+
 /*@
    STSetShift - Sets the shift associated with the spectral transformation.
 
@@ -826,7 +900,6 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   STType         cstr;
-  const char*    pat=NULL;
   char           str[50];
   PetscBool      isascii,isstring;
 
@@ -847,7 +920,7 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
       ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
-    ierr = SlepcSNPrintfScalar(str,50,st->sigma,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = SlepcSNPrintfScalar(str,sizeof(str),st->sigma,PETSC_FALSE);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  shift: %s\n",str);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of matrices: %D\n",st->nmat);CHKERRQ(ierr);
     switch (st->matmode) {
@@ -861,12 +934,7 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
       break;
     }
     if (st->nmat>1 && st->matmode != ST_MATMODE_SHELL) {
-      switch (st->str) {
-        case SAME_NONZERO_PATTERN:      pat = "same nonzero pattern";break;
-        case DIFFERENT_NONZERO_PATTERN: pat = "different nonzero pattern";break;
-        case SUBSET_NONZERO_PATTERN:    pat = "subset nonzero pattern";break;
-      }
-      ierr = PetscViewerASCIIPrintf(viewer,"  all matrices have %s\n",pat);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  all matrices have %s\n",MatStructures[st->str]);CHKERRQ(ierr);
     }
     if (st->transform && st->nmat>2) {
       ierr = PetscViewerASCIIPrintf(viewer,"  computing transformed matrices\n");CHKERRQ(ierr);
